#ifndef IBN_PHYSICS_H
#define IBN_PHYSICS_H
#include <cstdlib>
#include <cctype>
#include <iostream>
#include <valarray>

//#include "data/Data.h"

#include "PhysConst.h"
namespace math
{
	const double PI=4*atan(1.0);
	inline double sq(double x) { return x*x ;}

	template<class T> std::valarray<T> sq(const std::valarray<T> & v)
  {
            std::valarray<T> t;
		t = v * v;
		return t;
	}

	class Random
  { 
		unsigned long seed; 
		protected:

		double Rnd(double a=0, double b=1)
    {
			double rnd =  double(rand())/RAND_MAX;
			return a + (b-a)*rnd;

		}

		public:
		Random(int s)  { seed = s; srand(seed); }
		double operator()(double a=0, double b=1)  {return Rnd(a,b); }
	};

	class RandomJump : public  Random
  {
		const double X,D,A,K1,K2;
		double L;
		public:
		RandomJump(int s, double x0,double d,double a,double k1=0,double k2=0): Random(s),
		X(x0),D(d),A(a),K1(k1),K2(k2)
    {
			L=0;
		}
		void Set(double l) { L = l;}

		double operator()(double t, double err)
    {
			double rnd = sqrt(3.)*err * Random::Rnd(-1,+1);
			if(t<=X) return A + (t-X)*K1 + rnd;
			if(t>=X+L) return A + D + (t-X)*K2 + rnd;
			if(L!=0)  return A + D/L*(t-X) + rnd;
			return 0;
		}
	};
	
	class ExpJump: public Random
  {
		const double X,D,A,K1,K2;
		double T;
		
		double expmod(double x)
    {
			if(x==0) return 1;
			return (1.0-exp(-x))/x;
		}
    
		double expjump(double t, double D, double a, double k1, double k2)
    {
			if(t <=0 ) return a*exp(k1*t)*expmod(k1*T);
			if(t >=T ) return (a+D)*exp(k2*t)*expmod(k2*T);
			return a*(1.0-t/T)*expmod((T-t)*k1) + (a+D)*t/T*expmod(-t*k2);
		}

		public:	
		ExpJump(int s, double x0, double d, double a, double k1=0, double k2=0): Random(s), X(x0),D(d),A(a),K1(k1),K2(k2)
    {
			T=1;
		}

		void Set(double l) { T = l;}
    
		double operator()(double t, double err)
    {
			double rnd = sqrt(3.)*err * Random::Rnd(-1,+1);
			return expjump(t-X,D,A,K1,K2) + rnd;
		}
	};
}

class eat_nodigit
{
	public:
		eat_nodigit(void) {}
		~eat_nodigit(void) {}
};

inline std::istream & operator>> (std::istream & is, const eat_nodigit & eat)
{
	char ch;
	do is.get(ch);
	while(!isdigit(ch));
	is.putback(ch);
	return is;
}

#endif //IBN_PHYSICS_H
