/*
 * =====================================================================================
 *
 *       Filename:  PolarimeterConfig.h
 *
 *    Description:  Main config file for polarimeter
 *
 *        Version:  1.0
 *        Created:  07/19/2010 03:38:48 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_POLARIMETER_CONFIG_H
#define IBN_POLARIMETER_CONFIG_H
#include <string>
#include <iostream>

#include <boost/filesystem.hpp>

#include "Log.h"

struct PolarimeterConfig
{
  std::string name; //config name
  boost::filesystem::path config_file=".polarimeterrc";
  boost::filesystem::path work_dir;
  boost::filesystem::path save_dir;
  boost::filesystem::path print_dir;
  boost::filesystem::path camac_config_file;
  boost::filesystem::path measurement_config_file;
  boost::filesystem::path last_good_measurement_config_file; //last good measurement 
  boost::filesystem::path last_bad_measurement_config_file; //last bad measurement
  boost::filesystem::path tmp_save_file;
  boost::filesystem::path last_tmp_save_file;
  boost::filesystem::path log_file;
	boost::filesystem::path motor_file="/home/vepp4/kadrs/motor-exp-horizontal.dat";
	boost::filesystem::path high_resolution_depfreq_file="/home/vepp4/kadrs/polarimeter/depfreq.dat";
  unsigned bunch_number;
  unsigned counter_number;
  unsigned pair_number;
  double temperature_drift;
  unsigned points_before_jump;
  double min_attenuation;
	bool nocamac;
	bool restore;

  std::string vvi;
  struct lsrp_t
  {
    std::string ip_address="192.168.163.79";  //ip address of the rokk1m-lsrp server
    int port=9090;                            //port number of the lsrp-control
  } lsrp;

  struct vepp4_t
  {
    boost::filesystem::path stap_file;
    boost::filesystem::path eCCD_file;
    boost::filesystem::path pCCD_file;
    boost::filesystem::path orbit_file;
    boost::filesystem::path RF_file;
  } vepp4;

  struct gui_t
  {
    unsigned fit_max_par_number;
    unsigned autorare;
    std::string default_graph;
    bool autofit;
		int  show_last_point;
		std::string note_format="%11.6f";
    bool remember_graph_list=false; 
  } gui;

	struct measurement_t
	{
		std::string generator;
		double max_allowed_frequency_difference;
	} measurement;

	struct debug_t
	{
		int level=1; //debug level
    bool skip_data_storing=false;
	} debug;
  //Do some transformation with filename
  //calculating absulute file path
  void prepare(void)
  {
		std::clog << "Use configuration file: " << config_file << std::endl;
    save_dir = work_dir/save_dir;
    print_dir = work_dir/print_dir;
    camac_config_file = work_dir/camac_config_file;
    measurement_config_file = work_dir/measurement_config_file;
//    last_measurement_config_file = work_dir/last_measurement_config_file;
    last_good_measurement_config_file = work_dir/last_good_measurement_config_file;
    last_bad_measurement_config_file = work_dir/last_bad_measurement_config_file;
    tmp_save_file = save_dir/tmp_save_file;
    last_tmp_save_file = save_dir/last_tmp_save_file;
    log_file = work_dir/log_file;
    std::clog << "Set polarimeter work directory: " << work_dir << std::endl;
    std::clog << "Saving directory: " << save_dir << std::endl;
    std::clog << "Camac configuration file: " << camac_config_file << std::endl;
    std::clog << "Measurement configuration file: " << measurement_config_file<< std::endl;
    std::clog << "Temporary saving file: " << tmp_save_file << std::endl;
    std::clog << "Log file: " << log_file << std::endl;
  }
};

extern PolarimeterConfig PCFG;

#endif
