#ifndef IBN_TIME_H
#define IBN_TIME_H
//#=====================================================================================
//#
//#       Filename:  Time.h
//#
//#    Description:  Классы для управления временем, поскольку мне надоело
//#                   выписывать одни и те же функции для сравнения времен.
//#
//#        Version:  1.0
//#        Created:  16.04.2003
//#       Revision:  none
//#       Compiler:  GNU C/C++
//#
//#         Author:  Ivan B. Nikolaev
//#        Company:  BINP,Russia
//#          Email:  I.B.Nikolaev@inp.nsk.su
//#
//#=====================================================================================

#include <sys/time.h>

#include <string>
#include <sstream>
#include <iomanip>

#include <map>
#include <chrono>

//inline double currenttime(void)
//{
//	timeval t;
//	gettimeofday(&t,0);
//	return  (double)t.tv_sec + t.tv_usec*1.e-6;
// // auto t = std::chrono::system_clock::now();
//}


template<class Clock>
class Timer
{
  public:
    Timer(void)  {restart();};
    void restart(void) 
    {
      begin_time = Clock::now();
    }

    void restart(typename Clock::time_point t)
    {
      begin_time = t;
    }

    typename Clock::duration elapsed() const
    {
      return Clock::now() - begin_time;
    }
    typename Clock::time_point get_begin_time(void) const { return begin_time; }
  private:
    typename Clock::time_point begin_time;
};

template < class X, class Y>
Y fuzzy_find( const std::map<X, Y> & M, const X & x)
{
  auto p1 = M.upper_bound(x);
  if(p1 != M.begin()) --p1;
  return p1->second;
}


inline double linear_find(const std::map<std::chrono::system_clock::time_point, double> & M, const std::chrono::system_clock::time_point & x)
{
  auto p1 = M.upper_bound(x);
  auto p2 = p1;

  if(p1 != M.begin()) --p1;
  else return 0;

  if(p2 == M.end()) return 0;
  
  auto x1=p1->first;
  auto x2=p2->first;
  auto y1=p1->second;
  auto y2=p2->second;

  return y1 + (y2-y1)*double((x-x1).count())/(x2-x1).count();
}
 


#endif
