#ifndef IBN_V4PAR_H
#define IBN_V4PAR_H
/*
 * =====================================================================================
 * 
 *        Filename:  V4Par.h
 * 
 *     Description:  âÉÂÌÅÏÔÅËÁ ÄÌÑ ÒÁÂÏÔÙ Ó ÐÁÒÁÍÅÔÒÁÍÉ ÷üððÁ
 * 
 *         Version:  1.0
 *         Created:  09.03.2004 16:25:04 NOVT
 *        Revision:  none
 *        Compiler:  gcc
 * 
 *          Author:  Ivan Nikolaev ()
 *         Company:  BINP
 *           Email:  I.B.Nikolaev@inp.nsk.su
 * 
 * =====================================================================================
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <memory>

#include <cerrno>
#include <stdexcept>

#include <vector>
#include <list>
#include <map>
#include <chrono>
#include <thread>

#include <string> 

#include <boost/format.hpp>

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <fcntl.h>

#include <KDB/kdb.h>
#include "data/Data.h"

#include "ibn/regexp.h"
#include "ibn/averager.h"
#include "ibn/timer.h"

#include <pqxx/connection>
#include <pqxx/transaction>

#include <Log.h>

extern bool iserror;


class KDBError : public std::runtime_error
{
  const std::string tablename;
  const int tableid;
  std::string err_str;
  public:
  KDBError(std::string err,std::string tn, int id) : runtime_error(err),tablename(tn),tableid(id)
  {
    err_str = err;
  }
  KDBError(std::string err,std::string tn) : runtime_error(err),tablename(tn),tableid(0)
  {
    err_str = err;
  }
  ~KDBError(void) throw(){}
  std::string what(void)
  {
    std::ostringstream os;
    os << err_str << ": " << tablename << " (" << tableid << ")";
    return os.str();
  }
};

class KDBCalibration
{
  protected:
    const std::string   tablename;
    const int tableid;
    KDBconn * conn;
    std::vector <int32_t> data;
    void read( time_t t = 0)
    {
      if ( ! kdb_read ( conn, tableid, t, &data[0], data.size()))
      {
        throw KDBError("Unable to read data from KDB ",tablename,tableid);
      }
    }
    void read (int tid, time_t t, void * data, size_t size)
    {
      if ( ! kdb_read ( conn, tid, t, data, size))
      {
        throw KDBError("Unable to read data from KDB ",tablename,tableid);
      }
    }

    void write ( time_t t=0)
    {
      if ( ! kdb_write(conn, tableid, t, &data[0],data.size()))
      {
        throw KDBError("Unable to write data to KDB ",tablename,tableid);
      }
    }
    void write (int tid, time_t t, void * data, size_t size)
    {
      if ( ! kdb_write ( conn, tid, t, data, size))
      {
        throw KDBError("Unable to write data to KDB ",tablename,tableid);
      }
    }
    /*
       KDBCalibration ( KDBconn * c, const std::string tn) :
       conn (c),
       tablename (tn),
       tableid (kdb_get_id(c, tn.c_str())) {}
       */
    /*
       KDBCalibration ( KDBconn * c, std::string tn) :
       tablename (tn),
       tableid (kdb_get_id(c, tn.c_str())) {
       conn = c;
       }
       */

    KDBCalibration ( KDBconn * c, std::string  tn, int id ) : tablename (tn), tableid (id) 
  {
    conn =c;
  }

    KDBCalibration ( KDBconn * c, int id ) : tablename ("unknown"), tableid (id) 
  {
    conn =c;
  }


};

class V4NameConvertor	
{
  static const unsigned MAX_CBUF_SIZE = 1024;
  std::map <std::string, unsigned> map_number;
  public:
  V4NameConvertor(const char *name_file)  
  {
    char cbuf[MAX_CBUF_SIZE];
    ibn::regex empty("^ *$");
    std::string tmp_str;
    std::ifstream namefile(name_file);
    if (!namefile)
    {
      std::cerr << " Can't open file " << name_file << " with names\n";
      //throw KDBError("V4NameConvertor: Unable to open file", name_file);
    }
    for ( int i=0; !namefile.eof(); i++)   
    {
      namefile.getline(cbuf,MAX_CBUF_SIZE);
      if (!empty.ismatch(cbuf))	
      {
        std::istringstream istr(cbuf); 
        istr >> tmp_str;
        map_number[tmp_str]=i+1;
      }
    }
  }

  unsigned number(std::string s) const 
  {
    std::map <std::string, unsigned>::const_iterator I = map_number.find(s);
    if ( I  == map_number.end() ) return 0;
    return I->second;
  }
};


class V4Current 
{
  std::string data_fn, set_fn,name_fn;
  protected:
  static constexpr unsigned NUMBER_OF_ELEMENTS=400;
  static constexpr double ENERGY_SCALE=0.01;
  static constexpr double CURRENT_SCALE=0.01;
  public:
  std::vector <double> Iset;
  std::vector <double> Imeas;
  std::string mode;
  int mode_num;
  double energy;
  int status;
  double NTZ1,NTZ3,STZ1,STZ3; 
  double URF,JRF;


  V4Current	(  
      std::string dfn="/home/vepp4/kadrs/bank-v4-mea", 
      std::string sfn="/home/vepp4/kadrs/bank-v4-set", 
      std::string nfn="/home/vepp4/kadrs/bank-v4-names"
      )   
  {
    Iset.resize( NUMBER_OF_ELEMENTS );
    Imeas.resize( NUMBER_OF_ELEMENTS );
    data_fn = dfn;
    set_fn = sfn;
    name_fn = nfn;
    mode="----";
    energy=1850;
    status=0;
    fill();
  }

  /*
     V4Current(void)
     {
     Iset.resize( NUMBER_OF_ELEMENTS );
     Imeas.resize( NUMBER_OF_ELEMENTS );
     }
     */

  void fill(void)	
  {
    const unsigned IGNORENUM=100;
    union  A {
      char str[8];
      int num;
    } a;

    std::ifstream meas_file (data_fn.c_str());
    if ( ! meas_file )  {
      std::cerr << "Can't open file \"" << data_fn << "\" : " << strerror(errno) << std::endl;
      //throw KDBError("Unable to open file", data_fn);
    }
    std::ifstream set_file(set_fn.c_str());
    if(!set_file) {
      std::cerr << "Can`t open file \"" << set_fn << "\": " <<  strerror(errno) << std::endl;
      //throw KDBError("Unable to open file", set_fn);
    }



    /* ignore line with the time, it is 1 line*/
    meas_file.ignore(IGNORENUM,'\n');		
    set_file.ignore(IGNORENUM,'\n');

    /* read the mode of the VEPP4-M it is second line */
    meas_file >> a.str; 
    a.str[4]=0;
    set_file.ignore(IGNORENUM,'\n');
    mode = a.str; //read vepp4 mode
    mode_num = a.num;
    meas_file.ignore(IGNORENUM,'\n');
    set_file.ignore(IGNORENUM,'\n'); 	    

    meas_file >> status;		    
    set_file.ignore(IGNORENUM,'\n');

    meas_file >> energy;
    energy*=ENERGY_SCALE;

    double tmp;
    for ( unsigned i = 0; !set_file.eof() && !meas_file.eof() && i < Imeas.size(); i++ )	{
      meas_file >> tmp;
      Imeas[i]=CURRENT_SCALE*tmp;
      set_file >> tmp;
      Iset[i] = CURRENT_SCALE*tmp;
    }
    NTZ1=Imeas[107];
    NTZ3=Imeas[109];
    STZ1=Imeas[139];
    STZ3=Imeas[141];

    /*
       cerr << "NTZ1=" << NTZ1 << endl;
       cerr << "NTZ3=" << NTZ3 << endl;
       cerr << "STZ1=" << STZ1 << endl;
       cerr << "STZ3=" << STZ3 << endl;
       cerr <<  "TZ=" << TZ() << endl;
       */

    URF=Imeas[355];
    JRF=Imeas[356];
    meas_file.close();
    set_file.close();	
  }

  double TZ(void) 
  {
    return 1.104031*NTZ1 + 1.204396*STZ1 + 0.722639*NTZ3 + 0.8396870*STZ3;
  }

};


class V4Termo	
{
  const std::string data_file_name; 
  static constexpr unsigned NUMBER_OF_SENSORS=381-5;
  static constexpr double TEMP_SCALE=1./512;
  static constexpr unsigned ROW_SHIFT=7;
  public:
  std::vector <double> T;
  V4Termo(void):data_file_name("/home/vepp4/kadrs/temp-v4")
  {
    T.resize(NUMBER_OF_SENSORS);
    std::ifstream file(data_file_name.c_str());
    if(!file)
    {
      std::cerr << "Unable to open file: " << data_file_name << std::endl;
    }
    for ( unsigned i=0; i<6; i++) file.ignore(1024,'\n');
    for ( unsigned i=0; i<T.size();i++)	
    {
      file >> T[i];
      T[i]=TEMP_SCALE*T[i];
    }
    file.close();
    MAG_N12F=T[67-ROW_SHIFT];
    MAG_N7F=T[68-ROW_SHIFT];
    MAG_S15F=T[211-ROW_SHIFT];
    MAG_S12F=T[205-ROW_SHIFT];
    MAG_S8F=T[207-ROW_SHIFT];
    MAG_S3F=T[125-ROW_SHIFT];

    D_V4_OUT=T[168-ROW_SHIFT];
    D_V4_IN=T[167-ROW_SHIFT];

    STENA_NI=T[16-ROW_SHIFT];
    STENA_NE=T[46-ROW_SHIFT];
    STENA_SI=T[184-ROW_SHIFT];
    STENA_SE=T[214-ROW_SHIFT];
  }
  double MAG_S15F,MAG_S12F,MAG_S8F,MAG_S3F,MAG_N12F,MAG_N7F;
  double MAG_KM_I,MAG_KM_O;
  double MAG_SEM1,MAG_SEM2,MAG_NEM1,MAG_NEM2;
  double MAG_SIM3,MAG_NIM4;
  double AIR_S,AIR_E,AIR_N,AIR_T;
  double D_V3_IN,D_V3_OUT,D_V4_IN,D_V4_OUT,D_SE_OUT,D_NE_OUT,D_T_IN,D_T_OUT,D_R3_OUT,D_R4_OUT;
  double TECH_IN,TECH_OUT;
  double STENA_NI,STENA_NE,STENA_SI,STENA_SE;

};


class VDB
{
	protected:
	const std::string vdb_config="host=vepp4-pg user=vepp4 dbname=v4parameters";
	mutable std::unique_ptr<pqxx::connection> vdb_conn;
	//mutable std::unique_ptr<pqxx::asyncconnection> vdb_conn;
	mutable std::unique_ptr<pqxx::work> vdb_xact;
	public:
	VDB(void)
	{
		init_vdb_connection();
	}
	time_t get_unixtime(const char * time_string) const 
	{
		struct tm  tm;
		strptime(time_string, "%Y-%m-%d %H:%M:%S", &tm);
		return mktime(&tm);
	}
	protected:
	void init_vdb_connection(void) const
	{
		//if no connection try to establish conection
		if(!vdb_conn) 
		{
			std::clog << LOG(2) << "Init new connection: " << vdb_config << std::endl;
      try
      {
        //vdb_conn.reset(new pqxx::asyncconnection(vdb_config));
        vdb_conn.reset(new pqxx::connection(vdb_config));
        //define the query processor
        vdb_xact.reset(new pqxx::work(*vdb_conn.get(),"VEPP parameters"));
      }
      catch(std::runtime_error e)
      {
        std::clog<< LOG(WARNING_LOG) << "ERROR: Unable to open connection to vepp4 database: " << vdb_config << std::endl;
      }
		}
	}
};

class V4TermoNew  : VDB
{
	struct Data_t
	{
		std::string name;
		int kdb_column; //column number in kedr data base
		double T;    //temperature, grad C
	};
	//mutable std::unique_ptr<pqxx::connection> v4db_conn;
	//mutable std::unique_ptr<pqxx::work> v4db_xact;
	//const std::string v4db_config="host=vepp4-pg user=vepp4 dbname=v4parameters";
	const std::string vdb_query="SELECT * FROM \"Tv4&chan\" ORDER BY date_time DESC LIMIT 1";
	std::map<std::string, Data_t> Data;
	time_t Time;
	public:
	V4TermoNew(void)
	{
		read();
	}


	void read(void)
	{
    std::clog << LOG(DEBUG_LOG) <<  "V4TermoNew::read BEGIN" << std::endl;
    if(!vdb_conn) return;
		//read temperatures from vepp4 data base
		try
		{
			//init_vdb_connection();
			Data.clear();
			try
      {
        //execute query
        pqxx::result res = vdb_xact->exec(vdb_query);
        //loop over lines
        for(const auto & line : res)
        {
          for (const auto & column : line)
          {
            if(std::string(column.name())==std::string("date_time")) //read the time
            {
              Time = get_unixtime(column.c_str());
              continue; //skip the time and date
            }
            double T=0;
            try 
            {
              T = std::stod(column.c_str());
            }
            catch(...)
            {
              T = -273.15;
            }
            Data_t data;
            data.name = column.name();
            data.T = T;
            Data[data.name]=data;
          }
        }
      }
			catch (...)
			{
				std::cerr << "Failed to execute query: " << vdb_query<<  std::endl;
			}
		}
		catch(pqxx::broken_connection)
		{
			std::cerr << "Failed to establish connection with: " << vdb_config << std::endl;
		}
    std::clog << LOG(DEBUG_LOG) <<  "V4TermoNew::read END" << std::endl;
	}
	double operator[](std::string key)
	{
		return Data[key].T;
	}
};

class VeppDuty :VDB
{
	public:
	std::string duty_koi8;
	std::string duty_translit;
	VeppDuty(void)
	{
		read();
	};
	
	std::string transliterate(const char * text)
	{
	   unsigned char rus[33*2]= { 225, 193, 226, 194, 247, 215, 231, 199, 228, 196, 229, 197, 179, 163, 246,
			214, 250, 218, 233, 201, 234, 202, 235, 203, 236, 204, 237, 205, 238,
			206, 239, 207, 240, 208, 242, 210, 243, 211, 244, 212, 245, 213, 230,
			198, 232, 200, 227, 195, 254, 222, 251, 219, 253, 221, 255, 223, 249,
			217, 248, 216, 252, 220, 224, 192, 241, 209
		};
    std::string eng[33*2]={"A", "a", "B", "b",  "V", "v", "G", "g", "D", "d", "E", "e", "E", "E", "Zh", "zh", "Z", "z", "I", "i", "J", "y",  "K", "k",   
    "L", "l", "M", "m", "N", "n", "O", "o", "P", "p",   "R", "r", "S", "s", "T", "t", "U", "u", "F", "f",   "KH", "kh", "Ts", "ts", "Ch", "ch", "Sh", "sh",   
    "Sch", "sch", "'", "'", "Y", "y","'", "'", "E", "e", "Yu", "yu", "Ya", "ya"};

		std::map < unsigned char, std::string> T;
		for(int i=0;i<33*2;i++) 
		{
			T[rus[i]]=eng[i];
			//std::cout << rus[i] << ": " << eng[i] << std::endl;
		}
    std::string ret;
		for(int i=0; text[i]!=0; i++) ret += T[text[i]];
		return ret;
	};

	void read(void)
	{
    if(!vdb_conn) return;
		std::string vdb_query;
		try
		{
			//init_vdb_connection();
			try
			{
				int nduty=0;
				time_t t = time(0); //current time
				tm * tt =  localtime(&t);
				int hour = tt->tm_hour;
				if(hour<9)
				{
					t=t-86400; //go to yesterday
				};
				tt = localtime(&t);
				int year = tt->tm_year+1900;
				int month= tt->tm_mon+1;
				int day = tt->tm_mday;
				if(hour < 9 || hour > 21) nduty=2; //night shift
				else nduty = 1;
				boost::format query_format("SELECT dname FROM ttvduty where ddate='%04d-%02d-%02d' and nduty=%d LIMIT 1");
				vdb_query = (query_format % year % month % day % nduty).str();
				pqxx::result res = vdb_xact->exec(vdb_query);
				//loop over lines
				for(const auto & line : res)
				{
					for (const auto & column : line)
					{
						if(column.name()==std::string("dname")) duty_koi8=column.c_str();
					}
				}
				duty_translit = transliterate(duty_koi8.c_str());
			}
			catch (...)
			{
				std::cerr << "Failed to execute query: " << vdb_query<<  std::endl;
			}
		}
		catch(pqxx::broken_connection)
		{
			std::cerr << "Failed to establish connection with: " << vdb_config << std::endl;
		}
	}
};

class V3Polarization :VDB
{
	const std::string vdb_query=
		"SELECT date_time, v4_elpo, v3_status, v3_elpo, v3_staying, \"v3_SCAN_Q1\",  \"v3_SCAN_Q2\",  \"v3_SCAN_TIME\" FROM stap where (v4_elpo=5 or v4_elpo=6) and v3_staying>1800 ORDER BY date_time DESC LIMIT 1";
	bool isready=false;
	public:
	double Q1, Q2;
	time_t t;
	time_t poltime; //polarization time
	std::unique_ptr<std::thread>  exec_thread;
	V3Polarization(void)
	{
		Q1=0;
		Q2=0;
		t =0;
		poltime = 0;
		//exec_thread.reset(new std::thread([this]{ read();}));
		read();
	};

	~V3Polarization(void)
	{
		//exec_thread->detach();
	}

	bool ready(void) const { return isready;}

	void read(void)
	{
		//read temperatures from vepp4 data base
		isready=false;
		try
		{
			//init_vdb_connection();
			ibn::timer timer;
			if(vdb_conn)
			{
				try
				{
					//execute query

					pqxx::result res = vdb_xact->exec(vdb_query);
					//loop over lines
					for(const auto & line : res)
					{
						for (const auto & column : line)
						{
							if(column.name()==std::string("date_time")) t = get_unixtime(column.c_str());
							if(column.name()==std::string("v3_SCAN_Q1")) Q1 = std::stod(column.c_str());
							if(column.name()==std::string("v3_SCAN_Q2")) Q2 = std::stod(column.c_str());
							if(column.name()==std::string("v3_staying")) poltime = std::stoi(column.c_str());
						}
					}
					//std::cout << "Q1= " << Q1 << " Q2 = " <<  Q2 << "  poltime = " << poltime << std::endl;
					isready=true;
				}
				catch (...)
				{
					std::cerr << "Failed to execute query: " << vdb_query<<  std::endl;
				}
			}
		}
		catch(pqxx::broken_connection)
		{
			std::cerr << "Failed to establish connection with: " << vdb_config << std::endl;
		}
	}
};

class V4CurrentKDB  : public KDBCalibration , public V4Current 
{
  enum DESCRIPTION { REGULAR, TEST, CHANGE, CYCLE, BAD} description;
  public:
  V4CurrentKDB
    ( 
     KDBconn *c, 
     int descr = REGULAR,
     std::string dfn="/home/vepp4/kadrs/bank-v4-mea", 
     std::string sfn="/home/vepp4/kadrs/bank-v4-set", 
     std::string nfn="/home/vepp4/kadrs/bank-v4-names"
    ) :
      KDBCalibration (c, "vepp4current", 2103 ) ,
      V4Current(dfn,sfn,nfn)
  {
    descr = description;
    data.resize( NUMBER_OF_ELEMENTS + 3);
    data[0] = int (description); 
    data[1] = mode_num;
    data[2] = int ( energy/ENERGY_SCALE );
    for ( unsigned i = 0; i < Imeas.size();i++)
    {
      data[3 + 2 * i] = int(Imeas[i]/CURRENT_SCALE);
      data[4 + 2 * i] = int(Iset[i]/CURRENT_SCALE);
    }
    write ();
  }
};

class V4mwpc : public KDBCalibration {
  static const unsigned TBL_LENGTH=13;
  const int lo_tableid;
  std::string lo_tablename;
  KDBlo *lo;
  std::string data_fn;


  static constexpr int BUFIGNORE = 1024;
  static constexpr double SCALE = 1000;

  public:
  struct Hist {
    int entry;
    double rms;
    double mean;
    double min;
    double max;
    int bins;
    unsigned *his;
  } x, z ;


  V4mwpc ( KDBconn * c, std::string  tn, int id, std::string lotn, int lo_id, std::string dfn) :
    KDBCalibration (c,tn,id), lo_tableid(lo_id)
  {
    lo_tablename=lotn;
    data_fn=dfn;
    //ðÏÌÕÞÁÅÍ ×ÒÅÍÑ ÍÏÄÉÆÉËÁÃÉÉ ÆÁÊÌÁ 
    struct stat st;
    stat(data_fn.c_str(),&st);
    time_t last_time = kdb_get_begin_time(conn,id, time(0));
    time_t current_time = kdb_time(conn);
    time_t dt=current_time - time(0);
    if ( st.st_mtime+dt-10 > last_time )	
    {
      read_data();
      head_fill();
      write(st.st_mtime+dt);
      lo = new KDBlo[2];
      lowrite(st.st_mtime+dt);
    } 
    else 
    {
      data.resize(TBL_LENGTH);
      read();
      head_extract();
      //print_head(cerr);
      x.his = new unsigned[x.bins];
      z.his = new unsigned[z.bins];
      lo=new KDBlo[2];
      loread();
      //print_his();
    }
  }



  V4mwpc (KDBconn *c, int tblid,int lo_id, time_t time ):KDBCalibration(c,tblid),lo_tableid(lo_id)
  {
    data.resize(TBL_LENGTH);
    read(time);
    head_extract();
    print_head(std::cerr);


    x.his = new unsigned[x.bins];
    z.his = new unsigned[z.bins];
    lo=new KDBlo[2];
    loread(time);
  }

  ~V4mwpc(void) {
    delete[] x.his;
    delete[] z.his;
    delete[] lo;
  }
  void print_his(void) {
    for (int i=0; i<x.bins; i++)   {
      std::cerr << std::setw(15) << x.his[i] <<std::setw(15)<< z.his[i] << std::endl;
    }
  }
  private:
  void print_head(std::ostream &os)	
  {
    int w = 10;
    os << std::setw(w) << "x/z" << std::setw(w) << "entry" << std::setw(w) << "rms" ;
    os<<std::setw(w)<<"mean"<<std::setw(w)<<"min"<<std::setw(w)<<"max"<<std::setw(w)<<"bins"<< std::endl;

    os<<std::setw(w)<<"x"<<std::setw(w)<<x.entry<<std::setw(w)<<x.rms;
    os<<std::setw(w)<<x.mean<<std::setw(w)<<x.min<<std::setw(w)<<x.max<<std::setw(w)<<x.bins<<std::endl;

    os<<std::setw(w)<<"z"<<std::setw(w)<<z.entry<<std::setw(w)<<z.rms;
    os<<std::setw(w)<<z.mean<<std::setw(w)<<z.min<<std::setw(w)<<z.max<<std::setw(w)<<z.bins<<std::endl;

  }

  void read_data(void)    {
    std::ifstream head(data_fn.c_str());
    if (!head)   {
      std::cerr << "Cant open file " << data_fn << " for reading vepp4 beam profile\n";
      throw KDBError("mwpc: Unable to open file",data_fn);
    }
    //Skip date information
    head.ignore(BUFIGNORE,'\n');
    head.ignore(BUFIGNORE,'\n');
    head >> x.entry >> z.entry; head.ignore(BUFIGNORE,'\n');
    head >> x.rms >> z.rms;     head.ignore(BUFIGNORE,'\n');
    head >> x.mean >> z.mean;     head.ignore(BUFIGNORE,'\n');
    head >> x.min >> z.min;     head.ignore(BUFIGNORE,'\n');
    head >> x.max >> z.max;     head.ignore(BUFIGNORE,'\n');
    head >> x.bins >> z.bins;     head.ignore(BUFIGNORE,'\n');
    //cerr << "Reading histogram information: " << endl;
    //print_head(cerr);

    // filling histogram 
    unsigned bins_hist = x.bins < z.bins ? x.bins : z.bins;
    x.bins=bins_hist;
    z.bins=bins_hist;
    x.his = new unsigned[x.bins];
    z.his = new unsigned[z.bins];
    for ( unsigned i=0; i< bins_hist; i++)    {
      head >> x.his[i] >> z.his[i];
    }
    //print_his();

  }

  void head_fill (void)   {
    data.resize(TBL_LENGTH);
    data[0]=0;
    data[1]=x.entry; data[2]=z.entry;
    data[3]= int32_t(SCALE*x.rms);  data[4] = int32_t (SCALE*z.rms);
    data[5]= int32_t(SCALE*x.mean);  data[6] = int32_t (SCALE*z.mean);
    data[7]= int32_t(SCALE*x.min);  data[8] = int32_t (SCALE*z.min);
    data[9]= int32_t(SCALE*x.max);  data[10] = int32_t (SCALE*z.max);
    data[11]= x.bins;  data[12] = z.bins;
  }

  void head_extract(void) {
    x.entry=data[1]; z.entry=data[2];
    x.rms = double(data[3])/SCALE; z.rms = double(data[4])/SCALE; 
    x.mean = double(data[5])/SCALE; z.mean = double(data[6])/SCALE; 
    x.min = double(data[7])/SCALE; z.min = double(data[8])/SCALE; 
    x.max = double(data[9])/SCALE; z.max = double(data[10])/SCALE; 
    x.bins=data[11]; z.bins=data[12];
  }
  void loimport(KDBlo * l, Hist * h)	{
    const char * name="/tmp/mwpchis";
    int f=creat(name, S_IRUSR | S_IWUSR );
    if (f==-1)	{
      perror("Unable to open file /tmp/mwpchis");
      throw KDBError("Unable to open file /tmp/mwpchis","",0);
      exit(2);
    }
    int bs=::write(f,(void*)h->his,sizeof(unsigned)*h->bins);
    if (bs!=int(sizeof(unsigned)*h->bins )) { 
      perror("Unable to print into file /tmp/mwpchis"); 
      throw KDBError("Unable to print into ÆÁÊÌ /tmp/mwpchis","",0);
    }
    close(f);
    kdb_lo_import(conn,l,name);
  }
  void loexport(KDBlo *l, Hist *h )   {
    const char * name="/tmp/mwpchis";
    kdb_lo_export(conn, l,name);
    int f=open(name,0);
    if (f==-1)	{
      perror("Unable to open file /tmp/mwpchis");
      throw KDBError("Unable to open file /tmp/mwpchis","",0);
      exit(1);
    }
    ::read(f,(void*)h->his,sizeof(unsigned)*h->bins);
    close(f);
  }

  void lowrite(time_t t=0)	{
    kdb_lo_init (&lo[0]);
    kdb_lo_init (&lo[1]);
    kdb_lo_write (conn,&lo[0],(char *)x.his,sizeof(unsigned)*x.bins);
    kdb_lo_write (conn,&lo[1],(char *)z.his,sizeof(unsigned)*z.bins);
    //loimport(&lo[0],&x);
    //loimport(&lo[1],&z);
    write (lo_tableid, t, lo,2);
  }

  void loread(time_t time=0)   {
    kdb_lo_init(&lo[0]);
    kdb_lo_init(&lo[1]);
    read(lo_tableid,time, lo, 2);
    kdb_lo_read(conn, &lo[0], (char*)x.his, sizeof(unsigned)*x.bins);
    kdb_lo_read(conn, &lo[1], (char*)z.his, sizeof(unsigned)*z.bins);
    //loexport(&lo[0],&x);
    //loexport(&lo[1],&z);
  }

};

class V4mwpce : public V4mwpc {
  public:
    V4mwpce(KDBconn *c, std::string  data) : V4mwpc(c,"vepp4mwpce",2107,"vepp4mwpcelo",2109,data){}
    V4mwpce(KDBconn *c) : V4mwpc(c,"vepp4mwpce",2107,"vepp4mwpcelo",2109, "/home/vepp4/kadrs/mwpc-e.dat"){}
    V4mwpce(KDBconn *c,time_t time) : V4mwpc(c,2107,2109,time){}
};

class V4mwpcp : public V4mwpc {
  public:
    V4mwpcp(KDBconn *c, std::string  data) : V4mwpc(c,"vepp4mwpcp",2108,"vepp4mwpcplo",2110,data){}
    V4mwpcp(KDBconn *c) : V4mwpc(c,"vepp4mwpcp",2108,"vepp4mwpcplo",2110, "/home/vepp4/kadrs/mwpc-p.dat"){}
    V4mwpcp(KDBconn *c,time_t time) : V4mwpc(c,2108,2110,time){}
};


class V4DataMonitor {
  static const int MAX_CBUF_SIZE=1024;
  std::string data_fn, name_fn; // "fn" means "file name"
  time_t last_check;

  //std::list <std::string> ;
  //std::list < unsigned > ; // list of 
  std::map <std::string, unsigned> umap; //ÓÐÉÓÏË ÄÁÎÎÙÈ, ËÏÔÏÒÙÅ ÄÏÌÖÎÙ ÏÂÎÏ×ÌÑÔØÓÑ.
  std::map <unsigned, double> old;

  public:
  V4DataMonitor(const std::list < std::string > & ulist, std::string data_str, std::string name_str) {
    data_fn = data_str;
    name_fn = name_str;
    std::ifstream namefile(name_fn.c_str());
    if (!namefile)  {
      std::cerr << " Can't open file " << name_fn << " with names\n";
      //throw KDBError("DataMonitor: Unable to open file", name_fn);
    }
    std::ifstream datafile(data_fn.c_str());
    if (!datafile)  {
      std::cerr << " Can't open file " << data_fn << " with names\n";
      //throw KDBError("DataMonitor: Unable to open file: ", data_fn );
    }
    std::string tmp_str;
    double tmp_data;
    char cbuf[MAX_CBUF_SIZE],cbufd[MAX_CBUF_SIZE];
    ibn::regex empty("^ *$");
    std::cerr << "List of elements to monitor:" << std::endl;
    for ( int i=0; !namefile.eof(); i++)   {
      namefile.getline(cbuf,MAX_CBUF_SIZE);
      datafile.getline(cbufd,MAX_CBUF_SIZE);


      if (!empty.ismatch(cbuf))	{
        std::istringstream istr(cbuf); istr >> tmp_str;
        std::istringstream istrd(cbufd); istrd >> tmp_data;
        if ( find(ulist.begin(),ulist.end(),tmp_str) != ulist.end() )   {
          umap[tmp_str]=i;
          old[i]=tmp_data;
          //std::cout << cbuf << std::endl;
        }
      }
    }
    last_check = time(0);
  }

  time_t getlast_check (void) { return last_check; }
  bool ischange(void) {
    bool flag=false;
    std::map<unsigned, double>::iterator p = old.begin();
    char cbufd[MAX_CBUF_SIZE];
    double tmp_data;
    std::ifstream data_fstr(data_fn.c_str());
    if (!data_fstr)  {
      std::cerr << " Can't open file " << data_fn << " with names\n";
      throw KDBError("DataMonitor: Unable to open fuile", data_fn);
    }
    /* ðÒÏÍÁÔÙ×ÁÅÍ ÐÏ ÆÁÊÌÕ ÌÉÂÏ ÄÏ ËÏÎÃÁ ÆÁÊÌÁ ÌÉÂÏ ÄÏ ËÏÎÃÁ ÓÐÉÓËÁ 
       ÐÁÒÁÍÅÔÒÏ× ÎÁ ÁÐÄÅÊÔ 
       */
    for ( unsigned i = 0; !data_fstr.eof() && p!=old.end();  i++  )	{
      data_fstr.getline(cbufd,MAX_CBUF_SIZE);
      /* ÅÓÌÉ ÎÁÈÏÄÉÍ, ÞÔÏ ÎÏÍÅÒ ÐÏÚÉÃÉÉ × ÓÐÉÓËÅ ÓÏ×ÐÁÄÁÅÔ Ó ÎÏÍÅÒÏÍ
         ÓÔÒÏÞËÉ × ÆÁÊÌÅ, ÔÏÇÄÁ ÞÉÔÁÅÍ ÄÁÎÎÙÅ É ÐÒÏ×ÅÒÑÅÍ, ÉÚÍÅÎÉÌÉÓØ
         ÌÉ ÏÎÉ, ÅÓÌÉ ÉÚÍÅÎÉÌÉÓØ, ÔÏ ×ÙÓÔÁ×ÌÑÅÍ ×ÌÁÇ yes × ÐÏÌÏÖÅÎÉÅ äá
         É ×ÙÈÏÄÉÍ
         */
      if ( p->first == i) {
        std::istringstream istrd(cbufd); istrd >> tmp_data;
        if ( p->second != tmp_data)	{
          //std::cout << std::endl << i << " " << p->second << " -> " << tmp_data ;
          p->second = tmp_data;
          flag=true;
        }
        p++;
      }
    }
    last_check=time(0);
    return flag; 
  }
};


class V4nmr {
  double tmp;
  public:
  V4nmr(void)  {
    std::string nmr_file_name = "/home/vepp4/kadrs/nmr.dat";
    std::ifstream file(nmr_file_name.c_str());
    nmr_data.resize(7);
    if(!file)    
    {
      std::cerr << LOG(WARNING_LOG) << "WARNING: V4nmr: unable to open file " << nmr_file_name<< '\n';
      return;
    }
    file.ignore(1024,'\n');
    file >> H;    file.ignore(1024,'\n');
    file >> NEM;  file.ignore(1024,'\n');
    file >> SEM;  file.ignore(1024,'\n');
    file >> SIM;  file.ignore(1024,'\n');
    file >> EM3;  file.ignore(1024,'\n');
    file >> KEDR1;  file.ignore(1024,'\n');
    file >> KEDR2;  file.ignore(1024,'\n');
    if ( H < 10 ) 
		{
      std::cerr  << LOG(WARNING_LOG) << "WARNING: V4nmr: ERROR wrong nmr for H magnet: " << H << std::endl;
    }
    nmr_data[0]=H;
    nmr_data[1]=NEM;
    nmr_data[2]=SEM;
    nmr_data[3]=SIM;
    nmr_data[4]=EM3;
    nmr_data[5]=KEDR1;
    nmr_data[6]=KEDR2;
  } 
  double H,NEM,SEM,SIM,EM3,KEDR1,KEDR2;
  std::vector<double> nmr_data;
};




class V4Vacuum
{
  public:
    double E; /*  VEPP-4M energy, MeV */
    double I; /*   sum current current */
    double Ie; /*  electron current */
    double Ip; /*  positron current */
    data::PhysValue Pa; /*  average vacuum */
    std::map <std::string, double > P; /* vacuum in nTor */
    

    V4Vacuum(void)
    {
      read_data();
    }

    void read_data(void)
    {
      const char * data_file_name = "/home/vepp4/kadrs/vmb-vacm";
      const char * name_file_name = "/home/vepp4/kadrs/vmb-names";
      std::ifstream name_file(name_file_name);
      if(!name_file)
      {
        std::cerr << "ERROR: Unable to open file " << name_file_name << std::endl;
        return;
      }
      /*  skip vepp3 information */
      for(int i=0;i<15;i++) name_file.ignore(1024,'\n');
      std::string name;
      std::map<int, std::string> name_map;
      for(int i=15;i<44;i++) 
      {
        name_file >> name;
        //std::cout << i << " " << name << std::endl;
        name_map[i]=name;
      }
      std::ifstream data_file(data_file_name);
      if(!data_file)
      {
        std::cerr << LOG(WARNING_LOG) << "WARNING: Unable to open file " << data_file_name << std::endl;
        return;
      }
      /*  skip vepp3 information */
      for(int line=0;line<15;line++) data_file.ignore(1024,'\n');
      data_file >> E; E/=100.; /*  read energy */
      data_file>>  I; /* read total current */
      data_file >> Ie; /*  read electron current */
      data_file >> Ip; /*  read positron current  */
      double data;
      ibn::averager<double> averager;
      for(int line=19;line<44;line++) 
      {
        data_file >> data;
        std::map < int, std::string>::iterator nmi = name_map.find(line);
        if(nmi==name_map.end()) continue;
        P[nmi->second] = data;
        //std::cout << line << " "  << nmi->second << " " << data << std::endl;
        if(data) averager.add(data);
      }
      Pa=data::PhysValue(averager.average(), averager.sigma_average());
    };

    void print(void)
    {
      using namespace std;
      cout << "VEPP-4M energy: " << E << " MeV" << endl;
      cout << "Total current: " << I << " mA" << endl;
      cout << "e- current: " << Ie << " mA" << endl;
      cout << "e+ current: " << Ip << " mA" << endl;
      std::map <std::string, double>::iterator dit; /*  data iterator */
      for(dit=P.begin(); dit!=P.end();dit++)
      {
        cout << dit->first << ": " << dit->second << endl;
      }
    }
};


#endif
