#ifndef IBN_ATTENUATOR_H
#define IBN_ATTENUATOR_H
#include "Module.h"

#include <algorithm>
#include <vector>
#include <map>
#include "ModuleInventory.h"

#include <cmath>
namespace camac  
{
  class Attenuator : virtual public ModuleInventory
  {
    // Abstract classs determines attenuation interface
    public:
      typedef double attenuation_t;
      virtual void SetAttenuation(attenuation_t a) = 0 ;
      virtual attenuation_t GetAttenuation(void) = 0; 
      virtual attenuation_t GetMaxAttenuation(void) const =0 ;
      virtual attenuation_t GetStep(void)const =0  ;
  };

  class Attenuator_A0608 : public Attenuator,  public  Module, virtual public ModuleInventory
  {
    static constexpr attenuation_t MAX_ATTENUATION=15;
    static constexpr attenuation_t STEP=1;
    public:
    Attenuator_A0608(std::string nm/*bockname*/, const camac::Address adr) :
      ModuleInventory(nm, "Attenuator", "A0608"),
      Module(adr, nm, "Attenuator", "A0608")
    {
    }

    Attenuator_A0608(std::string nm) :
      ModuleInventory(nm, "Attenuator", "A0608"),
      Module(nm, "Attenuator", "A0608")
    {
    }



    virtual ~Attenuator_A0608(void) {}

    void SetAttenuation(attenuation_t att)
    {
      u32_t data = (u32_t) att;
      AF(0,16,&data);
      //std::cout << "AttenuatorA0608-1: " << att << std::endl;
    }

    attenuation_t GetAttenuation(void)
    {
      u32_t data;
      try 
      {
        AF(0,0,&data);
      }
      catch (...)	
      {
      }
      data &=0xf;
      return data;
    }

    attenuation_t GetMaxAttenuation(void) const
    {
      return MAX_ATTENUATION;
    }
    attenuation_t GetStep(void) const { return STEP; }

    void Bind(camac::Address adr, attenuation_t att)
    {
      Module::Bind(adr);
      SetAttenuation(att);
    }
  };

  class Attenuator_A0608_3 : public Attenuator, public  Module, virtual public ModuleInventory
  {
    static constexpr attenuation_t MAX_ATTENUATION=45;
    static constexpr attenuation_t STEP=3;
    public:
    Attenuator_A0608_3(std::string nm/*bockname*/, const camac::Address adr) :
      ModuleInventory(nm, "Attenuator", "A0608_3"),
      Module(adr)
    {}

    Attenuator_A0608_3(std::string nm) : 
      ModuleInventory(nm, "Attenuator", "A0608_3")
    {}

    virtual ~Attenuator_A0608_3(void) {}

    void SetAttenuation(attenuation_t att)    
    {
      u32_t data = (u32_t)(att/3);
      AF(0,16,&data);
      //std::cout << "AttenuatorA0608-3: " << att << std::endl;
    }

    attenuation_t GetAttenuation(void)
    {
      u32_t data;
      try {
        AF(0,0,&data);
      }
      catch (...)	{
      }
      data &=0xf;
      data*=3;
      return data;
    }

    attenuation_t GetMaxAttenuation(void)  const 
    {
      return MAX_ATTENUATION;
    }

    attenuation_t GetStep() const { return STEP; }

    void Bind(camac::Address adr, attenuation_t att)
    {
      Module::Bind(adr);
      SetAttenuation(att);
      //std::cout << "Attenuator 45db: BIND " << std::endl;
      //std::cout << adr.crate << " " <<  adr.station << std::endl;
    }


  };


  class AttenuatorArray  : public Attenuator 
  {
    public:
      AttenuatorArray(void)
			{
        MAX_ATTENUATION=0;
        att_quant=0;
      };

      virtual ~AttenuatorArray(void)  
      {
        for(amap_t::iterator i=amap.begin(); i!=amap.end(); i++) 
				{
          //delete i->second;
        }
      }

      void AddAttenuator(Attenuator * a)
      {
        amap.insert(std::make_pair(a->GetStep(),a));
        attenuation_t max=0, quant = amap.begin()->second->GetStep();
        for(amap_t::iterator i=amap.begin(); i!=amap.end(); i++) 
        {
          quant=std::min(quant,i->second->GetStep());
          max+=i->second->GetMaxAttenuation();
        }
        MAX_ATTENUATION=max;
        att_quant=quant;

        attenuator_list.push_back(a);// add module to list
        std::clog<<LOG(4)<<"AttenuatorArray::AddAttenuator FINISH" << std::endl;
      }

      attenuation_t GetAttenuation(void)
      {
        size_t size=amap.size();
        if(size==0) return 0;
        attenuation_t sum=0;
        for(amap_t::iterator i=amap.begin(); i!=amap.end(); i++) sum+=i->second->GetAttenuation();
        return sum;
      }

			/* Complex attenuation set.
			 * List of attenuators are sorted by decreasing of attenuation step.
			 * Desired attenuation is rounded to possible attenuation for first attenuation in queue.
			 * remains go to the next attenuation in queue
			 */
      void SetAttenuation(attenuation_t a)   
      {
        if(amap.empty())
        {
          att=0;
          return;
        }
        attenuation_t s=0,m=0,sub=0,save;
        a=fabs(a);
        save=a;
        if(a>MAX_ATTENUATION) a=MAX_ATTENUATION;
        for(amap_t::iterator I =amap.begin(); I!=amap.end(); I++)
        {
          s=I->first;
          m=I->second->GetMaxAttenuation();
          sub = unsigned(a/s)*s;
          if( sub > m) sub=m;
          I->second->SetAttenuation(sub);
          a-=sub;
        }
        att=save-a;
        if (a>att_quant)
				{
          std::cerr << LOG(NORMAL_LOG) << "ERROR: bad attenuation " << save << '\n';
        }
      }

      std::list<camac::ModuleInventory*>* GetModuleList(void)
      {
        return &attenuator_list;
      }

      attenuation_t GetMaxAttenuation(void)const { return MAX_ATTENUATION; }
      attenuation_t GetStep(void)const  { return att_quant; }
    private:
      std::list<camac::ModuleInventory*> attenuator_list;
      typedef  std::multimap<attenuation_t, Attenuator *,std::greater<attenuation_t> > amap_t ;
      typedef amap_t::iterator I_t;
      amap_t amap;
      attenuation_t att;
      attenuation_t att_quant;
      attenuation_t MAX_ATTENUATION;
  };
}
#endif //IBN_ATTENUATOR_H
