#ifndef IBN_CAMAC_COMMUTATOR_H
#define IBN_CAMAC_COMMUTATOR_H
#include"Module.h"
namespace camac {
	class Commutator
	{
		public:
			virtual ~Commutator(void){};
			virtual void On(void) = 0;
			virtual void Off(void) = 0;
			virtual void Commutate(unsigned, unsigned) = 0;
			virtual unsigned GetCommutation(unsigned) = 0;
			//virtual int GetType(void) const {};
	};

	class Commutator_LO403 : public Commutator, public Module,  virtual public ModuleInventory
	{
		public:
			enum { TYPE_2 = 2, TYPE_4 = 4, TYPE_8 = 8};

			Commutator_LO403(const char* nm="unknown") 
				: ModuleInventory(nm, "Commutator", "L0403")
			{
			}

			Commutator_LO403(const char* nm, const Address & adr) 
				: ModuleInventory(nm, "Commutator", "L0403"), Module(adr)  
			{
				Ident();
			}

			void Bind(const Address & a)	
			{
				Module::Bind(a);
				Ident();
			}

			void Commutate(unsigned ch_in, unsigned ch_out)
			{
				write(read() & ((~(mask << shift*ch_out )) | ((ch_in & mask) << shift*ch_out)));
			}
			unsigned  GetCommutation(unsigned ch_out)
			{
				unsigned in = (read() >> ch_out*shift) & mask;
				return in;
			}
			void Off(void)  { write(read() | 0x80); }
			void On(void)   { write( read() & ~0x80);}
			bool IsOn(void)  const  { return !bool(read() & 0x80);}
			int GetType(void) const
			{
				int type;
				switch( read()>>4 & 0x3)
				{
					case 0x1:
						type = TYPE_2;
						break;
					case 0x2:
						type = TYPE_8;
						break;
					case 0x3:
						type = TYPE_4;
						break;
					default: type = -1; break;
				}
				return type;
			}
		private:
			u16_t read(void)   const  { u16_t data;	AF(0,READ,&data); return data;	}
			void write(u16_t data)    { AF(0,WRITE, &data); }
			enum COMMAND { READ = 0 , WRITE = 16 };
			void Ident(void)
			{
				switch( GetType())
				{
					case TYPE_2:
						mask = 0x1;
						shift = 1;
						break;
					case TYPE_8:
						mask = 0x7;
						shift = 3;
						break;
					case TYPE_4:
						mask = 0x3;
						shift = 2;
						break;
					default:
						cerr << "Commutator: bad commutator type" << endl;
						break;
				}
			}
			u16_t mask;
			unsigned shift;
	};//end of class Commutator_L0406
}//end of namespace camac
#endif //IBN_CAMAC_COMMUTATOR_H
