#ifndef IBN_COUNTER_H
#define IBN_COUNTER_H

#include <stdexcept>
#include "Module.h"
#include <time.h>
#include <sys/time.h>
#include "ModuleInventory.h"

namespace camac
{

  class Counter_base : protected Module, virtual public ModuleInventory
  {
    public:
      Counter_base(void){}
      Counter_base(const camac::Address adr) : Module(adr){}
      virtual ~Counter_base(void){}
      unsigned int read(int channel)  const 
      {
        //union   {
        u32_t DATA;
        u16_t data[2];
        //	};
        AF(2*channel, 0, &data[0]);
        AF(2*channel+1, 0, &data[1]);
        DATA = data[0] + (u32_t(data[1])<<16);
        return DATA;
      }
      //прочитать, а затем сбросить
      unsigned int read_reset(int channel)
      {
        //union	{
        u16_t data[2];
        u32_t DATA;
        //};
        AF(2*channel, 2, &data[0]);
        AF(2*channel+1, 2, &data[1]);
        // Когда старший бит младшего слова равен единице, тогда после
        // вычитывания и сброса еденичка добавляется в старшее слово.
        // Поэтому после вычитывания и сброса старшего слова, надо эту
        // единицу вычесть.
        if( (data[0]>>15) == 0x1 ) data[1]--; DATA = data[0] + (u32_t(data[1])<<16);
        return DATA;
      }

      void reset(int channel)  
      {
        try	{	AF(2*channel, 9);   } catch( camac::Module::Error)  {};
        try	{	AF(2*channel+1, 9);   } catch( camac::Module::Error)  {};
      }
      //Увеличить значение на единицу
      void increment(int channel) { AF(2*channel, 25); }

      //Check for overload of counter
      bool check(void)  const
      {
        u16_t data;
        AF(12,1,&data);
        return bool(data);
      }
      bool iscount(void)  const   { return  !Module::GetI();	}
      bool islam(void)  const  
      {
        /*
           try {
           AF(12,8);
           return true;
           }
           catch(camac::Module::Error e) {
           return e.getQ();
           }
           */
        return Module::IsLam();
      }
  };

  class CounterC0303 : private Counter_base, virtual public ModuleInventory
  {
    public:
      CounterC0303(std::string nm="unknown") : ModuleInventory(nm, "Counter", "C0303"){
        init();
      }
      CounterC0303(std::string nm, const camac::Address adr) :
        ModuleInventory(nm, "Counter", "C0303"),
        Counter_base(adr)

    { 
      init();
    }
      virtual ~CounterC0303(void){ 
        init();
      }
      unsigned int GetCount() const
      {
        return Counter_base::read(GetChannel());
      }
      unsigned int GetCountReset()
      {
        return  Counter_base::read_reset(GetChannel());
      }
      void ResetCount(void)
      {
        Counter_base::reset(GetChannel());
      }

      void Increment(unsigned int number)
      {
        for(unsigned int i = 0; i<number; i++)	Counter_base::increment(GetChannel());
        if(Counter_base::check()) throw std::overflow_error(std::string(GetType()) + ": Increment Overflow ");
      }
      bool IsCount(void)  const {	return Counter_base::iscount() ;}

      bool Check(void)    const 
      {
        return Counter_base::check();
      }

      double GetLoad(void)	
      {
        timeval tv;
        gettimeofday(&tv,0);	
        unsigned c = GetCount();
        double dt = tv.tv_sec - last_time.tv_sec + (tv.tv_usec - last_time.tv_usec)*1e-6;
        //double load = (c - last_load)/dt;
        double l = (c - last_counted)/dt;
        if(l>0) load=l ; /* if counter was reset just return last load */
        last_counted = c;
        last_time = tv;
        return load;
      }

      void Bind(const Address &adr)   { Module::Bind(adr);}

    private:
      timeval last_time; /* last call of GetLoad */
      double last_counted; /* last counter value of GetLoad */
      double load; /* last measured load */
      void init(void)
      {
        gettimeofday(&last_time,0);
        last_counted = 0;
        load=0;
      }
  };
}
#endif //IBN_COUNTER_H
