#ifndef IBN_DELAY_H
#define IBN_DELAY_H
#include "Module.h"
#include "ModuleInventory.h"

#include <boost/format.hpp>

#include "Log.h"
namespace camac	
{
  //Abstract class prividing interface for delay line
  class DelayLine : virtual public ModuleInventory
  {
    public:
      typedef unsigned int delay_t;
      //typedef double delay_t;
      virtual ~DelayLine(void){}
      //Delay in ns
      virtual delay_t GetDelay(void) const = 0;
      virtual void SetDelay(delay_t) = 0;
  };

  class DelayLine_L0606 : public Module , public  DelayLine, virtual public ModuleInventory
  {
    /*
       Разработчик Бехтенев Евгений Алексеевич
       тел 39-48-00
       */
    delay_t Step;//минимальный шаг
    delay_t MaxDelay; //256*5;
    const delay_t MinDelay = 0;
    public:
    DelayLine_L0606(std::string nm="unknown") : ModuleInventory(nm, "DelayLine", "L0606")
    {

    }

    DelayLine_L0606(std::string nm, const Address adr, delay_t step, delay_t delay=0) : 
      ModuleInventory(nm, "DelayLine", "L0606"),
      Module(adr),
      Step(step),
      MaxDelay(Step*256),
      MinDelay(0)
    {
      SetDelay(delay);
    }

    void Bind(const Address a, delay_t step, delay_t delay=0)	
    {
      MaxDelay = step*256;
      Step = step;
      Module::Bind(a);
      SetDelay(delay);
    }

    ~DelayLine_L0606(void){std::clog << LOG(DEBUG_LOG)<< "~DelayLine_L0606 " << std::endl;}

    virtual void SetDelay(delay_t delay) 
    {
      if(delay < MinDelay || delay > MaxDelay) 
      {
        std::string what = std::string(GetType()) + ": Bad delay time for module L0606(Delay line) ";
        boost::format fmt(" delay %f <  MinDelay %f OR delay %f > MaxDelay %f");
        fmt % delay % MinDelay % delay % MaxDelay;
        what+=fmt.str();
        throw std::out_of_range(what);
      }
      u32_t data = u32_t ( (delay - MinDelay)/Step );
      AF(GetChannel(),16, &data);
    }

    delay_t GetDelay(void) const
    {
      u32_t data;
      AF(GetChannel(),0, &data);
      //u8_t dat = ~(u8_t(data)); //very strange 
      u8_t dat = (u8_t(data));
      return dat*Step + MinDelay;
    }
  };//end of class LO606 new 

  class DelayLine_L0606_OLD : public Module , public  DelayLine, virtual public ModuleInventory
  {
    /*
       Разработчик Бехтенев Евгений Алексеевич
       тел 39-48-00
       */
    delay_t Step;//минимальный шаг
    delay_t MaxDelay;// = 256*5;
    static const delay_t MinDelay = 0;
    public:
    DelayLine_L0606_OLD(std::string nm="unknown") : ModuleInventory(nm, "DelayLine", "L0606_OLD")
    {

    }

    DelayLine_L0606_OLD(std::string nm, const Address adr, delay_t step, delay_t delay=0) :
      ModuleInventory(nm, "DelayLine", "L0606_OLD"),
      Module(adr)

    {
      MaxDelay = step*256;
      Step = step;
      SetDelay(delay);
      //			DelayLine::name=string(nm);
    }

    ~DelayLine_L0606_OLD(void){std::clog << LOG(DEBUG_LOG) << "~DelayLine_L0606_OLD" << std::endl;}

    void Bind(const Address a, delay_t step, delay_t delay=0)	
    {
      MaxDelay = step*256;
      Step = step;
      Module::Bind(a);
      SetDelay(delay);
    }


    virtual void SetDelay(delay_t delay) 
    {
      if(delay < MinDelay || delay > MaxDelay) throw std::out_of_range(std::string(GetType()) + ": Bad delay time for module L0606(Delay line)");
      u32_t data = u32_t ( (delay - MinDelay)/Step );
      AF(GetChannel(),16, &data);
    }

    delay_t GetDelay(void) const
    {
      u32_t data;
      AF(GetChannel(),0, &data);
      u8_t dat = ~(u8_t(data)); //very strange 
      //u8_t dat = (u8_t(data));
      return dat*Step + MinDelay;
    }
  };//end of class LO606

  class DelayLine_L0320 : public DelayLine, public camac::Module, virtual public ModuleInventory {
    static const delay_t MaxDelay = 25500; //ns
    static const delay_t MinDelay = 140;   //ns
    //static const delay_t Step = 50;   //ns
    static const delay_t Step = 100;   //ns
    public:
    DelayLine_L0320(const char* nm = "unknown") : ModuleInventory(nm, "DelayLine", "L0320"){}
    DelayLine_L0320(const char* nm, const Address adr, delay_t  delay = 0) :
      ModuleInventory(nm, "DelayLine", "L0320"),
      Module(adr)

    {
      SetDelay(delay);
      //			DelayLine::name=string(nm);
    }

    void Bind(const Address adr, delay_t  delay = 0)
    {
      Module::Bind(adr);
      SetDelay(delay);
    }

    ~DelayLine_L0320(void){}

    virtual void SetDelay(delay_t delay)
    {
      if(delay > MaxDelay || delay < MinDelay) throw std::out_of_range(std::string(GetType()) + ": Bad delay time");
      u32_t  data = (delay - MinDelay)/Step;
      AF(GetChannel(),17,&data);
    }

    delay_t GetDelay(void) const
    {
      u32_t data;
      AF(GetChannel(),1,&data);
      return  data*Step + MinDelay;
    }
  };//end of class DelayLine_L0320


  class DelayLineL0101 : public DelayLine, public Module, virtual public ModuleInventory
  {
    public:
      DelayLineL0101(const char* nm = "unknown") : ModuleInventory(nm, "DelayLine", "L0101") {}
      DelayLineL0101(const char* nm, const camac::Address adr, delay_t  delay = 0) :
        ModuleInventory(nm, "DelayLine", "L0101"),
        Module(adr)

    {
      SetDelay(delay);
    }
      ~DelayLineL0101(void){}
      virtual void SetDelay(delay_t delay)
      {
        if (delay >= 267)
        {
          std::cout << "Превышено максимальное время задержки 256 нсек.\n Установлено 256 нс";
          delay = 256;
        }
        u16_t t= (u16_t)delay;     
        AF(GetChannel(),16, &t);
      }

      delay_t GetDelay(void) const
      {
        u32_t data;
        AF(GetChannel(),1,&data);
        return  data;
      }
  }; //end of class DelayLineL0101

}//end of namespace camac
#endif // IBN_TDISKR_H
