#ifndef IBN_ENGINE_H
#define IBN_ENGINE_H
#include<iostream>



#include<vector>
#include<map>
//#include<algorythm>


#include <stdlib.h>
#include <fcntl.h>
#include <ctype.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <unistd.h>

#include <errno.h>
#include <exception>
#include <string.h>

using namespace std;
namespace modules
{
	//    static const char * ENGINE_PORT = "/dev/ttyS1";
	class Port
	{
		int fd;
		public:
		int Getfd(void) const {return fd;}
		Port(const char * device)
		{
			fd = open( device,O_RDWR | O_NOCTTY );
			if ( fd == -1 )	throw std::runtime_error(strerror(errno));
			/*
				 struct termios devopts;
				 cfmakeraw(&devopts);
				 cfsetispeed( &devopts, B19200 );
				 cfsetospeed( &devopts, B19200 );
				 tcsetattr( fd, TCSANOW, &devopts );
				 */
			fcntl(fd,F_SETFL,O_NONBLOCK);

		}
		~Port(void) { close(fd); }
	};

	class Engine
	{
		//define special bytes
		typedef vector<int8_t> Pocket_t;

		static const int8_t START = 0xAA;
		static const int8_t STOP = 0xAB;
		static const int8_t SHIFT = 0xAC;
		//static const map<double,uchar_t> current_map;
		static const unsigned WAIT = 500000; //mksec


		int8_t Address;

		struct Ident    {
			unsigned version;
			unsigned number;
			Ident(void) {}
			Ident(Pocket_t p)   {
				number = (static_cast<unsigned>(p[1]) << 8) + p[0];
				version = (static_cast<unsigned>(p[3]) << 24) + (static_cast<unsigned>(p[2]) << 16);
			}
		};

		const int Port;

		//checking kontrol byte
		bool isupr(int8_t b) {
			if(b == START || b == STOP || b == SHIFT) return true;
			else return false;
		}


		void umake_poket(Pocket_t &p)  {
			//delete STOP
			p.erase(p.end()-1,p.end());

			//delete SHIFT and plus START
			for(unsigned int i=0; i < p.size(); i++)  {
				if(p[i] == SHIFT)  {
					p[i+1] += START;
					p.erase(p.begin()+i,p.begin()+i+1);
				}
			}
			//calculate control summ
			int8_t control_sum = 0;

			//delete address
			if(p[0] != Address) throw std::runtime_error("wrong recived address ");
			p.erase(p.end()-1,p.end());

			//delete byte and conroll sum
			for(unsigned int i=0; i < p.size(); i++) control_sum^=p[i];
			if(control_sum != 0)   {
				cerr << "    " << endl;
				exit(-1);
			}
			//delete control_sum;
			p.erase(p.end(),p.end()-1);
		}

		void send(int8_t b)    {
			int n; //amount of writen byte;
			//write the START
			n = write(Port,&b,1);
			if(n == -1) throw std::runtime_error(strerror(errno));
			if(n != 1)  throw std::runtime_error("Can not write byte");
		}

		void send_byte(int8_t b)  {
			if(b == START || b == STOP || b == SHIFT)  {
				send(SHIFT);
				b-=START;
			}
			send(b);
		}

		void send_pocket(const Pocket_t &p)
		{
			//calculate control summ
			int8_t control_sum = Address;
			for(unsigned int i=0; i < p.size(); i++) control_sum^=p[i];
			send(START);
			send_byte(Address);
			for(unsigned i = 0; i < p.size(); i++) send_byte(p[i]);
			send_byte(control_sum);
			send(STOP);
			usleep(WAIT);
		}

		void recive_pocket(Pocket_t &p)    {
			usleep(WAIT);
			int s = read(Port,&p[0],p.size());
			cerr << "after read;\n";

			if(s == -1) 
			{
				throw std::runtime_error(strerror(errno));
			}
			if((unsigned(s)!=p.size())) {
				//perror("errror read:");
				cerr << "can not read data " << s << endl;
			}
			//umake_poket(p);
		}

		double current;
		double velocyty;

		enum COMMAND    {
			SETADR = 0,
			IDENT = 1,
			REPEAT = 2,
			STATUS = 3,
			MOVE = 4,
			MOVE_NO_ACCEL = 5,
			CONFIG = 6,
			VELOCITY = 7,
			STP = 8,
			OFF = 9,
			SAVE = 10,
			IMPULSE = 11,
			REMAIN = 12,
			GET_CONFIG = 13,
			GET_VELOCITY = 14,
			TEST_FREQ = 15,
			CALIBRATE = 16,
			MOVE_PREC = 17,
			OUTPUT_IMPULSE = 18,
			SET_COORDINATE = 19,
			GET_COORDINATE = 20,
			SYNCHRO = 21
		};
		public:
		struct Status {
			bool end;
			bool prec;
			bool dat;
			bool kplus;
			bool kminus;
			bool go;
			bool ready;
			Status(int8_t b)   {
				ready = b & 0x01;
				go = b & 0x2;
				kminus = b & 0x4;
				kplus = b & 0x8;
				dat = b & 0x10;
				prec = b & 0x20;
				end = b & 0x40;
			}
			~Status(void){}
			void print(void)    {
				cout << "status: " <<endl;
				cout << "end: " << end << endl;
				cout << "precision velocity: " << prec << endl;
				cout << "detector: " << dat << endl;
				cout << "K+: " << kplus << endl;
				cout << "K-: " << kminus << endl;
				cout << "go: " << go << endl;
				cout << "ready: " << ready << endl;
			}
		};


		unsigned GetAddress(void)  const  {
			return Address;
		}

		Engine(int port, int adr): Port(port){
			Address = adr;
		};
		~Engine(void){};
		void IdentDevice(void){}
		void Repeat(void){}

		Status GetStatus(void)
		{
			//int8_t COMMAND;
			Pocket_t cmd(1);
			cmd[0] = IDENT;
			send_pocket(cmd);
			Pocket_t answer(1);
			recive_pocket(answer);
			return Status(answer[0]);
		}

		void Move(int step)
		{
			Pocket_t cmd(5);
			cmd[0] = MOVE;
			cmd[4] = step & 0xff;
			cmd[3] = (step >> 8) & 0xff;
			cmd[2] = (step >> 16) & 0xff;
			cmd[1] = (step >> 24) & 0xff;
			send_pocket(cmd);

			/*
				 Pocket_t answr(1);
				 recive_pocket(answr);
				 cerr << "after recive_pocket(answr);\n";
				 */
		}

		void SetSpeed();
		double GetSpeed();
	};

}
#endif
