#ifndef IBN_FORM_H
#define IBN_FORM_H

#include "Module.h"

namespace camac {
  class Form_base : public Module, virtual public ModuleInventory {
    static const unsigned int Max = 12800; //ns
    static const unsigned int Min = 60;//ns
    static const unsigned int Step = 100; //ns
    public:
    Form_base(void) {}
    Form_base(const camac::Address adr) :	Module(adr) {}
    void Bind(const camac::Address & a ) { Module::Bind(a); }
    ~Form_base(){}

    unsigned read(int channel) const {
      u16_t data;
      //F1
      AF(channel,1, &data);
      return Min + unsigned(data*Step);
    }

    void write(int channel, unsigned int time)	{
      if( time > Max && time < Min ) throw std::out_of_range(std::string(GetType()) + ": Bad time");
      u16_t data = (u16_t)((time - Min )/Step);
      AF(channel,17, &data);
    }
  }; //class Form_base

  class Form : private Form_base, virtual public ModuleInventory {
    public:
      Form(const char* nm="unknown") : ModuleInventory(nm, "Formirovatel", "any"){}
      Form(const char*nm, const camac::Address adr, unsigned int time = 160) :
        ModuleInventory(nm, "Formirovatel", "any"),
        Form_base(adr)
    {
      write(GetChannel(),time);
    }
      void Bind(const camac::Address & a, unsigned time = 160) 	{
        Form_base::Bind(a);
        SetTime(time);
      }
      ~Form(){}
      unsigned int GetTime(void) const   { return read(GetChannel()); }
      void SetTime(unsigned int time)  {   write(GetChannel(),time);   }

      //			string GetName(void){return name;}
      //			void SetName(string nm){name=nm;}

      //	    private:
      //			string name; // 
  };

}

#endif // IBN_FORM_H

