/*Класс управляющий частотометром
  Блоки камак сделаны так, чтобы частотомер работал в ведущем режиме.
  То есть он постоянно измереят частоту/период/отношение частот.
  По окончагии Tизм (время измерения) выдает LAM запрос.
  Запретить LAM F24A0
  Разрешить LAM F26A0
  Проверить LAM F8A0
  Сброс LAM F10A0 (используется, если не могли маскировать LAM в ЭВМ)
  Читать данные F3A0 (данные передаются в инверсеногм виде в двоично-десятичном
  формате 4 бита для кодирования цифр от 0-9)
  Старт измерения(если перефирия ведущая. у нас это не так, так что эта функция
  не нужна)
  Чтение регистра состояниявыходных ключей F0A0
  Чтение дескриптора устройства F6F0
  Разработка блока PO603 - Ощепков 4.04.2000
 */

#ifndef IBN_CAMAC_FREQ_H
#define IBN_CAMAC_FREQ_H

#include "Module.h"
#include <unistd.h>
#include <cmath>

#include "../Time.h"

#include <iostream>
#include <bitset>

using namespace std;
namespace camac
{
  class BlockP0603    : public Module,  virtual public ModuleInventory
  {
    public:
      BlockP0603(void) : ModuleInventory("BlockP0603", "Frequencymeter", "BlockP0603") {};
      BlockP0603(const camac::Address &adr)	
        : Module(adr,"BlockP0603","Frequencymeter","P0603")
      {
      }

      ~BlockP0603(void){}

      unsigned read(void)
      {
        u32_t data;
        AF(0,3,&data);
        return ~data & 0x00ffffff;
      }

      u16_t read_key(void)
      {
        u16_t key;
        AF(0,0,&key);
        return key;
      }

      bool islam(void)	const 
      {
        unlockL();
        bool l= IsLam(0,8);
        lockL();
				return l;
      }

      bool status(void) const { return !islam(); }


      void wait_lam(double timeout=0) const
      {
        timeout = 15;
        //double dead_line = currenttime()+fabs(timeout);
        auto dead_line = std::chrono::steady_clock::now() + std::chrono::milliseconds(int64_t(timeout*1e3));
        while(!this->islam())
        {
          usleep(250000);
          //if ( timeout != 0  && currenttime() > dead_line)
          if ( timeout != 0  && std::chrono::steady_clock::now() > dead_line)
          {
            throw Error(GetType(), "Can't wait LAM", GetAddress());
          }
        }
      }


      unsigned reset(void)
      {
        u32_t data = read();
        AF(0,10);
        return data;
      }

      void lockL(void) const   { try	{ AF(0,24); } catch(camac::Module::Error){} }
      void unlockL(void)  const { try	{ AF(0,26); } catch(camac::Module::Error){} }
  }; //class BlockP0603

  class Freq : public ModuleInventory
  {
    double frequency;
    private:
    typedef BlockP0603 Block;

    Block  Low; //For low digit of Freq
    /* 24 бита, читаемые из первого блока есть шесть десятичных цифры
       это так называемый двоично-десятичный формат
       */
    Block  High; //For high digit and pow
    /*Здесь 8 первых бит - это мантиса двух десятичных цифр
      затем 4 бита - порядок в двоично-десятичном формате
      еще оди бит - это дополнительный знак порядка
      всего порядок может изменяться от 10e-19 10e+19
      Затем 4 бита - знак порядка, (может быть для разных измерений)
      Еще четыре бита  -- биты единиц измерений

*/

    enum MODE{ FREQ = 1, PERIOD = 2, RATIO = 4, SMTH = 8 };

    mutable MODE Mode;

    //Thid function convert data low and high to ten_digit
    // i -- number of digit
    u32_t  man(u32_t low, u32_t high, unsigned i)  const
    {
			auto to_hex_string = [](u32_t data) -> std::string
			{
				std::stringstream os;
				os << hex << data;
				return os.str();
			};
			auto bitset_to_string = [] (const std::bitset<24> & b)
			{
				std::stringstream os;
				os << b;
				return os.str();
			};
      u32_t  ten_digit;
      if(i < 6)
      {
        ten_digit = (low >> (4*i)) & 0xf;
        if(ten_digit>9) 
				{
					std::string error_string = "camac::Freq: Wrong data in low (24bit) mantisa: \n";
					error_string += to_hex_string(high) + " " + to_hex_string(low) + ": \n";
					error_string += bitset_to_string(std::bitset<24>(high)) + " " + bitset_to_string(std::bitset<24>(low));
					error_string += "\n in digit " + ::to_string(ten_digit) + " at position " + ::to_string(i);
					throw std::range_error(error_string);
				}
        return ten_digit;
      }
      if(i < 8)
      {
        ten_digit = (high >> (4*(i-6))) & 0xf;
        if(ten_digit>9) throw std::range_error("Frequencymeter-high: wrong data in mantisa "+to_string(ten_digit) );
        return ten_digit;
      }
      throw std::invalid_argument("Frequencymeter: invalid argument");
      return 0;
    }


    //Return mantisu of data (before parser)
    double mantisa(u32_t low, u32_t high) const
    {
      double data = 0;
      for(unsigned i = 0; i < 8; i++)
			{
        data += pow(10.,double(i))*man(low,high,i);
      }
      return data;
    }

    //returns power
    int power(u32_t p) const
    {
      u32_t p1 = (p >> 8)  & 0xf;
      u32_t p2 = (p >> 12) & 0x1;
      if(p1>9 || p2>9) throw std::range_error("Frequencymeter: Wrong data in power");
      return p1 + p2*10;
    }

    //return sign of data
    int sign(u32_t p)  const
    {
      u32_t sig = (p >> 13) & 0xf;
      u32_t mode = (p >> 17) & 0xf;
      Mode = static_cast<MODE>(mode);
      return 12-sig;
    }

    public:
    Freq(std::string name="unknown") : ModuleInventory(name, "Frequencymeter", "BlockP0603") {}

    Freq(const camac::Address &adr1, const camac::Address &adr2)
    {
      Low.Bind(adr1);
      High.Bind(adr2);
    }

    void Bind(const Address &low, const Address & high)
    {
      Low.Bind(low);
      High.Bind(high);
    }

    virtual ~Freq(void)	{}

    //Return the measured frequency in Hz
    double GetFrequency(void)
    {
      Wait();
      u32_t low = Low.read();
      u32_t high = High.read();
			//std::cout << "row data = " << hex << low << " " << hex << high << endl;
			//std::bitset<24> xl(low);
			//std::bitset<24> xh(high);
			//std::cout  << xl << " " << xh << std::endl;

			//std::cout << "mantisa = " << mantisa(low, high) << std::endl;
			//std::cout << "power = " <<   power(high) << std::endl;
      Reset();
      frequency=mantisa(low,high) * pow(10., double(sign(high)*power(high)));
      return frequency;
    }

    void Reset(void)
    {
      Low.reset();
      High.reset();
    }

    void Wait(void) const
    {
      High.wait_lam();
      Low.wait_lam();
    }

    double GetLastMeasuredFrequency(void) const { return frequency; }


    bool GetStatus(void) const   { return !Low.islam() && !High.islam(); }

  }; //end of class Freq
}//end  of namespace camac
#endif //IBN_CAMAC_FREQ_H
