#include "InitModules.h"
#include "PolarimeterConfig.h"
#include <boost/algorithm/string.hpp>
namespace camac
{
  bool get_status(boost::property_tree::ptree & pt)
  {
    auto value = pt.get_optional<std::string>("status");
    bool status = true;
    if(value)
    {
      std::string  status_str = value.get();
      boost::algorithm::to_lower(status_str);
      status = !(status_str == "off" || status_str == "0");
      status = (status_str == "on" || status_str == "1");
    }
    return status;
  }

	std::map <std::string,  std::shared_ptr<camac::ModuleInventory> > InitModules(boost::property_tree::ptree & tree,  bool isCamac)
	{
		std::map <std::string,  std::shared_ptr<camac::ModuleInventory> > Modules;
		for(auto & module_type_item : tree) //loop over block type
		{
			for(auto & module_subtype_item : module_type_item.second) //loop over block subtype
			{
				for(auto & module_name_item : module_subtype_item.second)
				{
					std::string name = module_name_item.first;
					std::string type = module_type_item.first;
					std::string subtype = module_subtype_item.first;
					auto  address_str = module_name_item.second.get<std::string> ("address");
					auto address = camac::Address(address_str.c_str());
					std::clog << LOG(VERBOSE_LOG) << type << " (" << subtype << ") " << name << "  " << address_str << " ";
					std::shared_ptr<camac::ModuleInventory> Module;
					//std::shared_ptr<camac::Module> Module;
          bool is_bind=true;
          bool check_x = module_name_item.second.get<int> ("checkX",true);
          //std::cout << name << "  "  << check_x  << std::endl;
          auto check_bind = [&](boost::property_tree::ptree & pti)
          {
            if(auto b = pti.get_optional<std::string>("bind"))
            {
              //change bind state of the block
              if(b.get() == "no" ) is_bind=false;
              if(b.get() == "yes") is_bind = true;
            }
          };
          check_bind(module_type_item.second);
          check_bind(module_subtype_item.second);
          check_bind(module_name_item.second);
          is_bind = isCamac && is_bind;
					if(type == "Timer")
					{
						if(subtype == "C2602")
						{
							camac::Timer * module = new camac::Timer(name.c_str());
							if(is_bind) module->Bind(address);
              module->CheckX(check_x);
							Module.reset(module);
						}
					}
					if(type == "Vvi")
					{
            bool keep_previose_state=false;
            auto all_vvi_status = module_type_item.second.get_optional<std::string>("status");
            if(all_vvi_status)
            {
              if(all_vvi_status.get() == "keep")
              {
                keep_previose_state = true;
              }
            }
						if(subtype == "B0308")
						{
							camac::Vvi_B0308 * module = new camac::Vvi_B0308(name.c_str());
              auto value = module_name_item.second.get_optional<std::string>("status");
              std::string status;
              if(is_bind) module->Bind(address);
              module->CheckX(check_x);
              if(value)
              {
                status = value.get();
                boost::algorithm::to_lower(status);
                boost::algorithm::to_lower(PCFG.vvi);
                if(!keep_previose_state)
                {
                  if(status== "on"  && PCFG.vvi == "on") module->On();
                  if(status== "off") module->Off();
                }
              }
							auto  V = module_name_item.second.get<unsigned> ("volt");
							module->SetVolts(V);
							Module.reset(module);
							std::clog << LOG(VERBOSE_LOG) << " V=" << V;
						}
					}
					if(type == "DelayLine")
					{
						if(subtype == "L0606_OLD")
						{
							auto  delay = module_name_item.second.get<unsigned> ("delay");
							auto  step = module_name_item.second.get<unsigned> ("step");
							auto module = new camac::DelayLine_L0606_OLD(name);
							if(is_bind) module->Bind(address, step,  delay);
              module->CheckX(check_x);
							module->SetDelay(delay);
							Module.reset(module);
						}
						if(subtype == "L0606")
						{
							auto  delay = module_name_item.second.get<unsigned> ("delay");
							auto  step = module_name_item.second.get<unsigned> ("step");
							auto module = new camac::DelayLine_L0606(name);
							if(is_bind) module->Bind(address,  step, delay);
              module->CheckX(check_x);
							std::clog << LOG(VERBOSE_LOG) << " delay = " << delay << " ns,  step = " << step <<  " ns ";
							Module.reset(module);
						}
					}
					if(type == "Diskriminator")
					{
						if(subtype == "D0302")
						{
							auto  threshold = module_name_item.second.get<unsigned> ("threshold");
							auto module = new camac::DiskrD0302(name);
							if(is_bind) module->Bind(address, threshold);
              module->CheckX(check_x);
							std::clog << LOG(VERBOSE_LOG) << " threshold = " << threshold << " mV ";
							Module.reset(module);
						}
					}
					if(type == "Counter")
					{
						if(subtype == "C0301")
						{
							auto module = new camac::CounterC0303(name);
							//if(is_bind && get_status(module_name_item.second)) module->Bind(address);
							if(is_bind) module->Bind(address);
              module->CheckX(check_x);
							Module.reset(module);
						}
					}
					if(type == "Concurence")
					{
						if(subtype == "L0306")
						{
							auto  mask = module_name_item.second.get<std::string> ("mask");
							std::stringstream ss(mask);
							std::string item;
							std::list<unsigned> chan_list;
							std::clog << LOG(VERBOSE_LOG) << " mask = " << mask;
							while(std::getline(ss,item, ':'))
							{
								chan_list.push_back(stoi(item));
							}
							auto module = new camac::Concur_L0306(name);
							if(is_bind) module->Bind(address, chan_list);
              module->CheckX(check_x);
							Module.reset(module);
						}
					}
					if(type == "Attenuator")
					{
						if(subtype == "A0608")
						{
							auto  attenuation = module_name_item.second.get<unsigned> ("attenuation");
							auto module = new camac::Attenuator_A0608(name);
							if(is_bind) module->Bind(address, attenuation);
              module->CheckX(check_x);
							module->SetAttenuation(attenuation);
							Module.reset((camac::Module*)module);
							std::clog << LOG(VERBOSE_LOG) << " attenuation = " << attenuation << " dB";
						}
						if(subtype == "A0608_3")
						{
							auto  attenuation = module_name_item.second.get<unsigned> ("attenuation");
							auto module = new camac::Attenuator_A0608_3(name);
							if(is_bind) module->Bind(address, attenuation);
              module->CheckX(check_x);
							module->SetAttenuation(attenuation);
							Module.reset(module);
							std::clog << LOG(VERBOSE_LOG) << " attenuation = " << attenuation << " dB";
						}
					}
					if(type == "Generator")
					{
						if(subtype == "DDS300")
						{
							auto module = new camac::Generator_DDS300(name);
							if(is_bind) module->Bind(address);
              module->CheckX(check_x);
							Module.reset(module);
						}
						if(subtype == "DDS500")
						{
							auto module = new camac::Generator_DDS500(name);
							if(is_bind) module->Bind(address);
              module->CheckX(check_x);
							Module.reset(module);
						}
					}
					if(subtype == "L0102" && type=="Kvch")
					{
						auto module = new camac::Kvch(name);
						int output_channel = 0;
						for(auto & com : module_name_item.second.get_child("commutation"))
						{
							int input_channel = com.second.get<int>("");
							module->Commutate(input_channel, output_channel++);
						}
						std::clog << LOG(2) << *module << std::endl;
						Module.reset(module);
					}
					if(subtype == "BlockPO603" && type == "Frequencymeter")
					{
						auto address2 = module_name_item.second.get<std::string> ("address2");
						auto module = new camac::Freq(name);
						if(is_bind) module->Bind(address, address2.c_str());
            module->CheckX(check_x);
						Module.reset(module);
					}
					std::clog << LOG(VERBOSE_LOG) << std::endl;
					if(Module) Modules[type+"."+name] = Module;
				}
			}
		}
		return Modules;
	}
}
