
#pragma once
#include "Module.h"

#include <vector>
namespace camac 
{
  class Kvch : public Module, virtual public ModuleInventory 
	{
		enum Type_t
		{
			TYPE_NONE   = 0x0, 
			TYPE_2_TO_1 = 0x1, 
			TYPE_8_TO_1 = 0x2, 
			TYPE_4_TO_1 = 0x3
		} fType ;

		mutable std::vector<u16_t> fState;

    public:
    Kvch(std::string name = "None") : ModuleInventory(name,  "Kvch", "L0102") {}
    Kvch(std::string name,  const camac::Address adr) :	ModuleInventory(name.c_str(), "Kvch", "L0102")
		{
			Bind(adr);
		}

    void Bind(const camac::Address & a) 
		{ 
			std::clog << LOG(10) << "camac::Kvch::Bind BEGIN" << std::endl;
			Module::Bind(a);
			std::clog << LOG(11) << "After Module bind" << std::endl;
			setType();
			std::clog << LOG(11) << "After set type" << std::endl;
			std::clog << LOG(10) << "camac::Kvch::Bind END" << std::endl;
		}
    ~Kvch(){}

		size_t size(void)  const
		{
			return fState.size();
		};


		int GetState(int output_channel) const
		{
			if(output_channel >= fState.size()) return -1; //wrong output channel config
			ReadState();
			return fState[output_channel];
		}

		void Commutate(int input_channel,  int output_channel=-1)
		{
			if(output_channel >= fState.size()) return; //wrong output channel config
			ReadState();
			if(output_channel<0)
			{
				for(int i=0;i<fState.size();i++)
				{
					fState[i] = input_channel;
				}
			}
			else 
			{
				fState[output_channel] = input_channel;
			}
			WriteState();
		}


		void Print(void) //print the state of commutator
		{
		}

		private:
		u16_t read(void) const
		{
			u16_t data=0;
			AF(0, 0, &data);
			return data;
		}

    void write(u16_t data)
		{
			AF(0, 16, &data);
    }

		void setType(void)
		{
			fType = (Type_t)(((read() & 0xfff) >> 4) & 0x3);
			switch(fType)
			{
				case TYPE_NONE:
					//std::cout << "Type of NONE : code = " << TYPE_NONE << std::endl;
					fState.resize(0);
					break;
				case TYPE_2_TO_1:
					//std::cout << "Type of 2 to 1 : code = " << TYPE_2_TO_1 << std::endl;
					fState.resize(4);
					ModuleInventory::SetSubType("Type_4x2_to_1");
					break;
				case TYPE_8_TO_1:
					//std::cout << "Type of 8 to 1 : code = " << TYPE_8_TO_1 << std::endl;
					fState.resize(1);
					ModuleInventory::SetSubType("Type_1x8_to_1");
					break;
				case TYPE_4_TO_1:
					//std::cout << "Type of 4 to 1 : code  = " << TYPE_4_TO_1 << std::endl;
					fState.resize(2);
					ModuleInventory::SetSubType("Type_2x4_to_1");
					break;
			}
		}


		u16_t read_state(void) const //read raw state
		{
			return 0xf & read();
		}

		//read the state of commutator accroding its type
		//result will bi stored in fState vector
		void ReadState(void) const
		{
			u16_t state_code = read_state();
			switch(fType)
			{
				case TYPE_8_TO_1:
					fState[0] = state_code & 0x7;
					fState[1] = 0;
					fState[2] = 0;
					fState[3] = 0;
					break;
				case TYPE_4_TO_1:
					fState[0] = state_code & 0x3;
					fState[1] = (state_code & (0x3 << 2))>>2;
					fState[2] = 0;
					fState[3] = 0;
					break;
				case TYPE_2_TO_1:
					for(int i=0;i<4;i++)
					{
						fState[i]  = (state_code & (0x1 << i)) >> i;
					}
					break;
				default:
				case TYPE_NONE:
					fState[0] =0;
					fState[1] =0;
					fState[2] =0;
					fState[3] =0;
					break;
			}
		}

		void WriteState(void)
		{
			u16_t state_code = 0;
			switch(fType)
			{
				case TYPE_8_TO_1:
					state_code = fState[0] & 0x7;
					break;
				case TYPE_4_TO_1:
					fState[0] &= 0x3;
					fState[1] &= 0x3;
					state_code = fState[0] |  ( fState[1] << 2) ; 
					break;
				case TYPE_2_TO_1:
					for(int i=0;i<4;i++)
					{
						fState[i] &= 0x1;
						state_code = state_code | (fState[i] << i);
					}
					break;
				default:
				case TYPE_NONE:
					break;
			}
			write(state_code);
		}

  };
}

inline std::ostream & operator<<(std::ostream & os,  const camac::Kvch & kvch)
{
	os << "Module " << kvch.GetName() << ",  type " << kvch.GetType() << ",  subtype " << kvch.GetSubType() << ",  commutation state: "  << std::endl;
	for(int i=0;i<kvch.size();++i)
	{
		os << "   " << kvch.GetState(i) << " -> " << i << std::endl;
	}
	return os;
}



