/*******************************************************************
  This file includes declaration of camac station class, which i will use
  for every my camac modules
 **************************************************************************************/
#ifndef IBN_CAMAC_MODULE_H
#define IBN_CAMAC_MODULE_H

#include "defs.h"
#include <cstdio>
#include <cstring>
#include <cerrno>

#include <stdexcept>


#define PCI_CAMAC

#ifdef ISA_CAMAC
//Camac library
//#include<ppi2camac.h>
#include "ppi2camac.h"
#endif

#ifdef PCI_CAMAC
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>       
#include <unistd.h>
#include "lsi6camac.h"
#endif


#include <sstream>

#include <string>
#include <list>
#include <functional>
#include <algorithm>

#include "ModuleInventory.h"

#include "Log.h"


namespace camac
{
  class Crate;
  class Address;
  class Module;
  static std::list<Crate*> CRATELIST;

  /* operate with camac crate device */
  class	Crate
  {
    public:
      Crate(int iface, int cr);
      ~Crate(void);
      class Error;
      int GetFd()	const {return fd;}
      int GetInterface()	const {return interface;}
      int GetCrate() 	const {return crate;}
      inline void C(void);
      inline void Z(void);
    private:
      const int interface; //interface number
      const int crate; //crate number
      int fd; //crate File Deskriptor
  };

  //Description of camac module address
  struct Address
  {
    //		string name;    //Mosule name, any you want
    int interface;  //# camac interface in PC,s	
    int crate;      // creates # in interface
    int station;    // position # in crate
    int channel;    // subaddress #
    Address(void);
    Address(const Address &);
    Address(std::string, char);
    Address(int, int, int, int=-1);
    Address(const char *, char delim =  ':' );
    void Fill(const char *, char delim = ':');
    std::string str(void) const;
    ~Address(void){}
  };

  //Basic class for all camac blocks
  class Module :  public virtual ModuleInventory
  {
    public:     
      Module(std::string n="unknown", std::string t="unknown", std::string st="unknown") 
        : ModuleInventory::ModuleInventory(n,t,st)
      {				
        isinit = false;
      }

      Module(const Address &, std::string ="unknown", std::string ="unknown",std::string ="unknown");

      virtual ~Module(void)
			{
				std::clog << LOG(DEBUG_LOG) << "~camac::Module(void)" << GetName() << " " << GetType() << std::endl;
			}

      class Error;

      void Bind(const Address &);

      Address GetAddress(void)   const { return adr;} //Get modeule address
      int GetInterface()    	const {return adr.interface; } //Get interface number
      int	GetCrate() const { return adr.crate; } //Get crete number in interface
      int GetStation()    const {return adr.station;} //Get position numb,er in crate
      int GetChannel()    const {return adr.channel;} //Get channel number in create
      inline void C();
      inline void Z();
      //LAM request
      inline bool IsLam(void) const ;
      inline double WaitLam(double sec) const ;

      inline bool IsLam(int, int ) const ;
      inline void WaitLam(int , int ) const;

      inline void AF(int, int) const;
      inline void AF(int, int, u16_t *) const;
      inline void AF(int, int, u32_t *) const;
      inline void AFD(int, int, u32_t) const;

      inline void BlockAF(int, int, u16_t  *, int);
      inline void BlockAF(int, int, u32_t  *, int);
      inline void SetAndLockI(void);
      inline void ClearAndLockI(void);
      inline void UnlockI(void);
      inline void SetI(void);
      inline void ClearI(void);
      inline bool GetI(void) const;
      inline bool IsInit(void) const;
      static bool XQ_ERROR_EXCEPTION;

    private:	
      bool isinit;
      Address adr; //адресс
      Crate * crate;  //Указатель на Crate
      //Проверка на ошибку
      void Check_Error(const std::string &func_name, int xq, int err) const;
  }; //end of class module

  /* implementation */


  class Crate::Error  :  public std::runtime_error
  {
    const int Errno;
    public:
    Error(int err, int iface=0, int channel=0):	
      std::runtime_error(std::string(strerror(err)) + ": interface: " + std::to_string(iface) + ", channel " + std::to_string(channel)), Errno(err) {}
    int GetErrno()	const { return Errno; }
  };

  inline Crate::Crate(int iface, int cr)	: 
    interface(iface),  	//camac interface (card number in PC) initialisation
    crate(cr) 					//camac crate initialisation
  {
		std::string str;
#ifdef ISA_CAMAC
    const std::string device_name = "/dev/ppi";
    char c_iface[2] = {'a','b'};
    char c_crate[6] = {'0','1','2','3','4','5'};
    str = device_name + c_iface[interface] + c_crate[crate];
#endif
#ifdef PCI_CAMAC
    std::ostringstream s;
    s << "/dev/lsi6card"<<interface+1<<"channel"<<crate+1;
    str = s.str();
#endif
    fd = open(str.c_str(),O_RDWR);
    if( fd < 0 ) throw Error(errno, interface, crate);
    //Не знаю нужно ли здесь делать такие обнуление и сброс крейта.
    CAM_C(fd);
    CAM_Z(fd);
  }

  inline void Crate::Z(void)
  {
    int rc=CAM_Z(GetFd());
    if(rc==-1) throw Error(errno);
  }

  inline void Crate::C(void)
  {
    int rc=CAM_C(GetFd());
    if(rc==-1) throw Error(errno);
  }

  class Crate_eq : public std::unary_function<Crate,bool>
  {
    const int interface;
    const int crate;
    public:
    explicit Crate_eq(int i, int c): interface(i), crate(c) {}
    bool operator()( Crate * & cr_ptr) const {
      return  cr_ptr->GetInterface() == interface  && cr_ptr->GetCrate() == crate;
    }
  };
  inline Crate::~Crate(void) { close(fd);	}	//закрытие устройства


  // struct Address implementation
  inline Address::Address(void)
  {
    //		name = "none";
    interface = -1; //not real;
    crate = -1; 	//not real;
    station = -1; 	//not real;
    channel = -1; 	//not real;
  }

  inline Address::Address(const Address & a)
  {
    //		name = a.name;
    interface = a.interface;
    crate = a.crate;
    station = a.station;
    channel = a.channel;
  }


  inline Address::Address(int i, int c, int s, int ch )
  {
    //		name=n;
    interface = i;
    crate = c;
    station = s;
    channel = ch;
  }

  inline Address::Address(const char * str, char delim )	{ Fill(str,delim); }
  inline Address::Address(std::string str , char delim )           { Fill(str.c_str(),delim); }
  inline void Address::Fill(const char * str, char delim )
  {
    //		const unsigned  MAX_BUFSIZE = 128;
    //		char buf[MAX_BUFSIZE];
    char tmp;
    //		istrstream is(str);
    std::istringstream is(str);
    //		is.getline(buf,MAX_BUFSIZE, delim);
    //		name = buf;
    is >> interface >> tmp >> crate >> tmp>> station >> tmp >> channel;
  }

  inline std::string Address::str(void) const
  {
    std::ostringstream os;
    os << interface << "/"<<crate << "/"<<station << "/"<< channel;
    return os.str();
  }



  // Class Module implementation
  static Crate * getcrate(std::list<Crate*> &l, const Address & adr)
  {
    std::list<Crate*>::iterator p = find_if(l.begin(), l.end(),Crate_eq(adr.interface, adr.crate));
    if(p==l.end())
    {

      l.push_back(new Crate(adr.interface, adr.crate));
      p = --l.end();
    }
    return *p;
  };

  inline Module::Module(const Address &addr, std::string n, std::string t, std::string st)
    : ModuleInventory(n, t, st)
  {
    isinit = true;
    adr = addr;
    crate = getcrate(CRATELIST,addr);
  }

  class Module::Error : public std::runtime_error
  {
    const std::string type;
    const camac::Address Adr;
    const std::string function;
    const int xq;
    const bool x,q;
    static const int X = 2;
    static const int Q = 1;


    public:

    Error(const std::string &tp, const std::string &func, const Address &adr,const int rc = 3):
      runtime_error("Camac error: "),
      type(tp),
      Adr(adr),
      function(func),
      xq(rc), 
      x( (rc & X) ==  0x2),
      q( (rc & Q) ==  0x1)   {}

    std::string error(void)
    {
      std::ostringstream os;
      if (!x) os<<"camac module ("<< Adr.str()<<")" << "NOT_X ( xq= " << xq << " )" << std::ends;
      return os.str();
    }

    static std::string straddress(const camac::Address &adr)
    {
      char str[100];
      sprintf(str,
          "interface %d, crate %d, station %d",
          adr.interface,adr.crate, adr.station);
      return str;
    }


    ~Error(void) throw() 
    {
    }	

    bool getX() const { return x; }
    bool getQ() const { return q; }

    std::string what(void)   
    {
      return runtime_error::what() + type + " " + straddress(Adr) +	" in " + function + " : " + error();
    }
  };


  //Проверка на ошибку
  inline void Module::Check_Error(const std::string & func, int xq, int err)  const
  {
#ifdef PCI_CAMAC
    //Когда работаем с PCI платной то ответы инвертированы: то есть Q=0 это ответ ДА.
    xq = 3 & (~xq);
#endif
    if(xq==-1)	throw Crate::Error(err);
    bool X = bool(xq & 2);
    std::string module_id = std::string(GetType())+"/"+std::string(GetSubType())+"/"+std::string(GetName());
    //std::cout << "Module::Check_Error checkX=" << check_X << " " << adr.str() << " " << module_id << " " << func << std::endl;
    if(!X) 
    {
      std::string module_id = std::string(GetType())+"/"+std::string(GetSubType())+"/"+std::string(GetName());
      if(XQ_ERROR_EXCEPTION && check_X)
      {
        throw Module::Error(module_id, func, adr,xq);
      }
      else
      {
        std::cerr << LOG(1) << "ERROR: Module " << module_id << "  " << func << "  " << adr.str() <<  "  xq=" <<  xq << std::endl;
      }
    }
  }


  inline bool Module::IsInit(void)   const
  {
    return isinit;
  }

  // NAF инструкция без обмена данных
  inline void Module::AF(int A, int F)  const
  {

    if(!IsInit()) return;
    int xq = CAM(crate->GetFd(), NAF(adr.station,A,F), 0);
    Check_Error("AF()",xq,errno);
  }

  // NAF инструкция с обменом 16-битными данными.
  inline void Module::AF(int A, int F,u16_t *DATA) const
  {
    if(!IsInit()) return;
    u32_t data = static_cast<u32_t>(*DATA);

    int xq = CAM(crate->GetFd(), NAF(adr.station,A,F), &data);
    *DATA = static_cast<u16_t>(data);
    Check_Error("AF(16bit)",xq,errno);
  }

  // NAF инструкция с обменом 24-битными данными.
  inline void Module::AF(int A, int F, u32_t *DATA) const
  {

    if(!IsInit()) return;
    ul32_t data = static_cast<ul32_t>(*DATA);
    int xq = CAM24(crate->GetFd(), NAF(adr.station,A,F), &data);
    *DATA = (u32_t)data;
    Check_Error("AF(24bit)",xq,errno);
  }


  inline void Module::AFD(int A, int F, u32_t data) const 
  {
    AF(A,F,&data);
  }



  inline void  Module::Z(void)
  {
    if(!IsInit()) return;
    CAM_Z(crate->GetFd());
  }

  inline void Module::C(void)
  {
    if(!IsInit()) return;
    CAM_C(crate->GetFd());
  }

  /*
   *--------------------------------------------------------------------------------------
   *       Class:  Module
   *      Method:  WaitLam
   * Description:  Wait the LAM request
   * timeout > 0 - wait LAM during timeout seconds. If LAM happens it returns remain of timeout.  
   *		     Zero value is possible. 
   * timeout = 0 - check the LAM and return 0 if LAM happens or -1 otherwise.
   * timeout < 0 - wait LAM forever.
   * If no LAM during timeout ETIME error is returned.
   * Waiting for timeout might be interrupt by unmasking signal. Corresponding error is returned.
   *--------------------------------------------------------------------------------------
   */
  inline double Module::WaitLam(double timeout)  const 
  {
    //ATTENTION architecture dependent part timeout*100 - is number of ticks 
    //for x86 architecture 1tick = 10ms
    static double tick_time=1e-2;
    int rc = CAM_LWAIT(crate->GetFd(), adr.station, int(timeout/tick_time));
    return double(rc)*1e-2;
  }


  /*
   *--------------------------------------------------------------------------------------
   *       Class:  Module
   *      Method:  IsLam
   * Description:  Check the LAM and return true if LAM or false otherwise
   *--------------------------------------------------------------------------------------
   */
  inline bool Module::IsLam(void)  const  
  {

    int rc = CAM_LWAIT(crate->GetFd(), adr.station, 0);
    if(rc==0) return true;
    return false;
  }



  inline bool Module::IsLam(int A, int F)  const
  {
    if(!IsInit()) return false;
    int rc = CAM(crate->GetFd(), NAF(adr.station,A,F), 0);
#ifdef PCI_CAMAC
    rc = 0x1 & ~rc;
#endif 
    return rc & 0x1;
  }

  inline void Module::WaitLam(int A, int F)  const
  {
    if(!IsInit()) return;
    while(!IsLam(A,F)) usleep(10000);
  }

  //Установка бита I
  inline void Module::SetI(void)
  {
    if(!IsInit()) return;
    CAM_ION(crate->GetFd());
  }

  //Сброс бита I
  inline void Module::ClearI()
  {
    if(!IsInit()) return;
    CAM_IOFF(crate->GetFd());
  }

  //Получение бита I
  inline bool Module::GetI(void)	const
  {
    if(!IsInit()) return false;
    int rc = CAM_I(crate->GetFd());
    if(rc == 0 || rc ==1) return rc;
    Check_Error("GetI",rc,errno);
    return 0;
  }

  inline void Module::Bind(const Address & a)
  {
    if(!isinit)
    {
      isinit = true;
      adr = a;
      crate = getcrate(CRATELIST,adr);
    }
  }

} //end pf namespace camac


#endif //IBN_CAMAC_MODELU
