#ifndef IBN_CAMAC_TIMER_H
#define IBN_CAMAC_TIMER_H

#include "Module.h"

#include <unistd.h>
#include <mutex>
#include "signal.h"
#include <condition_variable>

#include <chrono>

namespace camac
{


  class Timer_base	:  public Module, virtual public ModuleInventory   
  {
    static constexpr double Max  = 655.35; //s    //1310.710; // 2^16 - 1=65535
    static constexpr double Step = 0.01;   //s    //0.01; //ms
    public:

    double wrtitten_time;

    Timer_base(void) 
    {
      wrtitten_time=0;
    }

    Timer_base(camac::Address adr) : Module(adr)
    {
      wrtitten_time=0;
    }

    virtual ~Timer_base(void) {}

    void set() 
    { 
      Z(); 
    }

    //Reset internal counter of the timer
    void reset() 
    { 
      C();  
    }

    // Register preset (How much will count)
    void write(double time) 
    {
      if(time>Max) throw std::out_of_range(std::string(GetType()) + ": Bad time");
      wrtitten_time = time;
      u16_t data = u16_t(time/Step);
      AF(0,17,&data);
    }

    void write_zero(void)
    { 
      AF(0,11); 
    }

    double  read(void) const 
    {
      u16_t data;
      AF(0,1,&data);
      return double(data)*Step;
    }

    //Работа Таймера

    //L-Reset and start the Timer_base
    //Сброс L и пуск Таймера
    //Но кажеться лам по старту не збрасывается.
    void start()
    {
      AF(0,10);
    }

    //read current value
    double  read_count()   const 
    {
      u16_t data;
      AF(0,0,&data);
      return double(data)*Step;
    }

    void reset_count()   
    {
      AF(0,9); 
    }

    //проверка лам зароса.
    bool islam()	const 
    {
      return IsLam();
    }

  }; //class Timer_base


  class Timer	:  public Timer_base, virtual public ModuleInventory  
  {
    static constexpr double wait_delay = 1e-3; //sec
    double Time; //in seconds
		typedef std::chrono::steady_clock  clock;//define monotonic clock
		clock::time_point begin_time;
		mutable volatile bool is_wait;
    public:

    Timer(const char* nm="unknown"):ModuleInventory(nm, "Timer", "C2602"), Time(0){}

    Timer(const char* nm, camac::Address adr,  double time = 0): ModuleInventory(nm, "Timer", "C2602"), Timer_base(adr), Time(time)
    {}

    ~Timer(void) 
    {
      Timer_base::set();
      Timer_base::reset();
      Timer_base::write(Time);
    }

    //Сброс таймера
    void Reset(void)
    { 
      Timer_base::reset(); 
    }

    //Получить время работы таймера
    double GetTime() const 
    { 
      return Timer_base::read(); 
    }

    double GetWorkTime() const 
    { 
      return Time; 
    }

    void SetTime(double time)
    {
      Time=time;
      Reset();
      Timer_base::write(time);
    }

    //Запуск таймера на время time
    void Start(double time)
    {
			SetTime(time);
      Timer_base::start();
			begin_time = clock::now();
    }

    void Start(void)
    {
      Reset();
      Timer_base::start();
			begin_time = clock::now();
    }

    //return true if timer is working
    bool GetStatus() const 
    {	
      return !IsLam(); 
    }

    //Ожидает конца работы таймера
    void Wait(void) const 
    {
      if(IsInit())
			{
        double t=WaitLam(Time*3);
        if(t<0)
				{
          std::cerr << "ERROR: No LAM from timer" << std::endl;
        }
      } 
			else 
			{
        usleep(unsigned(Time*1e6));
      }
    }


    //Ожидает конца работы таймера или пока параметр iswait не станет ложным
    void Wait(const volatile bool & is_external_wait) const 
    {
			is_wait=true;
      if(IsInit()) 
			{
        while(is_external_wait && is_wait && !islam() ) 
				{
					usleep(unsigned(wait_delay*1e6));
				}
      } 
			else 
			{
				clock::duration T = std::chrono::duration_cast<clock::duration>( std::chrono::duration<double>(Time));
        while(is_external_wait && is_wait && clock::now() - begin_time < T )  
				{
					usleep(unsigned(wait_delay*1e6));
				}
      }
    }

		void Stop(void)
		{
			is_wait = false;
			//raise(SIGUSR1);
		}



    void Bind(Address adr, double time = 0)
    {
      Module::Bind(adr);
      Timer_base::write(time);
    }

  }; //class Timer

} //namespace modules

#endif //IBN_TIMER_H
