#ifndef IBN_VVI_H
#define IBN_VVI_H

#include "defs.h"
#include "Module.h"
#include "unistd.h"
#include "ModuleInventory.h"

#include "Log.h"
namespace camac
{
	class Vvi_base : protected  Module, virtual public ModuleInventory 
	{
		static const int Vmax = 4000;
		public:
		Vvi_base(void){}
		Vvi_base(const Address adr)  :
			Module(adr)  {}
		virtual ~Vvi_base(void) {}
		void on(int Channel) 	{ AF(Channel,26);}
		void off(int Channel)	{ AF(Channel,24);}
		void write(int Channel, int volt)
		{
			if( (volt > Vmax) ||  (volt < 0) ) throw std::out_of_range(std::string(GetType()) + ": Volts is out of range");
			u16_t v = static_cast<u16_t>(volt);
			AF(Channel,16, &v);
		}

		unsigned int read(int Channel) const 
		{
			u16_t volt;
			AF(Channel,0,&volt);
			return static_cast<unsigned>(volt & 0xfff);
		}

		void reset(int Channel){ AF(Channel,10);}

		bool status(int Channel) const 
		{
			u16_t volt;
			AF(Channel,0, &volt);
			return (bool)((volt >> 12)&0x1);
		}

		bool ison(int Channel) const 
		{
			u16_t volt;
			AF(Channel,0, &volt);
			return (bool)((volt >> 12)&0x1);
		}
    bool is_overload(int Channel) const
    {
			u16_t volt;
			AF(Channel,0, &volt);
			return (bool)((volt >> 13)&0x1);
    }
    bool is_current_overload(int Channel) const
    {
			u16_t volt;
			AF(Channel,0, &volt);
			return (bool)((volt >> 14)&0x1);
    }
	}; //class Vvi


	class Vvi_B0308 : private Vvi_base, virtual public ModuleInventory 
	{
		const int max_volts;
		public:
		Address GetAddress(void) const  { return Vvi_base::GetAddress();}
		Vvi_B0308(const char* Name, const camac::Address addr, unsigned int _max_volts_ = 2500) :
			ModuleInventory(Name,"Vvi","B0308"),
			Vvi_base(addr),
			max_volts(_max_volts_)
		{}

		Vvi_B0308(const char* Name, unsigned int _max_volts_ = 2500) : ModuleInventory(Name,"Vvi","B0308"),
		max_volts(_max_volts_)
		{}
		virtual ~Vvi_B0308(void){std::clog << LOG(DEBUG_LOG) << "~Vvi_B0308(void)" << std::endl;} 

		void On(void)	
		{
			Vvi_base::on(GetChannel());
		}
		void Off(void)	
		{
			Vvi_base::off(GetChannel());
		}

		unsigned int GetVolts(void) const
		{
			return Vvi_base::read(GetChannel());
		}

		void SetVolts(int volts)
		{
			if( volts <= max_volts) Vvi_base::write(GetChannel(),volts);
			else throw std::out_of_range(std::string(GetType()) + ": Volts is out of range");
		}
		void ResetOverload(void)    {	Vvi_base::reset(GetChannel());  }
		bool GetStatus(void) const  {   return Vvi_base::status(GetChannel());  }
		bool IsOn(void) const { return Vvi_base::ison(GetChannel()); }
		bool IsOverload(void) const { return Vvi_base::is_overload(GetChannel()); }
		bool IsCurrentOverload(void) const { return Vvi_base::is_current_overload(GetChannel()); }
		void Bind(const Address & adr) { Module::Bind(adr);}
	};
	typedef Vvi_B0308 Vvi;
} //namespace modules
#endif //IBN_VVI_H

