/*
 * =====================================================================================
 *
 *       Filename:  Array.cpp
 *
 *    Description:  Implementation of data::Array
 *
 *        Version:  1.0
 *        Created:  07/13/2010 01:53:10 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#include <assert.h>
#include "Array.h"
#include <iostream>
namespace data
{

  void Array::push_back(Value_t v, Value_t e)
  {
    value_vector.push_back(v);
    error_vector.push_back(e);
  }
  void Array::push_back(const ibn::valer<Value_t>  & pv)
  {
    push_back(pv.value,pv.error);
  }

  void Array::push_back(const ibn::valer<Value_t>::const_reference & pv)
  {
    push_back(pv.value,pv.error);
  }

  void Array::push_back(const ibn::valer<Value_t>::reference & pv)
  {
    push_back(pv.value,pv.error);
  }

  void Array::reserve(size_t s)
  {
    value_vector.reserve(s);
    error_vector.reserve(s);
  }

  void Array::resize(size_t s)
  {
    value_vector.resize(s);
    error_vector.resize(s);
  }

  void Array::clear(void)
  {
    value_vector.clear();
    error_vector.clear();
  }


  ibn::valer<Array::Value_t>::reference   Array::operator[](size_t i)
  {
		if(i>=size())
		{
			std::cout << "ERROR i = " << i << "  size=" << size() << std::endl;
		}
    assert(i<size());
    return ibn::valer<Array::Value_t>::reference(value_vector[i],error_vector[i]);
  }

  const ibn::valer<Array::Value_t>::const_reference   Array::operator[](size_t i) const
  {
		if(i>=size())
		{
			std::cout << "ERROR i = " << i << "  size=" << size() << std::endl;
		}
    assert(i<size());
    return ibn::valer<Array::Value_t>::const_reference(value_vector[i],error_vector[i]);
  }

  ibn::valer<Array::Value_t>::reference Array::front(void)
  {
    return ibn::valer<Array::Value_t>::reference(value_vector.front(), error_vector.front());
  }

  const ibn::valer<Array::Value_t>::const_reference Array::front(void) const 
  {
    return ibn::valer<Array::Value_t>::const_reference(value_vector.front(), error_vector.front());
  }

  ibn::valer<Array::Value_t>::reference Array::back(void)
  {
    return ibn::valer<Array::Value_t>::reference(value_vector.back(), error_vector.back());
  }

  const ibn::valer<Array::Value_t>::const_reference Array::back(void) const 
  {
    return ibn::valer<Array::Value_t>::const_reference(value_vector.back(), error_vector.back());
  }

  void Array::set(unsigned point, const ibn::valer<Value_t> & x)
  {
    if(size()<=point)
    {
      for(unsigned i = size(); i<point;i++) push_back(0);
      push_back(x);
    }
    else 
    {
      (*this)[point]=x;
    }
  }
}

