/*
 * =====================================================================================
 *
 *       Filename:  Array.h
 *
 *    Description:  data array
 *
 *        Version:  1.0
 *        Created:  07/13/2010 01:43:23 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_DATA_ARRAY_H
#define IBN_DATA_ARRAY_H
#include <vector>

#include "ibn/valer.h"

namespace data
{
  using namespace std;
  class Array 
  {
    public:
    typedef double Value_t;
    typedef std::vector <Value_t> Vector_t;
    Array(void){};
    Array(const Array & pa): value_vector(pa.value_vector), error_vector(pa.error_vector) {};
    Array(size_t n) : value_vector(n), error_vector(n) {};
    Array(size_t n, const ibn::valer<Value_t> & v ) : value_vector(n,v.value), error_vector(n,v.error) {}
    Array(size_t n, double v, double e  ) : value_vector(n,v), error_vector(n,e) {}
    virtual ~Array(void){}
    size_t size(void) const { return value_vector.size(); }
    void push_back(Value_t, Value_t); 
    void push_back(const ibn::valer<Value_t> &);
    void push_back(const ibn::valer<Value_t>::reference & pv);
    void push_back(const ibn::valer<Value_t>::const_reference & pv);
    void reserve(size_t); //reserve the memory 
    void resize(size_t);  //resize arrrays
    void clear(void);
    ibn::valer<Value_t>::reference operator[](size_t);  
    const ibn::valer<Value_t>::const_reference operator[](size_t) const ; 
    Value_t * value(size_t i=0)  { return &value_vector[i]; }
    Value_t * error(size_t i=0)  { return &error_vector[i]; }
    const Value_t * value(size_t i=0) const { return &value_vector[i]; }
    const Value_t * error(size_t i=0) const  { return &error_vector[i]; }
    Vector_t &vvalue(void) { return value_vector;}
    Vector_t &verror(void) { return error_vector;}
    const Vector_t &vvalue(void) const { return value_vector;}
    const Vector_t &verror(void) const { return error_vector;}
    ibn::valer<Value_t>::reference back(void);
    ibn::valer<Value_t>::reference front(void);
    const ibn::valer<Value_t>::const_reference back(void) const ;
    const ibn::valer<Value_t>::const_reference front(void) const;
    void  set(unsigned point, const ibn::valer<Value_t> & x);
    virtual void Update(void){};
    private:
    Vector_t value_vector, error_vector;
  };
} //end of namespace data

#endif 

