/*
 * =====================================================================================
 *
 *       Filename:  ArraySerializaton.h
 *
 *    Description:  stream and boost array serializaton
 *
 *        Version:  1.0
 *        Created:  07/13/2010 02:40:48 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#include <iostream>
#include <sstream>

#ifndef IBN_ARRAY_SERIALIZATION_H
#define IBN_ARRAY_SERIALIZATION_H
#include "Array.h"


#include <boost/serialization/nvp.hpp>
#include <boost/serialization/vector.hpp>
/*-----------------------------------------------------------------------------
 *  iostream operations
 *-----------------------------------------------------------------------------*/
namespace std
{
  std::istream & operator>>(std::istream & is, ibn::valer<data::Array::Value_t> & v);
  std::istream & operator>>(std::istream & is, ibn::valer<data::Array::Value_t>::reference  v);
  std::istringstream & operator>>(std::istringstream & is, ibn::valer<data::Array::Value_t>::reference v);
  std::istringstream & operator>>(std::istringstream & is, ibn::valer<data::Array::Value_t> &v);
  std::ostream & operator<<(std::ostream & os, const ibn::valer<data::Array::Value_t> & v);
}


/*-----------------------------------------------------------------------------
 *  Boost serialization infrastructure
 *-----------------------------------------------------------------------------*/
namespace boost
{
  namespace serialization
  {
    template<class Archive, class T>
      void serialize(Archive & ar, ibn::valer<T> & v, const unsigned int version)
      {
        ar & BOOST_SERIALIZATION_NVP(v.value);
        ar & BOOST_SERIALIZATION_NVP(v.error);
      }

    /* 
     * ===  FUNCTION  ======================================================================
     *         Name:  serialize
     *  Description:  Serialization of the data::Array
     * =====================================================================================
     */
    template<class Archive>
      void serialize(Archive & ar, data::Array & v, const unsigned int version)
      {
        ar & v.vvalue();
        ar & v.verror();
      }
  } // namespace serialization
} // namespace boost

#endif
