#ifndef SMV_DATA_ENERGYCALIBRATIONSTORAGEEE_H
#define SMV_DATA_ENERGYCALIBRATIONSTORAGEEE_H

#include <sys/time.h>

#include <thread>

#include <iostream>
#include <string>


#include <chrono>
#include <list>
#include <map>

#include <boost/serialization/access.hpp>
#include <boost/serialization/version.hpp>


#include "Formula.h"
#include "Calculator.h"

#include "measurement/EnergyCalibrationConfig.h"

#include "Data.h"
#include "Storage.h"

#include "Array.h"

#include "PolarimeterConfig.h"


extern PolarimeterConfig PCFG;
namespace data
{

  typedef unsigned run_t;

  class EnergyCalibrationStorage : public Storage
  {
    friend class boost::serialization::access;
    template <class Archive> void serialize(Archive & ar,  const unsigned int version);

    public:
    static const int ARCHIVE_MAGIC_NUMBER=5; /*  this magic number is connected with archive id */

    EnergyCalibrationStorage(unsigned /* number of bunches */, unsigned /*number of counters*/);
    virtual ~EnergyCalibrationStorage(void);
		void Save(std::string);

    unsigned numbunch;  /*  number of bunches */
    unsigned numcounter;/* number of counters */
    unsigned numpair;   /*  number of pairs. default numpair=numcouner/2 */

		std::string shiftman; //shiftman of VEPP-4
		unique_ptr<std::thread> saving_thread;  /* saving thread */

    Array test;//
    typedef std::vector<Array> BunchArray_t;
    typedef std::vector<Array> CounterArray_t; 
    typedef std::vector< std::vector<Array> > BunchCounterArray_t; //Bunch-Counter-Pair Array
    typedef std::vector< std::vector<Array> > BunchPairArray_t; //Bunch-Counter-Pair Array

    Array t; // time
    Array RF;  // VEPP-4M radiofrequency \sim 181801260 Hz 
    Array DepFr; //set depolarizer frequency, Hz
    Array MDepFr; //measured depolarizer frequency, Hz
    std::map<std::chrono::system_clock::time_point, double>  * HRDepFreq = nullptr;

    std::vector <Array> NMR; //NMR H,NEM,SEM,SIM,EM3,KEDR1,KEDR2	


    /* Usefull  lists of arrays for general operations */
    std::list <Array*>  ArrayList; /* list of all arrays */
    std::list <Array*>  FixedArrayList; /* list of arrays geometry independent */
		std::list <CounterArray_t*> CounterArrayList;
    std::list <BunchArray_t*>  BunchArrayList; /* list of arrays for bunches */
    std::list <BunchCounterArray_t*> CounterBunchArrayList;  /*  list of arrays for bunches and counters */
    std::list <BunchPairArray_t*> PairCounterBunchArrayList; /* list of arrays for bunches and pair of counters */


		CounterArray_t  CounterPosition; //x position of counter CouterPosition[counter]

    BunchCounterArray_t CounterLoad;  //Load of counters
    BunchCounterArray_t Count; //Rate of events [bunch][counter]
    BunchCounterArray_t Rnd;   //Random coincidence

    BunchPairArray_t CoincLoad;//Coincidence load [bunch][pair]
    BunchPairArray_t Coinc;    //Coincidence rate [bunch][pair]
    BunchPairArray_t Cor;      //Correlated coincidence rate [bunch][pair]
    BunchPairArray_t RndCoinc; //Random coincidence

    BunchCounterArray_t DeltaCount; //Jump in rate for every counter
    BunchCounterArray_t DeltaRnd;   //Jump in random coincidence
    BunchPairArray_t DeltaCoinc;  //Jump in coincidence for every counter
    BunchPairArray_t DeltaCor;   //Jump in correlated events for every counter.
    BunchPairArray_t DeltaRndCoinc; // Jump in random coincidence

    BunchArray_t DeltaSum; //Jump in sum rate.
    BunchArray_t DeltaCoincSum; //Jump in sum of coincidence rate
    BunchArray_t DeltaCorSum;  //Jump in sum of correlated events.
    BunchArray_t DeltaRndSum;  //Jump in sum of random coincidence events.
    BunchArray_t DeltaRndCoincSum;  // Jump in random coincidence

    BunchArray_t CountSum;//Sum of all counters
    BunchArray_t CoincSum;//Sum of coincidence
    BunchArray_t CorSum; //sum of correlated events 
    BunchArray_t RndSum; //sum of random
    BunchArray_t RndCoincSum; //random coincidence

    BunchArray_t CountSumNormI2; /* Normalized on bunch current sum of count rate */
    BunchArray_t CoincSumNormI2; /* Normalized on bunch current sum of conicidence */
    BunchArray_t CorSumNormI2;  /* Normalized on bunch current sum of correlated */

    BunchArray_t CountSumNormI; /* Normalized on bunch current sum of count rate */
    BunchArray_t CoincSumNormI; /* Normalized on bunch current sum of conicidence */
    BunchArray_t CorSumNormI;  /* Normalized on bunch current sum of correlated */
    BunchArray_t RndSumNormI; //Normalized sum of random coincidence
    BunchArray_t RndCoincSumNormI; //random coincidence sum normalized on I

    BunchArray_t I; //Beam current, mA
    BunchArray_t IRatio; //Ratio of bunch current
    BunchArray_t SigmaX,SigmaZ,SigmaL; //radial, vertical and longitudal length of beam

    Array Isum; /*  total current */

    Array MediumVacuumPresure; /* Average vacuum presure */
    std::map < std::string, Array > Vacuum; /* Vacuum in vepp4 */

    Array Xtmp,Ytmp; /*  tmp vector for variouse usage */

    measurement::EnergyCalibrationConfig cfg; /* Measurement configuration */

    unsigned N;  //Number of data points
    timeval begin_time; //begin time of measurement;
    std::chrono::system_clock::time_point Begin_time;
    std::string mode; //VEPP4 operation mode
    double setE;//Setup energy of the VEPP-4M
    double veppFr; //revolution frequency
    bool isfit; //whether data is fitted or not 

    double calib=0; //Отклонения в калибровке
    double calib_fr=0; //частота резонатора
    bool success; //успешна ли деполяризация
    PhysValue E; //Energy measured, MeV
    PhysValue Fr; //Depolarization frequency measured, Hz
    int dep_fr_num; //harmonic number of depolarizer. Whole part of Fd/F0 
    bool isdepolon; //Depolarizer is switched on?
    std::vector<data::PhysValue> fit_param; //fit parameters are living here
    double chi2; // chi2 value dividing on freedom degree
    std::string SaveFileName;
    std::vector<double> nmr; //вычисляетя в момент скачка по данным NMR, котрый вычитывается для каждой точки
    
    void reserve(unsigned); // reserve memory.
    void resize(unsigned);  //resize arrays.
    void push_back(void); /*  add new point */

    void set_geometry(unsigned nbunch, unsigned ncounter, unsigned npair); /*  set geometry */



    double GetEnergy(double freq)       { return data::frequency2energy(freq,cfg.depolarizer.revolution_frequency,cfg.depolarizer.harmonic_number).value; }
    double GetFrequency(double energy) { return data::energy2frequency(energy,cfg.depolarizer.revolution_frequency, cfg.depolarizer.harmonic_number).value;}
    double GetDeltaEnergy(double df)    { return data::frequency2energy(PhysValue(df,0),cfg.depolarizer.revolution_frequency,cfg.depolarizer.harmonic_number).error; }

    PhysValue frequency2energy(const PhysValue & f) 
    {
      return data::frequency2energy(f, cfg.depolarizer.revolution_frequency, cfg.depolarizer.harmonic_number);
    }

    void InitArrayList(void);

    void SaveCurrent(std::string  filename); //add current data at file for following restoration
    void Restore(const std::string & );
    void SaveBinary(boost::filesystem::path); /*  save binary format */
    //void Save2FS(std::string); //Save data
    //void Save2FSNew(std::string); //New saving function based on boost library 
		void Save2XML(std::string); //save data to XML file
    void Save2FSSerial(std::string); //Save data using boost serialization
    void Save2FSSerial(boost::filesystem::path); //Save data using boost serialization
    void Save2kadrs(void);
    void Save4Utug(void); //Save data for UTUG system
    bool Save2DB(const char *);  //Save data to KDB (KEDR DATA BASE)

    void SaveConfig(std::string filename);

    void Load(std::string);    //Load data from file
    void XMLLoad(std::string);    //Load data from XML file
		void XMLLoad(std::istream &); 
    bool LoadFormatXML(const std::string &);
    bool LoadFormatXML(std::istream &);
    bool LoadFormat0(std::string); /* load first data format */
    bool LoadFormat1(std::string); /* load first data format */

		void ShiftTime(void); //shift time for old data format from begining to format 9
    bool LoadFormat0(std::istream &); /* load first data format */
    bool LoadFormat01(std::istream &); /* load first data format */
    bool LoadFormat1(std::istream &); /* load data format */
    bool LoadFormat32(std::istream &); /* load  data format 3.4 */
    bool LoadFormat34(std::istream &); /* load  data format 3.4 */
    bool LoadFormat4(std::istream &); /* load data format 4.0 */
    bool LoadFormat41(std::istream &); /* load  data format 4.1 */
    bool LoadFormat6(std::istream &); /* load data format 6.0 */
    bool LoadFormat7(std::istream &); /* load data format >7.0 */
    bool LoadBinary(std::istream &); /* load binary format */


    bool ParseHeader(std::map<std::string,std::string>&);

    void PostLoad(void); /*  calculate some aditional data */
    void InitVacuum(void); /* Init vacuum variables with no data*/

    void Clear(void);

    void FillCorrelatedData(unsigned point_number); //calculate and fill arrays with correlated events.
    void FillNormalizedData(unsigned point_number); 
    void FillCalculableData(unsigned point);
    void RemoveDeadVacuumChannels(void); //remove dead channels from vacuum averaging.
    void ReconsructLoad(void); //Reconsruct raw counter load


    double GetValueByTime(double tt, const Array & a) const
    {
      if(tt<t[0].value || tt>t.back().value)
      {
        throw(std::runtime_error("Unable to find time " + to_string(tt) + " in time array"));
      }
      unsigned index;
      for(index=0;index<t.size();index++)
      {
        if(t[index].value <= tt)   break;
      }

      double v1 = a[index].value;
      double t1 = t[index].value;
      if(index-1 < t.size())
      {
        double v2  = index-1 < t.size() ? a[index+1].value : v1;
        double t2  = index-1 < t.size() ? t[index+1].value : t1;
        return v1 + (v2-v1)/(t2-t1)*(tt-t1);
      }
      else
      {
        return v1;
      }
    }
    public:
    //Test  for high resolution depolarizer frequency
  };	

  //void set_config(EnergyCalibrationStorage*, const measurement::EnergyCalibrationConfig*);
}//end of name space data

/*  define storage version */
BOOST_CLASS_VERSION(data::EnergyCalibrationStorage,data::Storage::VERSION);

/*********** archive version ********************* 
 * version 5 first boost archive and with magic number = 5 (not for all run?) boost 1.32
 * version 6 boost archive  with vacuum with magic number =5 (no header) v0.6.x except v0.6.5 boost 1.32
 * version 7 boost archive with vacuum no magic number . boost 1.43
 * version 8 boost archive boost version 1.55
 **************************************************/

#endif //SMV_DATA_STORAGEENERGECALIBRATION_H
