/*
 * =====================================================================================
 *
 *       Filename:  VirtualArray.h
 *
 *    Description:  Virtual array.    ,       .
 *
 *        Version:  1.0
 *        Created:  08.11.2012 15:57:38
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */


namespace data
{
  class FitSquareCurrentNormalizedArray :  public  Array
  {
    public:
      FitSquareCurrentNormalizedArray(Array * data, Array * time, double i0=0, double tb=0) : DATA(data), TIME(time), I0(i0), TB(tb)
    {
      Update();
    }
      ~FitSquareCurrentNormalizedArray(void)
      {
      }

      void Update(void)
      {
        size_t old_size = size();
        size_t new_size = DATA->size();
        reserve(new_size);
        for(unsigned i=old_size; i<new_size; i++)
        {
          ibn::valer<double> x = (*DATA)[i];
          ibn::valer<double> t = (*TIME)[i];
          ibn::valer<double> I2 = I(t)*I(t);
          ibn::valer<double> xn = x/I2; //normalized value
          push_back(xn);
        }
      }

      //calculate current using fited parameters`
      ibn::valer<double> I(ibn::valer<double> t)
      {
        if(TB==0 || I0==0) return  ibn::valer<double>(1,0);
        ibn::valer<double> result;
        result.value = I0/(1.0+t.value/TB);
        result.error = result.value*t.error/(TB+t.value);
        return result;
      }

      void SetCurrent(double i0, double tb)
      {
        I0=i0;
        TB=tb;
      }

    private:
      Array * DATA;
      Array * TIME;
      double I0; //initial current
      double TB; //beam life time
  };


	/*  
  class FunctorArray :  public  Array
	{
		public:
			FunctorArray() : DATA(data), TIME(time), I0(i0), TB(tb)
		{
			Update();
		}
			~FunctorArray(void)
			{
			}

			void Update(void)
			{
				size_t old_size = size();
				size_t new_size = DATA->size();
				reserve(new_size);
				for(unsigned i=old_size; i<new_size; i++)
				{
					ibn::valer<double> x = (*DATA)[i];
					ibn::valer<double> t = (*TIME)[i];
					ibn::valer<double> I2 = I(t)*I(t);
					ibn::valer<double> xn = x/I2; //normalized value
					push_back(xn);
				}
			}

			//calculate current using fited parameters`
			ibn::valer<double> I(ibn::valer<double> t)
			{
				if(TB==0 || I0==0) return  ibn::valer<double>(1,0);
				ibn::valer<double> result;
				result.value = I0/(1.0+t.value/TB);
				result.error = result.value*t.error/(TB+t.value);
				return result;
			}

			void SetCurrent(double i0, double tb)
			{
				I0=i0;
				TB=tb;
			}

		private:
			Array * DATA;
			Array * TIME;
			double I0; //initial current
			double TB; //beam life time
	};
	*/

}

