/*
 * =====================================================================================
 *
 *       Filename:  AntiSlopeFitter.h
 *
 *    Description:  Fit using Root function infrastructure
 *
 *        Version:  1.0
 *        Created:  2013-01-31 14:32 NSK
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *   Organization:  Budker Institute of Nuclear Physics, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_FIT_ANTISLOPE_FITTER_H
#define IBN_FIT_ANTISLOPE_FITTER_H
#include "Fitter.h"
namespace fit
{
  class AntiSlopeFitter : public Fitter
  {
    data::Array * DATA;
    data::Array * TIME;
    public:
    AntiSlopeFitter(data::Array * data, data::Array * time)
    {
      DATA = data;
      TIME = time;
      type = MINUIT2;
      name = "ANTISLOPE";
      title = "Count and current fit";
      defpar.Add("A",    0, 1e4); //Initial count
      defpar.Add("kbg",  1e-4, 5e-5); //1/tau
      defpar.Add("ktsh", 1e-4, 5e-5); //1/tau
      inipar=defpar;
    }

    void Fit(void)
    {
      using namespace ROOT::Minuit2;
      if(DATA->size()>0) inipar.SetValue(0,(*DATA)[0].value);
      MnMigrad migrad(*this, inipar);
      FunctionMinimum minimum = migrad();
      minpar=minimum.UserParameters();
      //print();
    }

    void print(void)
    {
      std::cout << minpar << std::endl;
    }

    //double fitfun(double t, const std::vector<double> & par) const
    //{
    //  return par[0]*exp(-par[1]*t);
    //}
    //double fitfun(double t, const std::vector<double> & par) const
    //{
    //  return par[0]*par[1]/pow(1.+t/par[1], 2);
    //}
    double fitfun(double t, const std::vector<double> & par) const
    {
      double N0 = par[0];
      double Kbg = par[1];
      double Ktsh =  par[2];
      double e = exp(-Kbg*t);
      double f = e/(1.0 + (1.-e) * Ktsh/Kbg);
      return  N0*(Ktsh*f*f + Kbg*f)/(Kbg+Ktsh);
    }

    double fitfun(double t) const
    {
      return fitfun(t, minpar.Params());
    }

    double operator() (const std::vector<double> & par) const
    {
      double chi2=0;
      size_t size =std::min(DATA->size(), TIME->size());
      for(unsigned i=0; i<size ; i++)
      {
        double N =  (*DATA)[i].value;
        double dN = (*DATA)[i].error;
        if(dN==0) dN=1;
        double t =  (*TIME)[i].value;
        chi2 += pow((N - fitfun(t,par))/dN, 2);
      }
      return chi2;
    }
    double Up() const { return 1.; }
  };
}
#endif
