/*
 * =====================================================================================
 *
 *       Filename:  FitConfig.h
 *
 *    Description:  Fit configurations
 *
 *        Version:  1.0
 *        Created:  07/11/2010 12:44:31 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_FIT_CONFIG_H
#define IBN_FIT_CONFIG_H

#include <string>
#include <map>
#include <list> 
#include "FitParConfiguration.h"
//#include "CountCurrentFitter.h"
#include "Fitter.h"

namespace fit
{
	typedef double ( * RootFunction_t  ) ( double *, double *);


  struct FitMethodConfiguration
  {
    std::string name; //the name of the method
    enum 
    {
      FUNC,
      FCN,
    } type;
    enum
    {
      NONE,
      FIND_JUMP
    } flag; //some flag for example to find jump 1
    RootFunction_t  func;
    //std::vector < FitParConfiguration > par; //parameter configuration
    ROOT::Minuit2::MnUserParameters par;
    Fitter * fitter;
    FitMethodConfiguration(void) :
      name(""),type(FUNC), flag(NONE), func(0) {}
    FitMethodConfiguration(const FitMethodConfiguration & fm) : 
      name(fm.name), type(fm.type),flag(fm.flag), func(fm.func), par(fm.par), fitter(fm.fitter) {}
  };

  extern std::map < std::string, FitMethodConfiguration >  FitMethodList;
	extern void InitFitMethodList(void);
  extern std::list <unsigned> BadPointList;

}//namespace fit
#endif //IBN_FIT_CONFIG_H
