/*
 * =====================================================================================
 *
 *       Filename:  Fitter.h
 *
 *    Description:  Base class for fit using fcn
 *
 *        Version:  1.0
 *        Created:  16.11.2012 10:42:47
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *   Organization:  Budker Institute of Nuclear Physics, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_POLARIMETER_FIT_FITTER_H
#define IBN_POLARIMETER_FIT_FITTER_H
#include <Minuit2/MnUserParameters.h>
#include <Minuit2/FCNBase.h>
#include <Minuit2/MnMigrad.h>
#include <Minuit2/MnApplication.h>
#include "Minuit2/MnPrint.h"
#include "Minuit2/FunctionMinimum.h"

namespace data
{
	class Storage;
};

namespace gui
{
	class EnergyCalibrationView;
}

namespace fit
{
  class Fitter :  public  ROOT::Minuit2::FCNBase 
  {
    public:
      Fitter(void){type=MINUIT2;};
      virtual ~Fitter(void){}
      virtual void SetParameters(const std::vector<double>  & par) const {}
      virtual void SetParameters(const ROOT::Minuit2::MnUserParameters & p) {inipar = p;}
      virtual void Fit(void) {};
      virtual void Fit(double begintime, double endtime) {};
			virtual void SetStorage(data::Storage * storage, int bunch=-1, int counter=-1, int pair = -1)
      {
        fStorage = storage;
        fBunch = bunch;
        fCounter = counter;
        fPair = pair;
      };
			virtual void SetData(data::Storage * storage, std::string name)
      {
        fStorage = storage;
        fDataName = name;
      };
			virtual void SetView(gui::EnergyCalibrationView * view){fView = view;};
			virtual void Draw(int opt=0){};
			virtual void Print(void){};
      std::string name; //name if the fitter
      std::string title; //title of the fitter
      ROOT::Minuit2::MnUserParameters defpar; //default initial parameters (never changed during lifetime of Fitter object)
      ROOT::Minuit2::MnUserParameters inipar; //initial parameters (could be changed by user or after fit)
      ROOT::Minuit2::MnUserParameters minpar; //result of the fit
      enum Type
      {
        ROOT_FUNCTION=0,
        MINUIT2=1,
      };
      Type GetType(void){ return type;}
      double tmin, tmax;
      void SetTimeRange(double tmi=0, double tma=0)
      {
        tmin = tmi;
        tmax = tma;
      }
			void SetScan(bool s) { isscan=s;}
			bool IsScan(void) const {return isscan; }
      virtual void DrawFunction(void){}
    protected:
			bool isscan; //whether or scan over depolarization moment
      Type type;
      int fBunch=-1,fCounter=-1,fPair=-1;
      data::Storage * fStorage;
      gui::EnergyCalibrationView * fView;
      std::string fDataName;
  };
}
#endif
