// =====================================================================================
//
//       Filename:  BunchViewFrame.h
//
//    Description:  
//
//        Version:  2.0
//        Created:  17.02.2016 12:03:21
//       Revision:  none
//       Compiler:  g++
//
//         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
//   Organization:  Budker Institute of Nuclear Physics
//
// =====================================================================================

#pragma once 
#include <vector>
#include <map>
#include <memory>

#include <TGFrame.h>

class TGTextButton;
class TGLabel;

namespace data
{
  class BunchInfo_t;
}

namespace gui
{
  class EnergyCalibrationView;

  class BunchViewFrame : public TGGroupFrame 
  {
    //this class shows load of counters for each bunch
    public:
      BunchViewFrame(const TGWindow * w, const std::vector<data::BunchInfo_t> &b, EnergyCalibrationView * ecv=nullptr);
      ~BunchViewFrame(void);
      void Update(void);
			Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 );
    private:
      const std::vector<data::BunchInfo_t> & BI; //this is info of counters load. It is fill by StatusMeasurement
      EnergyCalibrationView * ECView;  //canvas manager know which graphics to show
			std::string format_str="%7.2f";
			
      struct Item_t
      {
        std::unique_ptr<TGTextButton> text_button;
        std::string title; //this title users see in the window
        std::string units; //value measurement units kHz,% ...
        std::string name; //internal name of the button
        int id;
      };

      struct BunchButtons_t
      {
        Item_t I; //current
        std::vector<Item_t> Cnt; //counter
        Item_t  CntSum; //sum of all counters
        Item_t  DeltaCntSum; //delta in sum of counters
        std::vector<Item_t>  Coinc; // coincidence
        Item_t  CoincSum; //sum of all coincidence
        Item_t  DeltaCoincSum; //delta in sum of coincidence
      };

      std::vector<BunchButtons_t> BB; //per bunch buttons 

      std::map <int, Item_t*> fBMap; //buttons index map. It is used for identification which button is pressed
  };
}
