#include <fstream>

#include "CalculatorConfigDialog.h"
#include "Log.h"
#include "PolarimeterConfig.h"
namespace gui
{

  CalculatorConfigDialog::CalculatorConfigDialog(const TGWindow *w, data::Calculator *cal ) :
    TGTransientFrame(gClient->GetRoot(), w,400,100)
  {
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::CalculatorConfigDialog BEGIN" << std::endl;
    calculator=cal;
    bf = new TGHorizontalFrame(this, 300,200);
    bl = new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX,2,2,2,2);
    b_name.push_back("<< Cancel");
    b_name.push_back("Ok");
    b.resize(b_name.size());
    for(size_t i = 0; i< b.size(); i++)
    {
      b[i] = new TGTextButton(bf,b_name[i].c_str(),i);
      b[i]->Associate(this);
      bf->AddFrame(b[i],bl);
    }
    AddFrame(bf,bl);

    ll=new TGLayoutHints( kLHintsLeft | kLHintsExpandX | kLHintsTop, 2,2,2,2);
    lr=new TGLayoutHints( kLHintsRight | kLHintsExpandX | kLHintsTop, 2,2,2,2);

    alpha_f = new TGHorizontalFrame(this, 200,200);
    N_f = new TGHorizontalFrame(this, 200,200);
    alpha = new TGNumberEntry(alpha_f, calculator->GetTemperatureShiftParameter(),12,0,TGNumberFormat::kNESRealTwo);
    N = new TGNumberEntry(N_f, calculator->GetN(), 12, 1, TGNumberFormat::kNESInteger);
    alpha_lbl = new TGLabel(alpha_f, "temperature drift [kEV/grad]");
    N_lbl = new TGLabel(N_f, "number of point before jump");
    alpha_f->AddFrame(alpha_lbl,ll);
    alpha_f->AddFrame(alpha,lr);
    N_f->AddFrame(N_lbl,ll);
    N_f->AddFrame(N,lr);

    AddFrame(alpha_f, ll);
    AddFrame(N_f, ll);

    Layout();
    MapSubwindows();
    MapWindow();
    fClient->WaitFor(this);
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::CalculatorConfigDialog END" << std::endl;
  }

  CalculatorConfigDialog::~CalculatorConfigDialog(void)
  {
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::~CalculatorConfigDialog BEGIN" << std::endl;
    for(size_t i=0;i<b.size(); i++) delete b[i];
    delete bf;
    delete bl;
    delete alpha_lbl;
    delete N_lbl;
    delete alpha_f;
    delete N_f;
    delete  alpha;
    delete N;
    delete ll;
    delete lr;
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::~CalculatorConfigDialog END" << std::endl;
  }

  void CalculatorConfigDialog::SetData(const data::Calculator * c)
  {
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::SetData BEGIN" << std::endl;
    N->SetIntNumber(c->GetN());
    alpha->SetNumber(c->GetTemperatureShiftParameter()*1000);
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::SetData END" << std::endl;
  }

  void CalculatorConfigDialog::GetData(data::Calculator *c)
  {
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::GetData BEGIN" << std::endl;
    c->SetN(N->GetIntNumber());
    c->SetTemperatureShiftParameter(alpha->GetNumber()/1000.);
    std::clog<<LOG(DEBUG_LOG)<<"CalculatorConfigDialog::GetData END" << std::endl;
  }

  Bool_t CalculatorConfigDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
  { 
    switch( GET_MSG(msg) )
    {
      case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)
            {
              case 0: //back
                delete this;
                break;
              case 1: //reset
                GetData(calculator);
                {
                  std::ofstream ofs((PCFG.work_dir.string()+"/calculator.cfg").c_str());
                  if(!ofs)
                  {
                    std::cerr << LOG(1) <<  "WARNING: Unable to save calculator config into file " <<  PCFG.work_dir.string()+"/calculator.cfg" << std::endl ;
                  }
                  ofs << *calculator;
                }
                break;
              default:
                break;
            }
            break;
        }
        break;
    }
    return kTRUE;
  }
}//end of namespace gui
