// =====================================================================================
//
//       Filename:  CounterConfigDialog.cpp
//
//    Description:  
//
//        Version:  1.0
//        Created:  07.07.2014 17:07:31
//       Revision:  none
//       Compiler:  g++
//
//         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
//   Organization:  Budker Institute of Nuclear Physics
//
// =====================================================================================

#include "CounterConfigDialog.h"
#include "measurement/EnergyCalibrationCamac.h"
#include <TGFrame.h>
namespace gui
{
	class CounterConfig : public TGHorizontalFrame
	{
		bool isOn;
		int fId;
		std::unique_ptr<TGLabel> fLabel;
		std::unique_ptr<TGNumberEntry> fNumberEntry;
		std::unique_ptr<TGTextButton> fOnOffButton;
		std::unique_ptr<TGTextButton> fResetButton;
		std::unique_ptr<TGLayoutHints> fLayout;
		int fNormalBackgroundColor;
		public:
		CounterConfig(const TGWindow *w,  std::string label, double  data, unsigned counter_id) : TGHorizontalFrame(w, 100, 200)
		{
			fId = counter_id;
			fLayout.reset(new TGLayoutHints(kLHintsLeft| kLHintsExpandX,2,10,2,2));
			fLabel.reset(new TGLabel(this,  label.c_str()));
			fNumberEntry.reset(new TGNumberEntry(this,data,6,3*counter_id,TGNumberEntry::kNESInteger));
			fOnOffButton.reset(new TGTextButton(this, "Power",3*counter_id+1));
			fResetButton.reset(new TGTextButton(this, "Reset",3*counter_id+2));
			fNormalBackgroundColor=fResetButton->GetBackground();
			AddFrame(fLabel.get(),  fLayout.get());
			AddFrame(fNumberEntry.get(),  fLayout.get());
			AddFrame(fOnOffButton.get(),  fLayout.get());
			AddFrame(fResetButton.get(),  fLayout.get());
			fNumberEntry->Associate(w);
			fOnOffButton->Associate(w);
			fResetButton->Associate(w);
		}
		void SetNumber(double data) 
		{
			fNumberEntry->SetNumber(data);
		}
		unsigned GetNumber(void) const 
		{
			return fNumberEntry->GetIntNumber();
		}

		bool IsOn(void) const 
		{
			return isOn;
		}

		void SetOn(void)
		{
			fOnOffButton->SetState(kButtonEngaged);
			//fOnOffButton->SetBackgroundColor(0xff00);
			//clog << "color = " << hex << fOnOffButton->GetBackground() << endl;
			isOn=true;
		}
		void SetOff(void)
		{
			std::clog << LOG(DEBUG_LOG) << "Set Off for normal background color "<< std::endl;
			//fOnOffButton->SetBackgroundColor(fNormalBackgroundColor);
			//clog << "color = " << hex << fOnOffButton->GetBackground() << endl;
			fOnOffButton->SetState(kButtonUp);
			isOn=false;
		}
		void SwitchOnOff(void)
		{
			if(isOn) 
			{
				clog << "Find state ON then turn OFF" << endl;
				SetOff();
			}
			else
			{
				clog << "Find state OFF then turn ON" << endl;
				SetOn();
			}
			cout << "The state after operation: " << IsOn() << endl;
		}

		void ResetOverload(void)
		{
			//fResetButton->SetBackgroundColor(fNormalBackgroundColor);
		}

		void SetOverLoad(void)
		{
			//fResetButton->SetBackgroundColor(0xff00ff);
		}
	};

	CounterConfigDialog::CounterConfigDialog(const TGWindow * w, camac::EnergyCalibrationCamac * c) : TGTransientFrame(gClient->GetRoot(), w,  600, 270)
	{
		camac = c;
		fLayout.reset(new TGLayoutHints(kLHintsLeft| kLHintsExpandX,2,2,2,2));
		unsigned counter_number = camac->GetCounterNumber();
		for(unsigned counter=0; counter < counter_number ; counter++)
		{
			fCounterConfig.push_back(new gui::CounterConfig(this, std::to_string(counter+1) + "   " + camac->GetVviAddress(counter).str(), camac->GetVoltage(counter),counter));
			if(camac->IsOn(counter)) fCounterConfig.back()->SetOn();
			AddFrame(fCounterConfig.back(), fLayout.get());
		}
		fButtonFrame.reset(new TGHorizontalFrame(this));
		int SHIFT=100;
		fUpdateButton.reset(new TGTextButton(fButtonFrame.get(), "Update",SHIFT+0));
		fOnAllButton.reset(new TGTextButton(fButtonFrame.get(), "On all",SHIFT+1));
		fOffAllButton.reset(new TGTextButton(fButtonFrame.get(), "Off all",SHIFT+2));
		fResetAllButton.reset(new TGTextButton(fButtonFrame.get(), "Reset all",SHIFT+3));
		fButtonLayout.reset(new TGLayoutHints(kLHintsLeft | kLHintsExpandX,2,2,2,2));

		fUpdateButton->Associate(this);
		fOnAllButton->Associate(this);
		fOffAllButton->Associate(this);
		fResetAllButton->Associate(this);

		fButtonFrame->AddFrame(fUpdateButton.get(),  fButtonLayout.get());
		fButtonFrame->AddFrame(fOnAllButton.get(),  fButtonLayout.get());
		fButtonFrame->AddFrame(fOffAllButton.get(),  fButtonLayout.get());
		fButtonFrame->AddFrame(fResetAllButton.get(),  fButtonLayout.get());
		AddFrame(fButtonFrame.get(),  fLayout.get());
		Update();

		SetWindowName("High voltage configuration");
    Layout();
    MapSubwindows();
    MapWindow();
	}
	CounterConfigDialog::~CounterConfigDialog(void)
	{
		for(auto & c : fCounterConfig) delete c;
	}

	void CounterConfigDialog::Update(void) //Read state of VVi and f
	{
		for(unsigned counter=0;counter<fCounterConfig.size(); counter++)
		{
			double V = camac->GetVoltage(counter);
			bool ison = camac->IsOn(counter);
			bool overload = camac->IsOverload(counter);
			fCounterConfig[counter]->SetNumber(V);
			if(ison) fCounterConfig[counter]->SetOn();
			else fCounterConfig[counter]->SetOff();
			if(overload) fCounterConfig[counter]->SetOverLoad();
			else  fCounterConfig[counter]->ResetOverload();
		}
	}

	Bool_t CounterConfigDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
	{
    switch( GET_MSG(msg) )
    {
      case kC_COMMAND:
        switch (GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
						if(parm1 <100)
						{
							int counter = parm1/3;
							auto c = fCounterConfig[counter];
							int button = parm1%3;
							switch(button)
							{
								case 1: //OnOff
									c->SwitchOnOff();
									if(!c->IsOn()) 
									{
										std::clog << LOG(NORMAL_LOG) << "Turn OFF VVI " << counter << std::endl;
										camac->Off(counter);
									}
									else 
									{
										std::clog << LOG(NORMAL_LOG) << "Turn ON VVI " << counter << std::endl;
										camac->On(counter);
									}
									break;
								case 2: //Reset
									c->ResetOverload();
									camac->Reset(counter);
									std::clog << LOG(NORMAL_LOG) << "Reset VVI " << counter << std::endl;
									break;
							}
						}
						else
						{
							switch(parm1-100)
							{
								case 0:
									std::clog << LOG(1) << "Update data in dialog" << endl;
									Update();

									break;
								case 1:
									std::clog << LOG(1) << "Turn ON all counter's VVI" << endl;
									for(unsigned i=0; i<fCounterConfig.size();i++)
									{
										fCounterConfig[i]->SetOn();
										camac->On(i);
									}
									break;
								case 2:
									std::clog << LOG(1) << "Turn OFF all counter's VVI" << endl;
									for(unsigned i=0; i<fCounterConfig.size();i++)
									{
										fCounterConfig[i]->SetOff();
										camac->Off(i);
									}
									break;
								case 3:
									std::clog << LOG(1) << "Reset all counter's VVI" << endl;
									for(unsigned i=0; i<fCounterConfig.size();i++)
									{
										camac->Reset(i);
									}
									break;
							}
						}
            break; //end of kCM_BUTTON
          case kCM_CHECKBUTTON:
            break;
        }
        break;

      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg))
        {
          case kTE_ENTER:
						{
							int counter = parm1/3;
							auto c = fCounterConfig[counter];
							camac->SetVoltage(counter, c->GetNumber());
							std::clog << LOG(NORMAL_LOG) << "Set voltage for counter " << counter << " to " << c->GetNumber() << std::endl;
						}
            break;
          case kTE_TAB:
            switch(parm1)
            {
              case 0:
                break;
            }
            break;
        }
        break;
    }//GET_MSG
		DoRedraw();
		RaiseWindow();
		return kTRUE;
	}
}
