#include "CounterDialog.h"
#include "Utils.h"


#include <TCanvas.h>
#include <TSystem.h>


enum EVENT { BACK, NEXT};


namespace gui
{ 

  //=================================================================
  //   .     :    ,  
  //     .     ,     
  //  ,    100% 

  CounterDialog::CounterDialog(const TGWindow * w, measurement::Manager *pol) :
    TGTransientFrame(gClient->GetRoot(), w,150,60) 
  {
    std::cout<<LOG(4)<<"CounterDialog::CounterDialog BEGIN\n";
    polarimeter = pol; 
    b[0]=new TGTextButton(this,"Landao",1);
    b[1]=new TGTextButton(this,"Classic",2);



    b_layout = new TGLayoutHints(kLHintsTop | kLHintsExpandX,4,4,4,4);
    for (bmap_t::iterator p=b.begin(); p!=b.end(); p++)  {
      AddFrame(p->second, b_layout);
      p->second->Associate(this);
      p->second->SetState(kButtonUp);
    }
    Layout();
    SetWindowName("CounterCalibration");
    MapSubwindows();
    MapWindow();
    // Resize(GetDefaultSize());
    fClient->WaitFor(this);
    std::cout<<LOG(4)<<"CounterDialog::CounterDialog FINISH\n";
  } 

  CounterDialog::~CounterDialog(void)   {
    std::cout<<LOG(4)<<"CounterDialog::~CounterDialog BEGIN\n";
    //for (bmap_t::iterator p=b.begin(); p!=b.end(); p++) delete p->second;
    for(size_t i=0;i<b.size(); i++) delete b[i];
    std::cout<<LOG(4)<<"CounterDialog::~CounterDialog FIFNISH\n";
  }

  Bool_t CounterDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )  
  {
    std::cout<<LOG(4)<<"CounterDialog::ProcessMessage BEGIN\n";
    //													measurement::CounterCalibrate CC;
    switch( GET_MSG(msg) )	
    {
      case kC_COMMAND:
        switch(GET_SUBMSG(msg)) 
        {
          case kCM_BUTTON:
            switch(parm1) {
              case 1: //back
                std::cout<<"CounterDialog::ProcessMessage: parm1=1\n";
                //delete this;
                //Counter calibrate by landao distribution 
                new CounterCalibrate_by_Landao_Dialog(this  , polarimeter->CounterCalibrate_Landao);

                break;
              case 2: //reset
                //delete this;

                std::cout<<"CounterDialog::ProcessMessage: parm1=2\n";
                // new CounterCalibrate_by_ClassicMethod_Dialog(this, polarimeter->CounterCalibrate_Classic); 
                break;
              default:
                std::cout<<"CounterDialog::ProcessMessage: parm1=default\n";
                break;
            }
            break;

        }
        break;
      default:
        std::cout<<"CounterDialog::ProcessMessage: GET_MSG(msg)=default\n";
        break;
    }
    std::cout<<LOG(4)<<"CounterDialog::ProcessMessage FIFNISH\n";
    return true;
  }

  void CounterDialog::LockButton(unsigned i )
	{
    std::cout<<LOG(4)<<"CounterDialog::LockButton BEGIN\n";
    ismeasure=true;
    for (bmap_t::iterator p=b.begin(); p!=b.end(); p++)  p->second->SetState(kButtonDisabled);
    b[i]->SetState(kButtonDown);
    gSystem->ProcessEvents();
    std::cout<<LOG(4)<<"CounterDialog::LockButton FIFNISH\n";
  }

  void CounterDialog::UnLockButton(void )
	{
    std::cout<<LOG(4)<<"CounterDialog::UnLockButton BEGIN\n";
    for (bmap_t::iterator p=b.begin(); p!=b.end(); p++)  p->second->SetState(kButtonUp);
    gSystem->ProcessEvents();
    ismeasure=false;
    std::cout<<LOG(4)<<"CounterDialog::UnLockButton FIFNISH\n";
  }

  void CounterDialog::CloseWindow(void)  {
    std::cout<<LOG(4)<<"CounterDialog::CloseWindow BEGIN\n";
    delete this;
    std::cout<<LOG(4)<<"CounterDialog::CloseWindow FIFNISH\n";
  }




  //=================================================================



  void* LanadaoDistributionGraphicView_in_thread(void* CounterCalibrateD_RAW)
  {
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: START\n";
    CounterCalibrate_by_Landao_Dialog* CounterCalibrateDialog = static_cast<CounterCalibrate_by_Landao_Dialog*>(CounterCalibrateD_RAW);
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 1\n";
    measurement::CounterCalibrate_by_Landao* CounterCalibr = CounterCalibrateDialog->CounterCalibr;
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 2\n";
    TH1F* Counter_calibr_landao_histogram;
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 3\n";
    std::string counter_num_str = setString<int>(int(CounterCalibrateDialog->counter_num->GetNumber()), 0, 0);
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 4\n";
    TCanvas * my_c = new TCanvas( ("Counter" + counter_num_str).c_str(), ("Counter" + counter_num_str).c_str(), 1);
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 5\n";
    Counter_calibr_landao_histogram = new TH1F(("CounterLandaoHisto_counter" + counter_num_str).c_str(), ("LandaoDistribution_counter" + counter_num_str).c_str(), 100, 0, 2000);
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: 6\n";
    double tmp;

    while( CounterCalibr->IsMeasure() )
    {
      gSystem->ProcessEvents();
      tmp = CounterCalibr->GetCurrentMeasurement();
      std::cout<<"tmp="<<tmp<<"\n";
      Counter_calibr_landao_histogram->Fill(tmp);
      Counter_calibr_landao_histogram->Draw();
      my_c->Modified();
      my_c->Update();

    }
    std::cout<<LOG(4)<<"LanadaoDistributionGraphicView_in_thread: FINISH\n";

    return NULL;
  }









  CounterCalibrate_by_Landao_Dialog::CounterCalibrate_by_Landao_Dialog (  const TGWindow * main,  measurement::CounterCalibrate_by_Landao* P) :  TGTransientFrame (gClient->GetRoot(),main,257,69, kFixedWidth)      
  {

    std::cout<<LOG(4)<<"CounterCalibrate_by_Landao_Dialog::CounterCalibrate_by_Landao_Dialog BEGIN\n";
    CounterCalibr = P;
    SetWindowName("Counters calibration configuration");

    lf = new TGLayoutHints(kLHintsTop | kLHintsExpandX,2,2,2,2);
    ll = new TGLayoutHints(kLHintsLeft,2,2,2,2);
    lr = new TGLayoutHints(kLHintsRight,2,2,2,2);

    counter_num_f=new TGHorizontalFrame(this,300,200);//frame for counter number
    AddFrame(counter_num_f,lf);

    delay_f=new TGHorizontalFrame(this,300,200);//frame for delay for delay line
    AddFrame(delay_f,lf);

    threshold_f=new TGHorizontalFrame(this,300,200); //frame for treshold for diskriminator
    AddFrame(threshold_f,lf);

    voltage_f=new TGHorizontalFrame(this,300,200); //frame for voltage for VVI
    AddFrame(voltage_f,lf);


    delay = new TGNumberEntry(delay_f, 20/*nsec. by default*/ , 12,0,TGNumberFormat::kNESInteger);
    threshold= new TGNumberEntry(threshold_f, 50/*mV by default*/ ,12,0,TGNumberFormat::kNESInteger);
    voltage= new TGNumberEntry(voltage_f,1700,12,1,TGNumberFormat::kNESInteger);
    counter_num= new TGNumberEntry(counter_num_f,1,12,1,TGNumberFormat::kNESInteger);


    //This is needed for updating without starting
    //				delay->Associate(this);
    //				threshold->Associate(this);
    voltage->Associate(this);
    //				counter_num->Associate(this);

    delay_l				=new TGLabel(delay_f, "Delay [ns]");
    threshold_l		=new TGLabel(threshold_f, "Threshold [mV]");
    voltage_l			=new TGLabel(voltage_f, "Voltage [V]");
    counter_num_l	=new TGLabel(counter_num_f, "Counter number");

    counter_num_f->AddFrame(counter_num_l,ll);
    counter_num_f->AddFrame(counter_num, lr);
    delay_f->AddFrame(delay_l,ll);
    delay_f->AddFrame(delay,lr);
    threshold_f->AddFrame(threshold_l,ll);
    threshold_f->AddFrame(threshold,lr);
    voltage_f->AddFrame(voltage_l,ll);
    voltage_f->AddFrame(voltage,lr);

    /*        std::string str="Expected energy: " + setString(cfg->energy.value,0) + "+-" + setString(cfg->energy.error,0) + " MeV";

              een=new TGLabel(this,str.c_str());
              AddFrame(een,ll);


              fe = new FrequencyEnergyFrame(this, cfg);
              AddFrame(fe,lf);
              */
    bf = new TGHorizontalFrame(this, 300,200);
    bl = new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX,2,2,2,2);
    b_name.push_back("<< Back");
    //b_name.push_back("Reset");
    //b_name.push_back("Calc");
    b_name.push_back("Start >>");
    b.resize(b_name.size());
    for(size_t i = 0; i< b.size(); i++) {
      b[i] = new TGTextButton(bf,b_name[i].c_str(),i);
      b[i]->Associate(this);
      bf->AddFrame(b[i],bl);
    }
    AddFrame(bf,bl);
    Layout();
    MapSubwindows();
    MapWindow();
    Resize(GetDefaultSize());
    //				std::cout<<LOG(4)<<"MeasurementConfigDialog::MeasurementConfigDialog WAIT_FOR\n";
    fClient->WaitFor(this);
    std::cout<<LOG(4)<<"CounterDialog::CounterDialog FINISH\n";
  }


  CounterCalibrate_by_Landao_Dialog::~CounterCalibrate_by_Landao_Dialog(void)
  {}

  Bool_t CounterCalibrate_by_Landao_Dialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 ) { 
    std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::ProcessMessage BEGIN\n";
    //TH1F* Counter_calibr_landao_histogram;
    switch( GET_MSG(msg) )	{
      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg)){
          case kTE_ENTER:
            switch(parm1){
              case 1://update menu
                if(!(measurement::Manager::isnocamac)) //  ,    
                {
                  CounterCalibr->SetVviVolt(int(counter_num->GetNumber()) ,int(voltage->GetNumber()));
                  std::cout<<LOG(1)<<"Vvi for counter "<<counter_num->GetNumber()<<" is seted\n";
                }
                break;
              default:
                break;
            }
            break;
        }
        break;
      case kC_COMMAND:
        switch(GET_SUBMSG(msg)) {
          case kCM_BUTTON:
            switch(parm1) {
              case 0: //back
                if(b[1]->GetState() == kButtonDisabled)
                {															
                  CounterCalibr->Stop();	
                  counter_num->SetState(kTRUE);
                  delay->SetState(kTRUE);
                  threshold->SetState(kTRUE);
                  voltage->SetState(kTRUE);
                  b[1]->SetState(kButtonUp);
                }
                else
                {
                  delete this;
                }
                break;
              case 1: 
                counter_num->SetState(kFALSE);
                delay->SetState(kFALSE);
                threshold->SetState(kFALSE);
                voltage->SetState(kFALSE);
                b[1]->SetState(kButtonDisabled);
                std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::ProcessMessage: 1\n";
                std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::ProcessMessage: 2\n";
                pthread_create(&landao_view_thread,0,LanadaoDistributionGraphicView_in_thread,this); // 
                break;
              default:
                break;
            }
            break;
        }
        break;
        std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::ProcessMessage FINISH\n";
    }
    return kTRUE;
  }
}
