#include "DataViewDialog.h"

#include <map>
#include <string>
#include <memory>
#include <regex>

#include <TGFrame.h>
#include <TGTab.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGTableLayout.h>

#include "ibn/regexp.h"

#include "fit/FitConfig.h"

namespace gui 
{


  DataViewDialog::DataViewDialog(const TGWindow * main, EnergyCalibrationView *ecv)  :	
    TGTransientFrame(gClient->GetRoot(),main)	
  {
    lockWindow=false;
    ECV=ecv;
    int cb_id  = 0;
    lt.reset(new TGLayoutHints(kLHintsTop | kLHintsExpandX,5,5,0,0));

    tab.reset(new TGTab(this));

    unsigned BN = ECV->BunchGraphList.begin()->second.size();
		auto title_layout_opts=kLHintsExpandX;
		auto layout_opts=kLHintsExpandX | kLHintsFillX;

    ibn::regex counter_re("counter");
    ibn::regex coincidence_re("coincidence");
    ibn::regex correlated_re("correlated");
    ibn::regex random_re("random");
    ibn::regex load_re("load");
    ibn::regex coincidence_load_re("coincidence_load");
    ibn::regex normalized_re("normalized");
    ibn::regex vacuum_medium_re("vacuum-medium");
    ibn::regex vacuum_re("vacuum");
    ibn::regex delta_count_re("delta_count");

    std::regex counter(R"(^counter)");
    std::regex normalized(R"(^normalized)");
    std::smatch sm;

    TGCompositeFrame * main_tab = tab->AddTab("main");

    //for ( GraphList_t::const_iterator p = NonBunchGraphList.begin(); p != NonBunchGraphList.end(); p++)	
    for(auto & p : ECV->NonBunchGraphList)
    {
      if(vacuum_re.ismatch(p.first.c_str()))  continue; /*  skip vacuum data */
      CheckButton_t cbt = { cb_id,  new TGCheckButton(main_tab, new TGHotString(p.first.c_str()), cb_id )}; 
      CBList[p.first] = cbt;
      IdGraphMap[cb_id] = p.first;
      cb_id++;
      cbt.button->Associate ( this );
      main_tab->AddFrame ( cbt.button , lt.get() );
    }


    GFBunch.reset(new TGGroupFrame(main_tab, new TGString("Data")));
    GFBunch->SetLayoutManager(new TGMatrixLayout(GFBunch.get(),0,BN+1,5,5));
    main_tab->AddFrame(GFBunch.get(),lt.get());
    GFBunch->AddFrame(new TGLabel(GFBunch.get(),"bunch"));
    for( size_t i=0; i<BN; i++)   
    {
      GFBunch->AddFrame(new TGLabel(GFBunch.get(), setString(i+1).c_str()));
    }

    for(auto & p : ECV->BunchGraphList)
    {
      auto & name  = p.first;

      if( 
          (    std::regex_search(p.first,sm,counter)
            || std::regex_search(p.first,sm,normalized)
            || counter_re.ismatch(p.first.c_str())     
            || coincidence_re.ismatch(p.first.c_str())  
            || correlated_re.ismatch(p.first.c_str())
            || random_re.ismatch(p.first.c_str())
            || load_re.ismatch(p.first.c_str()) 
            || coincidence_load_re.ismatch(p.first.c_str())
            || delta_count_re.ismatch(p.first.c_str())
          )
          && ! std::regex_match(name, std::regex("counter_sum"))
        ) continue;
      GFBunch->AddFrame(new TGLabel(GFBunch.get(),p.first.c_str()));
      for( size_t i=0; i<BN; i++)   
      {
        CheckButton_t cbt = { cb_id,  new TGCheckButton( GFBunch.get(), new TGHotString(""), cb_id )}; 
        CBList[p.first+"-"+setString(i+1,0,0)] = cbt;
        IdGraphMap[cb_id] = p.first+"-"+setString(i+1,0,0);
        cb_id++;
        cbt.button->Associate ( this );
        GFBunch->AddFrame ( cbt.button );
      }
    }

    auto add_tab = [&](std::string tabname, std::function<bool(std::string)> filter)
    {
      TGCompositeFrame * theTab = tab->AddTab(tabname.c_str());
      theTab->SetLayoutManager(new TGTableLayout(theTab,200, BN+1));
      int row=0;
      theTab->AddFrame(new TGLabel(theTab,"bunch"),new TGTableLayoutHints(0, 1, row, row+1));
      for(unsigned i=0;i<BN;i++)
      {
        theTab->AddFrame(new TGLabel(theTab, std::to_string(i+1).c_str()), new TGTableLayoutHints(i+1, i+2, row, row+1,  kLHintsLeft));
      }
      row++;
      for(auto & p : ECV->BunchGraphList)
      {
        if(!filter(p.first)) continue;
        theTab->AddFrame(new TGLabel(theTab,p.first.c_str()),new TGTableLayoutHints(0, 1, row, row+1,  title_layout_opts)); 
        for(unsigned i=0;i<BN;i++)
        {
          CheckButton_t cbt = { cb_id,  new TGCheckButton( theTab, new TGHotString(""), cb_id )}; 
          auto bstr=std::to_string(i+1);
          auto name = p.first+"-"+bstr;
          CBList[name] = cbt;
          IdGraphMap[cb_id] = name;
          cb_id++;
          cbt.button->Associate ( this );
          theTab->AddFrame ( cbt.button, new TGTableLayoutHints(i+1, i+2, row, row+1,  layout_opts, 0, 0, 2, 2));
        }
        row++;
      }
    };


    add_tab("counter",
        [&](std::string name) 
        { 
          //return std::regex_search(name,sm, counter);
          return counter_re.ismatch(name.c_str());
        });

    add_tab("coincidence", [&](std::string name) 
        { 
          return      coincidence_re.ismatch(name.c_str()) 
                  && !coincidence_load_re.ismatch(name.c_str()) 
                  && !normalized_re.ismatch(name.c_str()); 
        });

    add_tab("correlated", [&](std::string name) 
        { 
          return     correlated_re.ismatch(name.c_str())
                  && !normalized_re.ismatch(name.c_str());
        });

    add_tab("random", [&](std::string name) 
        { 
          return     random_re.ismatch(name.c_str())
                  && !normalized_re.ismatch(name.c_str());
        });

    add_tab("load", [&](std::string name) 
        { 
          return     load_re.ismatch(name.c_str()) 
                  && !coincidence_load_re.ismatch(name.c_str());
        });

    add_tab("coincidence_load", [&](std::string name) 
        { 
          return   coincidence_load_re.ismatch(name.c_str());
        });

    add_tab("normalized", [&](std::string name) 
        { 
          return  normalized_re.ismatch(name.c_str()); 
        });

    TGCompositeFrame * vacuum_tab = tab->AddTab("vacuum");
    TGHorizontalFrame * vacuum_hf = new TGHorizontalFrame(vacuum_tab);
    vacuum_tab->AddFrame(vacuum_hf, lt.get());
    TGGroupFrame * vacuum_E  = new TGGroupFrame(vacuum_tab,"experimental area",kVerticalFrame);
    TGGroupFrame * vacuum_N  = new TGGroupFrame(vacuum_tab,"north area",kVerticalFrame);
    TGGroupFrame * vacuum_S  = new TGGroupFrame(vacuum_tab,"south area",kVerticalFrame);
    TGGroupFrame * vacuum_T  = new TGGroupFrame(vacuum_tab,"technical area",kVerticalFrame);
    vacuum_hf->AddFrame(vacuum_T, lt.get());
    vacuum_hf->AddFrame(vacuum_S, lt.get());
    vacuum_hf->AddFrame(vacuum_E, lt.get());
    vacuum_hf->AddFrame(vacuum_N, lt.get());
    ibn::regex vacuumE_re("vacuum-v4-E");
    ibn::regex vacuumN_re("vacuum-v4-N");
    ibn::regex vacuumS_re("vacuum-v4-S");
    ibn::regex vacuumT_re("vacuum-v4-T");
    TGGroupFrame * tmp_group_frame;
    //for(GraphList_t::iterator I=NonBunchGraphList.begin(); I!=NonBunchGraphList.end(); I++) 
    for(auto & p : ECV->NonBunchGraphList)
    {
      tmp_group_frame = 0;
      if(vacuumE_re.ismatch(p.first.c_str()))  tmp_group_frame = vacuum_E;
      if(vacuumN_re.ismatch(p.first.c_str()))  tmp_group_frame = vacuum_N;
      if(vacuumS_re.ismatch(p.first.c_str()))  tmp_group_frame = vacuum_S;
      if(vacuumT_re.ismatch(p.first.c_str()))  tmp_group_frame = vacuum_T;
      if(tmp_group_frame)
      {
        CheckButton_t cbt = { cb_id,  new TGCheckButton( tmp_group_frame, p.first.c_str(), cb_id )}; 
        CBList[p.first] = cbt;
        IdGraphMap[cb_id] = p.first;
        cbt.button->Associate ( this );
        tmp_group_frame->AddFrame ( cbt.button, lt.get());
        cb_id++;
      }
    }
    {
      std::string graph_name="vacuum-medium";
      CheckButton_t cbt = { cb_id,  new TGCheckButton( vacuum_tab, graph_name.c_str(), cb_id )}; 
      CBList[graph_name] = cbt;
      IdGraphMap[cb_id] = graph_name;
      cbt.button->Associate ( this );
      vacuum_tab->AddFrame ( cbt.button , lt.get());
      cb_id++;
    }
    {
      std::string graph_name="vacuum-v4-med";
      CheckButton_t cbt = { cb_id,  new TGCheckButton( vacuum_tab, graph_name.c_str(), cb_id )}; 
      CBList[graph_name] = cbt;
      IdGraphMap[cb_id] = graph_name;
      cbt.button->Associate ( this );
      vacuum_tab->AddFrame ( cbt.button , lt.get());
      cb_id++;
    }

    TGCompositeFrame * advanced_tab = tab->AddTab("advanced");
    create_f.reset(new TGHorizontalFrame(advanced_tab, 300,20));
    xdata_combo.reset(new TGComboBox(create_f.get(), COMBO_XDATA));
    ydata_combo.reset(new TGComboBox(create_f.get(), COMBO_YDATA));
    create_f->AddFrame(xdata_combo.get(), lt.get());
    create_f->AddFrame(ydata_combo.get(), lt.get());
    xdata_combo->Resize(100,20);
    ydata_combo->Resize(100,20);

    /*  fill combo box */
    int combo_id=0;
    xdata_combo->AddEntry("Select data for axis x", combo_id);
    ydata_combo->AddEntry("Select data for axis y", combo_id);
    combo_id++;
    //for(std::list<Data>::iterator it=ECV->data.begin(); it!=ECV->data.end();it++)
    for(auto data : ECV->DataList)
    {
      xdata_combo->AddEntry(data->name.c_str(), combo_id);
      ydata_combo->AddEntry(data->name.c_str(), combo_id);
      combo_map[combo_id] = data.get();
      combo_id++;
    }

    xdata_combo->Select(0);
    ydata_combo->Select(0);
    create_tb.reset(new TGTextButton(create_f.get(),"Create", TB_CREATE));
    create_tb->Associate(this);
    create_f->AddFrame(create_tb.get(), lt.get());
    advanced_tab->AddFrame(create_f.get(), lt.get());
    //TGListTree* listtree = new TGListTree(advanced_tab, 200, 600, kSunkenFrame);

    FillCheckButtons();
    AddFrame(tab.get());

    //btn_f.reset(new  TGHorizontalFrame(cf));
    //RareButton.reset(new TGTextButton(btn_f.get(),"Rare", TB_RARE));
    //RareButton->Associate(this);


    //btn_f->AddFrame(RareButton.get());
    //cf->AddFrame(btn_f.get(),lt.get());
    //new TGCheckButton( correlated_tab, new TGHotString(""), cb_id )

    TGCompositeFrame * output_tab = tab->AddTab("output");
    output_tab->SetLayoutManager(new TGMatrixLayout(output_tab,0,BN+1,5,5));
    output_tab->AddFrame(new TGLabel(output_tab,"bunch"));
    for( size_t i=0; i<BN; i++)   
    {
      output_tab->AddFrame(new TGLabel(output_tab, setString(i+1).c_str()));
    }
    for(unsigned pair = 0; pair<ECV->storage->numcounter/2;pair++)
    {
      output_tab->AddFrame(new TGLabel(output_tab, (std::string("pair-")+std::to_string(pair+1)).c_str()));
      for(unsigned bunch=0;bunch<ECV->storage->numbunch;bunch++)
      {
        CheckButton2_t cb = {&(ECV->counter_output[bunch][pair]),new TGCheckButton( output_tab, new TGHotString(""), cb_id )};
        CBFlag[cb_id] = cb;
        cb.button->Associate(this);
        output_tab->AddFrame(cb.button);
        cb_id++;
      }
    }
    TGCompositeFrame * filter_tab = tab->AddTab("filter");
    skip_bad_point_cb_id = cb_id++;
    skip_bad_point_cb.reset(new TGCheckButton(filter_tab, "Edit bad points", skip_bad_point_cb_id));
    skip_bad_point_cb->Associate(this);
    filter_tab->AddFrame(skip_bad_point_cb.get(), lt.get());

    draw_notes_cb_id = cb_id++;
    draw_notes_cb.reset(new TGCheckButton(filter_tab, "Draw notes", draw_notes_cb_id));
    draw_notes_cb->Associate(this);
    if(ECV->draw_notes) draw_notes_cb->SetState(kButtonDown);
    filter_tab->AddFrame(draw_notes_cb.get(), lt.get());

    Layout();
    SetWindowName("Graphics");
    MapSubwindows();
    MapWindow();
    Resize ( GetDefaultSize() );
  }

  DataViewDialog::~DataViewDialog(void)   
  {
    /* Delete buttons in list */
    for ( CBList_t::iterator p = CBList.begin(); p != CBList.end() ; p++ )	
    {
      delete p->second.button;
      p->second.button=0;
    }
  }

  void DataViewDialog::FillCheckButtons(void)	
  {
    //for ( GraphList_t::const_iterator p = GraphList.begin(); p != GraphList.end(); p++)	
    for ( auto &  p : ECV->GraphList)	
    {
      CBList_t::iterator cb = CBList.find(p.first);
      if(cb==CBList.end())
      {
        std::clog << LOG(4) << "No " << p.first << " in CheckButton list (CBList)\n";
        continue;
      }
      if ( ECV->IsExist ( p.first ) ) 
      {
        cb->second.button->SetState ( kButtonDown );
      }
      else 
      {
        cb->second.button->SetState ( kButtonUp );
      }
    };
  }

  void DataViewDialog::UpdateView(void)	
  {
    ECV->Update();

    for ( CBList_t::iterator p = CBList.begin(); p != CBList.end() ; p++ )	
    {
      if ( p->second.button -> GetState () == kButtonDown ) 
      {
        //XY xy = DataList[p->first];
        //ECV->Add ( p->first, p->first + "_g", xy.X, xy.Y, canvas_size_x, canvas_size_y );
        ECV->Add(p->first);
      }
      else ECV->DeleteCanvas ( p->first );
    }
    ECV->Draw();
    ECV->Update();
  }

  void DataViewDialog::AdvancedDraw(void)
  {
    int xcombo_id = xdata_combo->GetSelected();
    int ycombo_id = ydata_combo->GetSelected();
    if(xcombo_id==0 || ycombo_id ==0 )
    {
      std::clog << LOG(1) << "Nothing to draw. Select data\n";
      return;
    }
    gui::Data  *xdata = combo_map[xcombo_id];
    gui::Data  *ydata = combo_map[ycombo_id];
    //string name = ydata->name+"-vs-"+xdata->name;
    std::string name = ydata->name;
    std::cout << LOG(100) << xdata->name << " " << ydata->name << std::endl;
    std::clog << LOG(3) << "Adding graphic " << std::endl;
    //ECV->AddGraphic(name,xdata,ydata);
    ECV->Add(name, name,xdata,ydata)->Draw();
    //clog << LOG(3) << "Drawing graphic " << endl;
    ECV->Update();
  }

  void DataViewDialog::CloseWindow(void)
  {	
    while(lockWindow) 
    {
      std::cout << "window locked" << std::endl;
      usleep(100000);
    }
    delete this;
  }

  Bool_t DataViewDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
  {
    switch( GET_MSG(msg) )	
    {
      case kC_COMMAND:
        switch (GET_SUBMSG(msg)) 
        {
          case kCM_BUTTON:
            switch(parm1)  
            {
              case TB_RARE:
                std::cout << "Rare data" << std::endl;
                ECV->Rare();
                std::cout << "Done" << std::endl;
                break;
              case TB_CREATE:
                AdvancedDraw();
                break;

                /*
                   case TB_OK:
                   UpdateView();
                   CloseWindow();
                   break; // end of B_SCAN
                   case TB_APPLY:
                   UpdateView();
                   break;
                   case TB_CANCEL:
                   CloseWindow();
                   break;
                   case TB_RESET:
                   FillCheckButtons();
                   break;
                   */
            } //end of switch parm1
            break; //end of kCM_BUTTON
          case kCM_CHECKBUTTON:
            {
              std::map < unsigned , std::string >::iterator it = IdGraphMap.find(parm1); /*  find appropriate graphic */ 
              if(it!=IdGraphMap.end())  /*  have found */
              {
                lockWindow =true;
                std::string gr_name = it->second;
                TGCheckButton * Cb = CBList[gr_name].button;
                if ( Cb -> GetState () == kButtonDown ) ECV->Add (gr_name);
                else ECV->DeleteCanvas ( gr_name );
                ECV->Draw();
                lockWindow = false;
              }
              else 
              {
                CBFlag_t::iterator flit=CBFlag.find(parm1);
                if(flit!=CBFlag.end()) 
                  *(flit->second.flag)=flit->second.button->GetState()==kButtonDown ? 1 : 0;
                else 
                {
                    if(parm1==skip_bad_point_cb_id)
                    {
                      if(skip_bad_point_cb->GetState()==kButtonDown)
                        ECV->edit_bad_points=true;
                      else
                      {
                        std::cout << LOG(1) << "reseting bad point list\n";
                        ECV->edit_bad_points=false;
                        fit::BadPointList.clear();
                      }
                    }
                    if(parm1==draw_notes_cb_id)
                    {
                      if(draw_notes_cb->GetState()==kButtonDown)
                        ECV->draw_notes=true;
                      else 
                        ECV->draw_notes=false;
                    }
                }
              }
            }//kCM_CHECKBUTTON
            break;
        }
        break;
    }
    return kTRUE;
  }

  void ReadBadPointList(std::istream & is)
  {
    unsigned run;
    std::clog << LOG(1) << "Reading bad point list: ";
    while(is>>run) 
    {
      fit::BadPointList.push_back(run);
      LOG << run << " ";
    }
    std::clog << LOG(1) << std::endl;
  }
}
