/*
 * =====================================================================================
 *
 *       Filename:  DelayLineDialog.cpp
 *
 *    Description:  Implementation of DelayLineDialog
 *
 *        Version:  1.0
 *        Created:  09/23/2008 04:25:56 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physics
 *
 * =====================================================================================
 */

#include "DelayLineDialog.h"

#include <sstream>
using namespace std;
namespace gui
{
  DelayLineDialog::DelayLineDialog(const TGWindow *main, camac::EnergyCalibrationCamac * cam, measurement::EnergyCalibration * ec) 
    : 	TGTransientFrame(gClient->GetRoot(),main)
  {
    camac=cam;
    EC = ec;
    unsigned nbunch=camac->GetBunchNumber();
    unsigned ncounter=camac->GetCounterNumber();
    number_entry.resize(nbunch);
    for(unsigned b=0;b<nbunch;b++) number_entry[b].resize(ncounter/2);
    TGGroupFrame * entry_frame = new TGGroupFrame(this, "Delay, ns");
    entry_frame_ptr=auto_ptr<TGGroupFrame>(entry_frame);
    entry_frame->AddFrame(new TGLabel(entry_frame,"counter\\bunch"));
    for(unsigned b =0;b<nbunch; b++)
		{
      entry_frame->AddFrame(new TGLabel(entry_frame,std::to_string(b+1).c_str()));
    }
    for(unsigned c=0;c<ncounter/2;c++)
		{
      ostringstream os;
      os<<"c"<<2*c+1<<"&"<<2*c+2;
      entry_frame->AddFrame(new TGLabel(entry_frame,os.str().c_str()));
      for(unsigned b=0;b<nbunch;b++){
        number_entry[b][c]=new TGNumberEntry(entry_frame,camac->GetBunchDelay(c,b),5,c*number_entry.size()+b,TGNumberFormat::kNESInteger);
        number_entry[b][c]->Associate(this);
        entry_frame->AddFrame(number_entry[b][c]);
      }
    }

    entry_frame->SetLayoutManager(new TGMatrixLayout(entry_frame,ncounter/2+1,nbunch+1,2,2));
    AddFrame(entry_frame);


    TGLayoutHints * layout = new TGLayoutHints(kLHintsExpandX,2,2,2,2);
    auto_ptr<TGLayoutHints> layout_optr(layout);
    
    toggle_measurement.reset(new TGCheckButton(this, "Toggle measurement"));
    toggle_measurement->Associate(this);
    if(EC->IsDelayLineMeasurement()) toggle_measurement->SetState(kButtonDown);
    else toggle_measurement->SetState(kButtonUp);
    AddFrame(toggle_measurement.get(),layout);


    TGHorizontalFrame * button_frame=new TGHorizontalFrame(this);
    button_frame_ptr=auto_ptr<TGHorizontalFrame>(button_frame);
    TGButton*apply_button = new TGTextButton(button_frame ," Apply ",TB_APPLY);
    TGButton*ok_button = new TGTextButton(button_frame ," Ok ",TB_OK);
    apply_button_ptr=auto_ptr<TGButton>(apply_button);
    ok_button_ptr=auto_ptr<TGButton>(ok_button);
    apply_button->Associate(this);
    ok_button->Associate(this);

    button_frame->AddFrame(apply_button,layout);
    button_frame->AddFrame(ok_button,layout);
    AddFrame(button_frame,layout);


    SetWindowName("Delay Line Configuration");
    MapSubwindows();
    MapWindow();
    Resize ( GetDefaultSize() );
  }

  DelayLineDialog::~DelayLineDialog(void){
    for(unsigned b=0; b< number_entry.size(); b++)
      for(unsigned c=0; c<number_entry[b].size();c++) delete number_entry[b][c];
  }

  void DelayLineDialog::ReadDelay(void) 
  {
    for(unsigned b=0;b<number_entry.size();b++)
      for(unsigned c=0;c<number_entry[b].size(); c++)
			{
        number_entry[b][c]->SetNumber(camac->GetBunchDelay(c,b));
      }
  }

  void DelayLineDialog::WriteDelay(void)
  {
    for(unsigned b=0;b<number_entry.size();b++)
      for(unsigned c=0;c<number_entry[b].size(); c++)
			{
        camac->SetBunchDelay(c,b,unsigned(number_entry[b][c]->GetNumber()));
      }
  }

  Bool_t DelayLineDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)	
  {
    switch( GET_MSG(msg) )
		{
      case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)
            {
              case TB_APPLY:
                WriteDelay();
                break; // end of B_SCAN
              case TB_OK:
                WriteDelay();
                //delete this;
                CloseWindow();
                break;
            }
            break;//kCM_BUTTON
          case kCM_CHECKBUTTON:
            switch(toggle_measurement->GetState())
            {
              case kButtonUp:
                if(EC->IsDelayLineMeasurement())
                  EC->ToggleDelayLineMeasurement();
                clog << "Turn OFF delay line measurement" << endl;
                break;
              case kButtonDown:
                if(!EC->IsDelayLineMeasurement())
                  EC->ToggleDelayLineMeasurement();
                clog << "Turn ON delay line measurement" << endl;
                break;
              default:
                break;
            }
            break;
        }
        break;

      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg))
				{
          case kTE_ENTER:
            {
              unsigned b=parm1%number_entry.size();
              unsigned c=parm1/number_entry.size();
              cout << "Setting delay for counter-"<<2*c+1<<"&"<<2*c+2 << " bunch-"<<b+1;
              cout << " " << number_entry[b][c]->GetNumber();
              camac->SetBunchDelay(c,b,unsigned(number_entry[b][c]->GetNumber()));
              //number_entry[b][c]->SetNumber(camac->GetBunchDelay(c,b));
              //cout <<" : " << number_entry[b][c]->GetNumber() << " ns" << endl;
              cout <<" : " << camac->GetBunchDelay(c,b) << " ns" << endl;

            }
            break;
        }
        break;
    }
    return kTRUE;
  }
}
