/*
 * =====================================================================================
 *
 *       Filename:  DepolarisatorDialog.cpp
 *
 *    Description:  DepolarizerDialog implementation
 *
 *        Version:  1.0
 *        Created:  07/07/2010 08:26:29 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */

#include <TGLabel.h>
#include "DepolarisatorDialog.h"
#include "data/Calculator.h"

namespace gui
{
  enum	
  {
    COUNT_TIME_TE,
    SCAN_TIME_TE,
    IFREQ_SCAN_TE,
    IENERGY_SCAN_TE,
    FFREQ_SCAN_TE,
    FENERGY_SCAN_TE,
    SFREQ_SCAN_TE,
    SENERGY_SCAN_TE,
    ATTENUATION_TE,
    MIN_ATTENUATION_TE,
    DEVIATION_FREQ_TE,
    DELAYLINE_TE,
    DELAYLINE2_TE,
    SHIFT_TE,
    FREQUENCY_STEP_TE,
    ALGORITHM_COMBO,
    DEPOLARIZER_COMBO,
		SHOW_AMPLITUDE_COMBO, 
    COMBO_LINEAR,
    COMBO_HARMONIC,
    COMBO_GAUSS,
    COMBO_UNIFORM,
    COMBO_MONO,
    COMBO_COMMON_DEPOLARIZER,
    COMBO_SELECTIVE_DEPOLARIZER,
    COMBO_SELECTIVE_DEPOLARIZER2,
    COMBO_ARRAY_DEPOLARIZER,
		COMBO_SHOW_ELECTRON_AMPLITUDE, 
		COMBO_SHOW_POSITRON_AMPLITUDE, 
    HARMONIC_NUMBER_TE,
    HARMONIC_SIGN_TE
  };

  enum /*  Buttons ID */
  {
    OK_BUTTON,
    APPLY_BUTTON,
    CANCEL_BUTTON,
    INIT_BUTTON,
    START_BUTTON,
    CONTINUE_BUTTON,
    MUTE_BUTTON,
    STOP_BUTTON
  };

  DepolarisatorDialog::DepolarisatorDialog(const TGWindow *main, measurement::DepolarizerConfig * dcfg, camac::EnergyCalibrationCamac * ecc) : 	TGTransientFrame(gClient->GetRoot(),main,320,170, kFixedWidth)	
  {
		std::clog << LOG(DEBUG_LOG) << "DepolarisatorDialog::DepolarisatorDialog BEGIN" << std::endl;
    cfg=dcfg;
    ECCamac=ecc;
    Depol=ECCamac->depolarizer;


    LHGeneral.reset(new TGLayoutHints(kLHintsTop | kLHintsExpandX,5,5,5,5));

    ScanGroupFrame.reset(new TGGroupFrame(this, new TGString("Depolarisator parameters"),kHorizontalFrame));
    ScanGroupFrame->SetLayoutManager(new TGMatrixLayout(ScanGroupFrame.get(),0,2,5,0));

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Depolarizer type")));
    DepolarizerCombo.reset(new TGComboBox(ScanGroupFrame.get(), DEPOLARIZER_COMBO));
		DepolarizerCombo->Associate(this);
    ScanGroupFrame->AddFrame(DepolarizerCombo.get());
    DepolarizerCombo->AddEntry("standart  depolarizer \"Generator0\" (model DDS300)", COMBO_COMMON_DEPOLARIZER);
    DepolarizerCombo->AddEntry("e- depolarizer \"Generator1\" (model DDS500)", COMBO_SELECTIVE_DEPOLARIZER);
    DepolarizerCombo->AddEntry("e+ depolarizer \"Generator2\" (model DDS500)", COMBO_SELECTIVE_DEPOLARIZER2);
    DepolarizerCombo->AddEntry("e+ and e- depolarizer \"Generator1/Generator2\" (models DDS500)", COMBO_ARRAY_DEPOLARIZER);
		cout << Depol->generator->GetName() << std::endl;
    cout << LOG(1) << "Current depolarizer subtype=" << Depol->generator->GetSubType() << " with name " << Depol->generator->GetName() << "." << std::endl;
    if(string(Depol->generator->GetName()) == "Generator0") DepolarizerCombo->Select(COMBO_COMMON_DEPOLARIZER);
    if(string(Depol->generator->GetName()) == "Generator1") DepolarizerCombo->Select(COMBO_SELECTIVE_DEPOLARIZER);
    if(string(Depol->generator->GetName()) == "Generator2") DepolarizerCombo->Select(COMBO_SELECTIVE_DEPOLARIZER2);
    if(string(Depol->generator->GetName()) == "SelectiveGeneratorDouble") DepolarizerCombo->Select(COMBO_ARRAY_DEPOLARIZER);
    DepolarizerCombo->Resize(400, 20);

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Show depolarizer amlitude")));
		clog << LOG(2) << "Before show amplitude combo" << std::endl;
    ShowAmplitudeCombo.reset(new TGComboBox(ScanGroupFrame.get(), SHOW_AMPLITUDE_COMBO));
		ShowAmplitudeCombo->Associate(this);
		ScanGroupFrame->AddFrame(ShowAmplitudeCombo.get());
		ShowAmplitudeCombo->AddEntry("electron channel", COMBO_SHOW_ELECTRON_AMPLITUDE);
		ShowAmplitudeCombo->AddEntry("positron channel", COMBO_SHOW_POSITRON_AMPLITUDE);
		switch(ECCamac->WhichAmplitudeToShow())
		{
			case 0:
				ShowAmplitudeCombo->Select(COMBO_SHOW_ELECTRON_AMPLITUDE);
				break;
			case 1:
				ShowAmplitudeCombo->Select(COMBO_SHOW_POSITRON_AMPLITUDE);
				break;
		}
		clog << LOG(2) << "After show amplitude combo" << std::endl;
    ShowAmplitudeCombo->Resize(250, 20);


    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Scan Algorithm")));
    AlgorithmCombo.reset(new TGComboBox(ScanGroupFrame.get(), ALGORITHM_COMBO));
		AlgorithmCombo->Associate(this);
    ScanGroupFrame->AddFrame(AlgorithmCombo.get());
    AlgorithmCombo->AddEntry("Linear", measurement::Depolarisator::LINEAR);

    AlgorithmCombo->AddEntry("Gauss random",measurement::Depolarisator::GRND);
    AlgorithmCombo->AddEntry("Uniform random",measurement::Depolarisator::URND);
    AlgorithmCombo->AddEntry("Monotonic",measurement::Depolarisator::MONO);

    AlgorithmCombo->Select(Depol->GetStrategy());

    AlgorithmCombo->Resize(200, 20);


		std::cout << LOG(2) << "Before get attenuation" << std::endl;
    AttenuationTE.reset(new TGNumberEntry(ScanGroupFrame.get(),Depol->GetAttenuation(),12,ATTENUATION_TE,TGNumberFormat::kNESInteger));
		std::cout << LOG(2) << "After get attenuation" << std::endl;
    AttenuationTE->Associate(this);
    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Attenuation, dB")));
    ScanGroupFrame->AddFrame(AttenuationTE.get());

		std::cout << LOG(2) << "Before get minimum attenuation" << std::endl;
    MinAttenuationTE.reset(new TGNumberEntry(ScanGroupFrame.get(),Depol->GetMinimumAttenuation(),12,MIN_ATTENUATION_TE,TGNumberFormat::kNESInteger));
		std::cout << LOG(2) << "After get minimum attenuation" << std::endl;
    MinAttenuationTE->Associate(this);
    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Minimum Attenuation, dB")));
    ScanGroupFrame->AddFrame(MinAttenuationTE.get());

		std::cout << LOG(2) << "Before get Delay" << std::endl;
    DelayLineTE.reset(new TGNumberEntry(ScanGroupFrame.get(),ECCamac->DepolarizerDelayLine->GetDelay(),12,DELAYLINE_TE,TGNumberFormat::kNESInteger, TGNumberFormat::kNEAPositive, TGNumberFormat::kNELLimitMinMax, 0, 1275));
		std::cout << LOG(2) << "After get Delay" << std::endl;
    DelayLineTE->Associate(this);

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Depolarizer Delay Line, ns")));
    ScanGroupFrame->AddFrame(DelayLineTE.get());

		std::cout << LOG(2) << "Before get Delay2" << std::endl;
    DelayLine2TE.reset(new TGNumberEntry(ScanGroupFrame.get(),ECCamac->DepolarizerDelayLine2->GetDelay(),12,DELAYLINE2_TE,TGNumberFormat::kNESInteger, TGNumberFormat::kNEAPositive, TGNumberFormat::kNELLimitMinMax, 0, 1275));
		std::cout << LOG(2) << "After get Delay2" << std::endl;
    DelayLine2TE->Associate(this);

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("Depolarizer Delay Line 2, ns")));
    ScanGroupFrame->AddFrame(DelayLine2TE.get());

		std::clog << LOG(2) << "Before GetShift " << std::endl;
    ShiftTE.reset(new TGNumberEntry(ScanGroupFrame.get(),ECCamac->double_generator->GetShift(),12,SHIFT_TE));
    ShiftTE->Associate(this);
		std::clog << LOG(2) << "After GetShift " << std::endl;

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("shift, Hz")));
    ScanGroupFrame->AddFrame(ShiftTE.get());

		double fspeed = Depol->GetScanSpeed();
		double speed = 1e6*data::dfrequency2energy(Depol->GetFrequencyConfig(),  fspeed);
    FrequencyStep.reset(new TGNumberEntry(ScanGroupFrame.get(),speed,12,FREQUENCY_STEP_TE));
    FrequencyStep->Associate(this);

    ScanGroupFrame->AddFrame(new TGLabel(ScanGroupFrame.get(), new TGHotString("scan speed, eV/sec")));
    ScanGroupFrame->AddFrame(FrequencyStep.get());

    AddFrame(ScanGroupFrame.get(), LHGeneral.get());

    ButtonFrame.reset(new TGHorizontalFrame ( this, 300,200));

    InitButton.reset(new TGTextButton(ButtonFrame.get(), "INIT", INIT_BUTTON));
    OkButton.reset(new TGTextButton(ButtonFrame.get(), "Ok", OK_BUTTON));
    ApplyButton.reset(new TGTextButton(ButtonFrame.get(), "Aply", APPLY_BUTTON));
    CancelButton.reset(new TGTextButton(ButtonFrame.get(), "Cancel", CANCEL_BUTTON));
    InitButton->Associate(this);
    OkButton->Associate(this);
    ApplyButton->Associate(this);
    CancelButton->Associate(this);
    ButtonFrame->AddFrame(InitButton.get(),LHGeneral.get());
    ButtonFrame->AddFrame(OkButton.get(),LHGeneral.get());
    ButtonFrame->AddFrame(ApplyButton.get(),LHGeneral.get());
    ButtonFrame->AddFrame(CancelButton.get(),LHGeneral.get());
    AddFrame(ButtonFrame.get(), LHGeneral.get());

    /*  Scan control buttons */
    ScanButtonFrame.reset(new TGHorizontalFrame ( this, 300,200));

    StartButton.reset(new TGTextButton(ScanButtonFrame.get(), "START", START_BUTTON));
    StopButton.reset(new TGTextButton(ScanButtonFrame.get(), "STOP", STOP_BUTTON));
    ContinueButton.reset(new TGTextButton(ScanButtonFrame.get(), "CONTINUE", CONTINUE_BUTTON));
    MuteButton.reset(new TGTextButton(ScanButtonFrame.get(), "MUTE", MUTE_BUTTON));
    StartButton->Associate(this);
    StopButton->Associate(this);
    ContinueButton->Associate(this);
    MuteButton->Associate(this);
    ScanButtonFrame->AddFrame(StartButton.get(), LHGeneral.get());
    ScanButtonFrame->AddFrame(StopButton.get(), LHGeneral.get());
    ScanButtonFrame->AddFrame(ContinueButton.get(),LHGeneral.get());
    ScanButtonFrame->AddFrame(MuteButton.get(), LHGeneral.get());
    AddFrame(ScanButtonFrame.get(), LHGeneral.get());

    Layout();
    SetWindowName("Depolarizer configuration");
    MapSubwindows();
    MapWindow();
    Resize(840,311);
		std::clog << LOG(DEBUG_LOG) << "DepolarisatorDialog::DepolarisatorDialog END" << std::endl;
  }

  void DepolarisatorDialog::SetAttenuation(void)
  {
    try 
    {
      Depol->SetAttenuation(AttenuationTE->GetIntNumber());
      unsigned attenuation = unsigned(Depol->GetAttenuation());
      std::cout << LOG(NORMAL_LOG) << "Set Attenuation: " << attenuation << " dB" << std::endl;
      AttenuationTE->SetIntNumber(attenuation);
      cfg->attenuation=attenuation;
    } 
    catch (camac::Module::Error e)
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << std::endl;
    }
  }

  void DepolarisatorDialog::SetFrequencyStep(void)
  {
    try 
    {
      Depol->SetFrequencyStep(FrequencyStep->GetNumber());
			cout << "Set frequency step: " << Depol->GetFrequencyStep() << " Hz" << std::endl;
    } 
    catch (camac::Module::Error e)
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << '\n';
    }
  }

  void DepolarisatorDialog::SetScanSpeed(void)
  {
    try 
    {
			double speed = FrequencyStep->GetNumber();
			double fspeed = data::denergy2frequency(Depol->GetFrequencyConfig(),speed*1e-6);
      Depol->SetScanSpeed(fspeed);
			cout << "Set scan speed: " << data::dfrequency2energy(Depol->GetFrequencyConfig(),  Depol->GetScanSpeed())*1e6 << " eV/sec" << std::endl;
    } 
    catch (camac::Module::Error e)
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << std::endl;
    }
  }

  void DepolarisatorDialog::SetMinimumAttenuation(void)
  {
    Depol->SetMinimumAttenuation(MinAttenuationTE->GetIntNumber());
    MinAttenuationTE->SetIntNumber(unsigned(Depol->GetMinimumAttenuation()));
    cout << LOG(NORMAL_LOG) << "Set minimum attenuation to " << Depol->GetMinimumAttenuation() << " dB" << std::endl;
  }

  Bool_t DepolarisatorDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)  
  {
    switch( GET_MSG(msg) )	
    {
      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg))
        {
          case kTE_ENTER:
            switch(parm1) 
            {
              case ATTENUATION_TE:
                SetAttenuation();
                break;
              case MIN_ATTENUATION_TE:
                SetMinimumAttenuation();
                break;
              case DELAYLINE_TE:
                SetDepolarizerDelay();
                break;
              case DELAYLINE2_TE:
                SetDepolarizerDelay2();
                break;
              case SHIFT_TE:
                SetShift();
                break;
							case FREQUENCY_STEP_TE:
								//SetFrequencyStep();
								SetScanSpeed();
								break;
            }
            break;
          case kTE_TEXTCHANGED:
            switch(parm1)
            {
              //case ATTENUATION_TE:
              //  try 
              //  {
              //    unsigned new_attenuation = AttenuationTE->GetIntNumber();
              //    unsigned old_attenuation = Depol->GetAttenuation();
              //    if(fabs(new_attenuation-old_attenuation)==1)
              //    {
              //      Depol->SetAttenuation(new_attenuation);
              //      unsigned attenuation = unsigned(Depol->GetAttenuation());
              //      std::cout << LOG(NORMAL_LOG) << "Set Attenuation: " << attenuation << " dB" << std::endl;
              //      AttenuationTE->SetIntNumber(attenuation);
              //      cfg->attenuation=attenuation;
              //    }
              //  } 
              //  catch (camac::Module::Error e)
              //  {
              //    std::cerr << LOG(1) << "WARNING: " << e.what() << '\n';
              //  }
              //  break;
              case MIN_ATTENUATION_TE:
                SetMinimumAttenuation();
                break;

                //case DELAYLINE_TE:
                //  SetDepolarizerDelay();
                //  break;
            }
            break;
        } 
        break; 
      case kC_COMMAND:
        switch (GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)	
            {
              case INIT_BUTTON:
                Depol->generator->Init();
                break;
              case OK_BUTTON:
                Update();
                CloseWindow();
                break;
              case APPLY_BUTTON:
                Update();
                break;
              case CANCEL_BUTTON:
                CloseWindow();
                break;
              case START_BUTTON:
                Depol->Start();
                break;
              case STOP_BUTTON:
                Depol->Stop();
                break;
              case CONTINUE_BUTTON:
                Depol->Continue();
                break;
              case MUTE_BUTTON:
                if(Depol->IsMute())
                {
                  clog << LOG(1) << "Unmute depolarizer" << endl;
                  Depol->UnMute();
                  MuteButton->SetState(kButtonUp);
                }
                else
                {
                  clog << LOG(1) << "Mute depolarizer" << endl;
                  Depol->Mute();
                  MuteButton->SetState(kButtonDown);
                }
                break;
              default:
                break;
            }
            break;
          case kCM_COMBOBOX:
            switch(parm1)
            {
              case ALGORITHM_COMBO:
                SetStrategy();
                break;
              case DEPOLARIZER_COMBO:
								SetGenerator();
                break;
							case SHOW_AMPLITUDE_COMBO:
								SetWhichAmplitudeToShow();
								break;
              default:
                break;
            }
            break;
        } 
        break;
      case kCM_COMBOBOX:
        break;
    }
    return kTRUE;
  }
  void DepolarisatorDialog::SetDepolarizerDelay(void)
  {
    try 
    {
      ECCamac->DepolarizerDelayLine->SetDelay(DelayLineTE->GetIntNumber());
      cout << LOG(1)<<"Set depolarizer delay: " << ECCamac->DepolarizerDelayLine->GetDelay() << " ns" << endl;
    } 
    catch (camac::Module::Error e)	
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << '\n';
    }
  }

  void DepolarisatorDialog::SetDepolarizerDelay2(void)
  {
    try 
    {
      ECCamac->DepolarizerDelayLine2->SetDelay(DelayLine2TE->GetIntNumber());
      cout << LOG(1)<<"Set depolarizer delay2: " << ECCamac->DepolarizerDelayLine2->GetDelay() << " ns" << endl;
    } 
    catch (camac::Module::Error e)	
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << '\n';
    }
  }

  void DepolarisatorDialog::SetShift(void)
  {
    camac::GeneratorArray * g = ECCamac->double_generator;
    try 
    {
      g->SetShift(ShiftTE->GetNumber());
      cout << LOG(1)<<"Set shift: " << g->GetShift() << " Hz (" << 1e3*data::dfrequency2energy(cfg, g->GetShift()) << " keV)" << endl;
    } 
    catch (camac::Module::Error e)	
    {
      std::cerr << LOG(1) << "WARNING: " << e.what() << endl;
    }
  }

  void DepolarisatorDialog::SetStrategy(void)
  {
    Depol->SetStrategy( measurement::Depolarisator::STRATEGY(AlgorithmCombo->GetSelected()));
    std::string strategy;
    switch(Depol->GetStrategy())
    {
      case measurement::Depolarisator::LINEAR: 
        strategy="LINEAR"; 
        break;
      case measurement::Depolarisator::GRND:
        strategy="GAUSS RANDOM";
        break;
      case measurement::Depolarisator::URND:
        strategy="UNIFORM RANDOM";
        break;
      case measurement::Depolarisator::MONO:
        strategy="MONOTONIC";
        break;
    }
    std::cout << LOG(1) << "Set depolarization strategy: " << strategy << std::endl;
  }

  void DepolarisatorDialog::SetGenerator(void)
  {
    switch(DepolarizerCombo->GetSelected())
    {
      case COMBO_SELECTIVE_DEPOLARIZER: 
        std::cout << LOG(1) << "Set Depolarizer: selective depolarizer DDS500" << std::endl;
        Depol->SetGenerator(ECCamac->generatorDDS500);
        break;
      case COMBO_SELECTIVE_DEPOLARIZER2: 
        std::cout << LOG(1) << "Set Depolarizer: second selective depolarizer DDS500" << std::endl;
        Depol->SetGenerator(ECCamac->generatorDDS500_2);
        break;
      case COMBO_COMMON_DEPOLARIZER: 
        std::cout << LOG(1) << "Set Depolarizer: common depolarizer DDS300" << std::endl;
        Depol->SetGenerator(ECCamac->generatorDDS300);
        break;
      case COMBO_ARRAY_DEPOLARIZER: 
        std::cout << LOG(1) << "Set Depolarizer: array generator" << std::endl;
        Depol->SetGenerator(ECCamac->double_generator);
        break;
    }
  }

	void DepolarisatorDialog::SetWhichAmplitudeToShow(void)
	{
		switch(ShowAmplitudeCombo->GetSelected())
		{
			case COMBO_SHOW_ELECTRON_AMPLITUDE:
				std::cout << LOG(1) << "Show electron depolarizer amplitude." << std::endl;
				ECCamac->ShowElectronDepolarizerAmplitude();
				break;
			case COMBO_SHOW_POSITRON_AMPLITUDE:
				std::cout << LOG(1) << "Show positron depolarizer amplitude." << std::endl;
				ECCamac->ShowPositronDepolarizerAmplitude();
				break;
			default:
				std::cout << LOG(1) << "Unable to set up which depolarizer amplitude to show" << std::endl;
				break;
		}
	}



  void DepolarisatorDialog::Update(void)
  {
    //SetHarmonicNumber();
    SetDepolarizerDelay();
    SetAttenuation();
    SetStrategy();
    SetGenerator();
		SetWhichAmplitudeToShow();
  }

}//end of namespace gui
