#ifndef IBN_GUI_DEPOLARISATOR_DIALOG_H
#define IBN_GUI_DEPOLARISATOR_DIALOG_H

#include <memory>

#include <TGFrame.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGNumberEntry.h>
#include <TGComboBox.h>

#include "Utils.h"
#include "Log.h"


#include "data/Calculator.h"
#include "data/EnergyCalibrationStorage.h"
#include "measurement/Depolarisator.h"

#include "measurement/EnergyCalibrationCamac.h"

namespace gui
{

  class DepolarisatorDialog : public TGTransientFrame 
  {
    camac::EnergyCalibrationCamac * ECCamac; //harwdware
    measurement::Depolarisator * Depol; //depolarizator
    measurement::DepolarizerConfig * cfg; //Depolarizer config from EnergyCalibrationStorage::cfg

    unique_ptr <TGLayoutHints> LHGeneral;

    unique_ptr<TGGroupFrame> ScanGroupFrame;
    unique_ptr<TGHorizontalFrame> ButtonFrame;

    unique_ptr <TGComboBox> AlgorithmCombo;
		unique_ptr <TGComboBox> DepolarizerCombo; /* widget to select type of depolarizer*/
		unique_ptr <TGComboBox> ShowAmplitudeCombo; /* widget to select which amplitude to show */
    unique_ptr <TGNumberEntry> AttenuationTE;
    unique_ptr <TGNumberEntry> MinAttenuationTE;
    unique_ptr <TGNumberEntry> DelayLineTE;
    unique_ptr <TGNumberEntry> DelayLine2TE;
    unique_ptr <TGNumberEntry> ShiftTE;
		unique_ptr <TGNumberEntry> FrequencyStep;

    unique_ptr <TGTextButton> InitButton, OkButton, ApplyButton, CancelButton;

    unique_ptr<TGHorizontalFrame> ScanButtonFrame;
    unique_ptr<TGTextButton> StartButton, ContinueButton, MuteButton, StopButton;


    public:

    DepolarisatorDialog(const TGWindow *main, measurement::DepolarizerConfig *, camac::EnergyCalibrationCamac * ecc); 

    virtual ~DepolarisatorDialog(void)   { }

    private:
    void CloseWindow(void)	
    {
      delete this;
    }

    void SetGenerator(void);
		void SetWhichAmplitudeToShow();
    void SetAttenuation(void);
		void SetFrequencyStep(void);
		void SetScanSpeed(void);
    void SetMinimumAttenuation(void);
    void SetDepolarizerDelay(void);
    void SetDepolarizerDelay2(void);
    void SetShift(void);
    //void SetHarmonicNumber(void) ;
    void SetStrategy(void);

    void Update(void);

    Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);
  };
}


#endif //IBN_GUI_DEPOLARISATOR_DIALOG_H
