// =====================================================================================
//
//       Filename:  ECStatusDialog.cpp
//
//    Description:  
//
//        Version:  1.0
//        Created:  11.12.2014 14:16:45
//       Revision:  none
//       Compiler:  g++
//
//         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
//   Organization:  Budker Institute of Nuclear Physics
//
// =====================================================================================

#include "ECStatusDialog.h"
#include "V4Par.h"
#include <Log.h>

enum
{
	BUTTON_FIT,
	BUTTON_YES, 
	BUTTON_NO
};

namespace gui
{
	ECStatusDialog::ECStatusDialog(const TGWindow *w, ECStatus * s) : TGTransientFrame(gClient->GetRoot(),w,350,150)
	{
		status = s;
		left_lhint.reset(new TGLayoutHints(kLHintsTop |  kLHintsLeft |  kLHintsExpandX , 2,2,2,2));
		right_lhint.reset(new TGLayoutHints(kLHintsTop |  kLHintsRight |  kLHintsExpandX , 2,2,2,2));
		//name of the shiftman
		fShiftmanFrame.reset(new TGHorizontalFrame(this, 200, 150));
		VeppDuty vd;
		status->shiftman = vd.duty_translit;
		fShiftmanEntry.reset( new TGTextEntry(fShiftmanFrame.get(),vd.duty_translit.c_str()));
		fShiftmanEntry->Associate(this);
		fShiftmanLabel.reset( new TGLabel(fShiftmanFrame.get(),"shiftman"));
		fShiftmanFrame->AddFrame(fShiftmanLabel.get(), left_lhint.get());
		fShiftmanFrame->AddFrame(fShiftmanEntry.get(), right_lhint.get());
		AddFrame(fShiftmanFrame.get(), left_lhint.get());


		f2UEntry.reset(new LabeledNumberEntry(this,"2U,  mV", TGNumberFormat::kNESInteger, 0,  1,  12));
		AddFrame(f2UEntry.get(),  left_lhint.get());

		fPrint.reset(new TGCheckButton(this, "print"));
		AddFrame(fPrint.get(),  left_lhint.get());
		fPrint->SetState(kButtonDown);
		fPrint->Associate(this);

		fSuccessQuestion.reset( new TGLabel(this,"Is Energy Calibration Successful?"));
		AddFrame(fSuccessQuestion.get(), left_lhint.get());

		fButtonFrame.reset(new TGHorizontalFrame(this, 200, 200));
    auto make_button = [&](std::string name, int id)
    {
      fButton[name].reset(new TGTextButton(fButtonFrame.get(), name.c_str(), id));
      fButtonFrame->AddFrame(fButton[name].get(), left_lhint.get());
      fButton[name]->Associate(this);
    };
    make_button("FIT",BUTTON_FIT);
    make_button("YES",BUTTON_YES);
    make_button("NO",BUTTON_NO);
		AddFrame(fButtonFrame.get(), left_lhint.get());

		Layout();
    MapSubwindows();
    MapWindow();
    fClient->WaitFor(this);
	}
	ECStatusDialog::~ECStatusDialog(void)
	{

	}
	Bool_t ECStatusDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
	{
		switch( GET_MSG(msg) )	
		{
			case kC_COMMAND:
				switch (GET_SUBMSG(msg)) 
				{
					case kCM_BUTTON:
						status->shiftman = fShiftmanEntry->GetText();
						std::clog << LOG(1) << "Set shiftman to: " << status->shiftman << std::endl;
						status->print = (fPrint->GetState() == kButtonDown);
						std::clog << LOG(1) << "Set print option: " << status->print << std::endl;
						status->U = 1e-3*f2UEntry->GetIntNumber();
						std::clog << LOG(1) << "Set 2U: " << status->U*1e3 << " mV" <<  std::endl;
						switch(parm1)
						{
							case BUTTON_FIT:
								status->back_to_fit=true;
								std::clog << LOG(1) << "Go back to fit" << std::endl;
								CloseWindow();
								break;
              case BUTTON_YES:
								status->success=true;
								std::clog << LOG(1) << "Success energy calibration" << std::endl;
								CloseWindow();
								break;
							case BUTTON_NO:	
								status->success=false;
								std::clog << LOG(1) << "Fail energy calibration"  << std::endl;
								CloseWindow();
								break;
							default: 
								break;
						}
						break; //end of switch param 1
				} //end of GET_SUBMSG(msg)
				break; //end of kC_COMMAND
		}//end of GET_MSG(msg)
		return kTRUE;
	}
}
