// =====================================================================================
//
//       Filename:  ECStatusDialog.h
//
//    Description:  Status of the Energy calibration
//
//        Version:  1.0
//        Created:  11.12.2014 14:14:10
//       Revision:  none
//       Compiler:  g++
//
//         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
//   Organization:  Budker Institute of Nuclear Physics
//
// =====================================================================================


#ifndef  IBN_POL_GUI_ECSTATUSDIALOG_H
#define  IBN_POL_GUI_ECSTATUSDIALOG_H
#include <memory>
#include <map>

#include <TGFrame.h>
#include <TGLabel.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGNumberEntry.h>

#include "measurement/EnergyCalibrationConfig.h"
#include "LabeledNumberEntry.h"


namespace gui
{
	struct ECStatus
	{
		bool success=false;
		bool print=true;
    bool back_to_fit=false;
		std::string shiftman="";
		double U=0;
	};

	class ECStatusDialog : public TGTransientFrame
	{
		ECStatus * status;
		public:
			ECStatusDialog(const TGWindow * w, ECStatus * s);
			~ECStatusDialog(void);
			Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 );
		private:
			TGLayoutHints * nel;

			std::unique_ptr<TGLayoutHints> left_lhint,  right_lhint;

			std::unique_ptr<TGHorizontalFrame> fShiftmanFrame; //Shiftman frame
			std::unique_ptr<TGTextEntry> fShiftmanEntry;
			std::unique_ptr<TGLabel> fShiftmanLabel; //shifter family name


			std::unique_ptr<LabeledNumberEntry> f2UEntry;

			std::unique_ptr<TGCheckButton> fPrint; //print or not

			std::unique_ptr<TGHorizontalFrame> fButtonFrame;
			std::unique_ptr<TGLabel> fSuccessQuestion; 
      std::map<std::string, std::unique_ptr<TGTextButton> > fButton;
			std::unique_ptr<TGLayoutHints> fButtonLayout;
	};
}
#endif   // ----- #ifndef ECStatusDialog_INC  ----- 
