//#=====================================================================================
//#
//#       Filename:  EnergyCalibrationView.h
//#
//#    Description:  Class wich control data view while measuring
//#
//#        Version:  1.0
//#        Created:  31.01.2003
//#       Revision:  $ID:
//#       Compiler:  GNU C/C++
//#
//#         Author:  Ivan B. Nikolaev
//#        Company:  BINP,Russia
//#          Email:  I.B.Nikolaev@inp.nsk.su
//#
//#=====================================================================================

#ifndef IBN_GUI_ENERGY_CALIBRATION_VIEW_H
#define IBN_GUI_ENERGY_CALIBRATION_VIEW_H

#include "data/EnergyCalibrationStorage.h"

#include "Canvas.h"
#include "Graph.h"

#include <memory>
#include <map>
#include <thread>

namespace gui
{

  class EnergyCalibrationView
  {
    data::EnergyCalibrationStorage * EC;
    typedef std::map<std::string, Canvas* > display_t; 
		int last_number_of_points = 0;
    public:
    display_t displays;
    bool draw_notes;
    bool edit_bad_points;
    unsigned auto_rare; //automatic raring
    data::EnergyCalibrationStorage * storage;
    typedef display_t::iterator iterator;

    std::list <std::shared_ptr<Data>> DataList; /*  data list */
    std::map <std::string, std::shared_ptr<Data>> NonBunchDataList;
    std::map <std::string, std::map<unsigned, std::shared_ptr<Data>>> CounterDataList; //CounterDataList["name"][counter]
    std::map <std::string, std::map<unsigned, std::shared_ptr<Data>>> BunchDataList; //BunchCounterDataList["name"][bunch]
    std::map <std::string, std::map<unsigned, std::map<unsigned, std::shared_ptr<Data>>>> CounterBunchDataList; //CounterBunchDataList["name"][counter][bunch]
    std::map <std::string, std::map<unsigned, std::map<unsigned, std::shared_ptr<Data>>>> PairBunchDataList; //PairBunchDataList["name"][pair][bunch]

    typedef std::map< std::string, std::shared_ptr<Graph> > GraphList_t;
    typedef std::map< std::string, std::map < size_t, std::shared_ptr<Graph> > > BunchGraphList_t;
    typedef std::map< std::string, std::map < size_t, std::shared_ptr<Graph> > > CounterGraphList_t;
    GraphList_t GraphList;
    GraphList_t NonBunchGraphList;
    BunchGraphList_t BunchGraphList;
		CounterGraphList_t CounterGraphList;

    std::vector < std::vector<int> > counter_output;


    EnergyCalibrationView(data::EnergyCalibrationStorage * ec);
		~EnergyCalibrationView(void);

    void InitDataList(void);
    void InitGraphList(void);

    Canvas * Add(std::string du_name, Data *x, unsigned w=600, unsigned h=400);	
    Canvas * Add(std::string canvas_name, std::string graphic_name, Data *x, Data *y, unsigned w=600, unsigned h=400);
    Canvas * Add(std::string canvas_name,  data::Array *x, unsigned w=600, unsigned h=400);
    Canvas * Add(std::string canvas_name, std::string graphic_name, data::Array *x, data::Array *y, unsigned w=600, unsigned h=400);
    Canvas * Add( std::string canvas_name, std::shared_ptr<Graph> g,  unsigned w, unsigned h);
    Canvas * Add( std::string n );
    bool IsCanvasExist(const std::string & name);	

		/* Remove  canvas with name from canvas list (display). This function must be
		 * call from canvas desctructor in order not to loose pointer */
    void Erase(std::string name);  

		/* Delete canvas. Deallocate memory. Canvas will be automaticaly 
		 * remove (erase) himself from canvas list*/
		void DeleteCanvas(std::string);

    Canvas * GetCanvas(std::string name);
    Graph  * GetGraph(std::string name);
    Data   * GetData(std::string name);


    //data::EnergyCalibrationStorage * GetData(void) { return EC; }
    data::EnergyCalibrationStorage * GetStorage(void) { return EC; }
    data::EnergyCalibrationStorage * GetEnergyCalibrationStorage(void) { return EC; }

		std::thread::id startthreadid;

    volatile bool updatable;
    void Update(void);
    void Notify(void);
    bool IsUpdatable(void) const { return updatable; }

    void Clear (void);

    bool IsExist(std::string nm)
    {
      return (displays.find(nm) != displays.end());
    }

    void Draw(void);

    void Print(const boost::filesystem::path);

    void Rare(unsigned rare=2);

    void ProcessedEvent(Canvas * c, Int_t event,Int_t x,Int_t y ,TObject* selected);

    iterator begin(void) { return displays.begin();}
    iterator end(void) { return displays.end();}

		void ShowLastPoints(int n) { last_number_of_points = n; }

    void OrderWindows(void);
  };

  void LoadBadPointList(std::istream &); //Read bad point list from file in text mode.
  void SaveBadPointList(std::ostream &); //Save bad point list from file in text mode.
}
#endif
