#include "Log.h"
#include "ExpectedEnergyDialog.h"
#include "Event.h"
#include "data/Calculator.h"

namespace gui
{

  ExpectedEnergyDialog::ExpectedEnergyDialog(const TGWindow * w,measurement::EnergyCalibrationConfig *o,
      measurement::EnergyCalibrationConfig *c, int * r) :
    TGTransientFrame(gClient->GetRoot(),w,200,300)
  {
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::ExpectedEnergyDialog START" << std::endl;
    old=o;
    cur=c;
    var_old = *old;
    var_cur = *cur;
    retval=r;
    char time_string[1024];
    strftime(time_string,1024,"%F-%T",localtime(&old->date));
    std::string old_name = "Previous energy measurement: " + std::string(time_string);
    gold=new EnergyEntry(this,old_name, old);
    gcur=new EnergyEntry(this,"Expected energy", cur);
    gcur->EditEnable();
    el = new TGLayoutHints(kLHintsTop |  kLHintsLeft |  kLHintsExpandX , 2,2,2,2);
    AddFrame(gold,el);
    AddFrame(gcur,el);
    gcur->ne_nmr->SetState(kFALSE);

    bf = new TGHorizontalFrame(this, 300,200);
    bl = new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX,2,2,2,2);
    b_name.push_back("<< Back");
    b_name.push_back("Reset");
    b_name.push_back("Calc");
    b_name.push_back("Next >>");
    b.resize(b_name.size());
    for(size_t i = 0; i< b.size(); i++)
    {
      b[i] = new TGTextButton(bf,b_name[i].c_str(),i);
      b[i]->Associate(this);
      bf->AddFrame(b[i],el);
    }
    b[1]->SetState(kButtonDisabled);
    b[2]->SetState(kButtonDisabled);
    AddFrame(bf,bl);
    SetWindowName((cur->name).c_str());
    MapSubwindows();
    MapWindow();
    Resize(GetDefaultSize());
    fClient->WaitFor(this);
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::ExpectedEnergyDialog FINISH" << std::endl;
  }

  ExpectedEnergyDialog::~ExpectedEnergyDialog(void)
  {
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::~ExpectedEnergyDialog START" << std::endl;
    for ( size_t i = 0; i< b.size(); i++)   delete b[i];
    delete bl;
    delete bf;
    delete el;
    delete gcur;
    delete gold;
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::~ExpectedEnergyDialog FINISH" << std::endl;
  }

  Bool_t ExpectedEnergyDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
  {
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::ProcessMessage START" << std::endl;
    switch( GET_MSG(msg) )
    {
      case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)
            {
              case 0: //back
                *retval=BACK;
                delete this;
                break;
              case 1: //reset
                var_old = *old;
                var_cur = *cur;
                gold->SetData(&var_old);
                data::GetNMR(&var_cur);
                data::GetRF(cur);
                gcur->SetData(&var_cur);
                break;
              case 2: //calc
                gold->EditEnable();
                gcur->EditEnable();
                //gold->GetData(&var_old);
                gcur->GetData(&var_cur);
                gcur->SetData(&var_cur);
                break;
              case 3:
                gcur->GetData(&var_cur);
                *cur=var_cur;
                //*old=var_old; I should not edit old
                *retval=NEXT;
                delete this;
                break;
              default:
                break;
            }
            break;
        }
        break;
    }
    return kTRUE;
    std::clog<<LOG(DEBUG_LOG)<<"ExpectedEnergyDialog::~ExpectedEnergyDialog FINISH" << std::endl;
  }

  void ExpectedEnergyDialog::CloseWindow(void)
  {
  }







  EnergyEntry::EnergyEntry(const TGWindow *w, const std::string  & name, measurement::EnergyCalibrationConfig *d, int option) 
    : TGGroupFrame(w,name.c_str())
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EnergyEntry BEGIN"<<std::endl;
    f_energy = new TGHorizontalFrame(this,300,200);
    f_nmr = new TGHorizontalFrame(this,300,200);
    lf = new TGLayoutHints(kLHintsTop | kLHintsRight |kLHintsExpandX,2,2,2,2);
    AddFrame(f_energy,lf);
    AddFrame(f_nmr,lf);

    lr=new TGLayoutHints(kLHintsRight,2,2,2,2);
    ll=new TGLayoutHints(kLHintsLeft,2,2,2,2);

    ne_error = new TGNumberEntry(f_energy, d->energy.error,12,0,TGNumberFormat::kNESRealFour);
    ne_energy = new TGNumberEntry(f_energy, d->energy.value,12,0,TGNumberFormat::kNESRealFour);
    ne_error->Associate(w);
    ne_energy->Associate(w);
    l_energy_error=new TGLabel(f_energy,"+-");
    l_energy=new TGLabel(f_energy,"E [MeV]");
    f_energy->AddFrame(l_energy,ll);
    f_energy->AddFrame(ne_error,lr);
    f_energy->AddFrame(l_energy_error,lr);
    f_energy->AddFrame(ne_energy,lr);

    ne_nmr = new TGNumberEntry(f_nmr, d->nmr,12,0,TGNumberFormat::kNESRealThree);
    ne_nmr->Associate(w);
    l_nmr = new TGLabel(f_nmr,"NMR [Gs]");
    f_nmr->AddFrame(l_nmr,ll);
    f_nmr->AddFrame(ne_nmr,lr);

    EditDisable();
    MapSubwindows();
    MapWindow();
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EnergyEntry FIFNISH"<< std::endl;
  }

  void EnergyEntry::EditDisable(void)
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EditDisable START" << std::endl;
    ne_energy->SetState(kFALSE);
    ne_error->SetState(kFALSE);
    ne_nmr->SetState(kFALSE);
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EditDisable FINISH" << std::endl;
  }

  void EnergyEntry::EditEnable(void)
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EditEnable START" << std::endl;
    ne_energy->SetState(kTRUE);
    ne_error->SetState(kTRUE);
    ne_nmr->SetState(kTRUE);
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::EditEnable FINISH" << std::endl;
  }

  void EnergyEntry::SetData(const measurement::EnergyCalibrationConfig * c)
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::SetData START" << std::endl;
    ne_energy->SetNumber(c->energy.value);
    ne_error->SetNumber(c->energy.error);
    ne_nmr->SetNumber(c->nmr);
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::SetData FINISH" << std::endl;
  }

  void EnergyEntry::GetData(measurement::EnergyCalibrationConfig * c) const 
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::GetData START" << std::endl;
    c->energy = data::PhysValue(ne_energy->GetNumber(), ne_error->GetNumber());
    c->nmr = ne_nmr->GetNumber();
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::SetData FINISH" << std::endl;
  }

  EnergyEntry::~EnergyEntry(void)
  {
    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::~EnergyEntry START" << std::endl;
    delete ne_nmr; 
    delete ne_energy; 
    delete ne_error;

    delete ll; 
    delete lr; 
    delete lf;
    delete l_nmr; 
    delete l_energy; 
    delete l_energy_error;

    delete f_energy; 
    delete f_nmr; 

    std::clog<<LOG(DEBUG_LOG)<<"EnergyEntry::~EnergyEntry FINISH" << std::endl;
  }
}
