#ifndef IBN_FITDIALOG_H 
#define IBN_FITDIALOG_H

#include <memory>
class TGWindow;
class TGHorizontalFrame;
class TGTextEntry;
class TGTextButton;
class TGGroupFrame;
class TGComboBox;
class TGCheckButton;
class TGLayoutHints;
class TGNumberEntry;
class TGLabel;
#include <TGFrame.h>

#include <Minuit2/MnUserParameters.h>

namespace data { class EnergyCalibrationStorage;}
#include "fit/FitManager.h"
#include "gui/EnergyCalibrationView.h"


namespace gui
{
  class FitDialog	: public TGTransientFrame
  {
    public:
    FitDialog(const TGCompositeFrame *main,gui::EnergyCalibrationView *, fit::FitManager *, bool isw=false) ;
    virtual ~FitDialog(void);
    private:
    void UpdateCheckButtons(void);
    void FillCheckButtons(void);
    void FillParametersFrame(void);
    void ClearParametersFrame(void);
    void UpdateFunctionCombo(void);
    void UpdateGraphCombo(void);
    void UpdateParIni(void);
    void SetTimeRange(void);
    void SetBeginTime(void);
    void SetEndTime(void);
    void Fit(void);

    //void SetFitParameters(void);
    void ShowParameters(const ROOT::Minuit2::MnUserParameters & par);
    ROOT::Minuit2::MnUserParameters GetParameters(void);

    void DrawBG(void);
    void DrawBG2(void);
    void CloseWindow(void);
    virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);

    //std::vector <fit::FitParConfiguration> pcfg;
    ROOT::Minuit2::MnUserParameters pcfg;

    /* Here is 3 frames */
    /* Input fields  */
		std::unique_ptr <TGGroupFrame>  GFTextEntry;
    std::unique_ptr <TGNumberEntry> TEBeginTime, TEEndTime;
    std::unique_ptr <TGTextEntry> TEFormula;

    struct Parameters_t
    {
      int number;
      std::string name;
			std::unique_ptr<TGCheckButton> fix;
      std::unique_ptr<TGCheckButton> limit;
      std::unique_ptr<TGNumberEntry> value;
      std::unique_ptr<TGNumberEntry> error;
      std::unique_ptr<TGNumberEntry> lower;
      std::unique_ptr<TGNumberEntry> upper;
      void addTo(TGCompositeFrame *f, TGLayoutHints *l=0);
      void removeFrom(TGCompositeFrame *f);
    };


    std::unique_ptr<TGHorizontalFrame> DrawHF; 
		std::unique_ptr<TGCheckButton> DrawCheckButton;
		std::unique_ptr<TGCheckButton> FullDrawCheckButton;


    std::unique_ptr<TGGroupFrame> ParameterFrame;	
    std::vector < Parameters_t > par_vgt;

    std::unique_ptr <TGHorizontalFrame> HFTextButton;
    std::unique_ptr <TGTextButton> TBReset;
    std::unique_ptr <TGTextButton> TBFit;
    std::unique_ptr <TGTextButton> TBClose;

    std::unique_ptr <TGComboBox> FitCombo;
    std::unique_ptr <TGComboBox> FunctionsCombo;

    TGLayoutHints	*LHButton, *LHGeneral;
    data::EnergyCalibrationStorage  * EC;
    gui::EnergyCalibrationView * ECView;
    fit::FitManager *theFitManager;
    Canvas * canvas;
    std::vector<std::unique_ptr<TGraph> > graph_tmp;
  };
}
#endif
