#include <TSystem.h>
#include <TGMsgBox.h>
#include <TGFileDialog.h>

#include "measurement/Manager.h"
#include "MainWindow.h"
#include "Log.h"
#include "MeasurementViewDialog.h"
#include "ScanConfigFrame.h"
#include "BunchViewFrame.h"
#include "FitDialog.h"
#include "DepolarisatorDialog.h"
#include "DataViewDialog.h"
#include "CounterConfigDialog.h"
#include "DelayLineDialog.h"
#include "MeasurementManagerDialog.h"
#include "CalculatorConfigDialog.h"


extern std::recursive_mutex  RootProcessEventsMutex;
extern std::string POLARIMETER_VERSION;

namespace gui
{
  MainWindow::MainWindow(const TGWindow * w) : TGMainFrame(w,1030,450)
  {
    std::clog<<LOG(DEBUG_LOG)<<"MainWindow::MainWindow BEGIN" << std::endl;
    gmanager.reset(new MeasurementManagerDialog(this));
    ismeasure=false;
    MenuBar.reset(new TGMenuBar(this, 1, 1, kHorizontalFrame));

    FileMenu.reset(new TGPopupMenu(fClient->GetRoot()));
    FileMenu -> AddEntry("&Save"  , M_FILE_SAVE);
    FileMenu -> AddEntry("&Save as"  , M_FILE_SAVE_AS);
    FileMenu -> AddEntry("&Load"  , M_FILE_LOAD);
    FileMenu -> AddEntry("&Restore"  , M_FILE_RESTORE);
    FileMenu -> AddEntry("&Print" , M_FILE_PRINT);
    FileMenu -> AddEntry("&DB" , M_FILE_DB);
    FileMenu -> AddSeparator();
    FileMenu -> AddEntry("Load bad point list", M_FILE_LOAD_BAD_POINT_LIST);
    FileMenu -> AddEntry("Save bad point list", M_FILE_SAVE_BAD_POINT_LIST);
    FileMenu -> AddSeparator();
    FileMenu -> AddEntry("&Exit",M_FILE_EXIT);
    FileMenu -> Associate(this);

    SettingsMenu.reset(new TGPopupMenu(fClient->GetRoot()));
    SettingsMenu -> AddEntry("&Graphics", M_SET_GRAPHICS);
    SettingsMenu -> AddEntry("&Fit panel",M_SET_FIT);
    SettingsMenu -> AddEntry("&Concurrence configuration", M_SET_CONCUR_CFG);
    SettingsMenu -> AddEntry("&Depolarizer configuration", M_SET_DEPOLARISATOR_CFG);
    SettingsMenu -> AddEntry("&Counters", M_SET_COUNTERS);
    SettingsMenu -> AddEntry("&Calculator", M_SET_CALCULATOR);
    SettingsMenu -> AddEntry("&Delay Line configuration", M_SET_DELAYLINE);
    SettingsMenu -> AddSeparator();
    SettingsMenu -> Associate(this);

    ViewMenu.reset(new TGPopupMenu(fClient->GetRoot()));
    ViewMenu -> AddEntry("&Rare", M_SET_RARE);
    ViewMenu -> AddEntry("&UnRare", M_SET_UNRARE);
    ViewMenu -> AddEntry("&Remember graph list", M_SET_REMEMBER_GRAPH_LIST);
    ViewMenu -> AddEntry("&Order canvases", M_VIEW_ORDER_CANVAS);
    ViewMenu -> Associate(this);



		HelpMenu.reset(new TGPopupMenu(fClient->GetRoot()));
		HelpMenu->Associate(this);
    HelpMenu -> AddEntry("Current instruction", M_HELP_CURRENT_INSTRUCTION);
		HelpMenu -> AddEntry("Nmr stabilization instruction",  M_HELP_NMR_STABILIZATION_INSTRUCTION);
    //HelpMenu -> AddEntry("Polarimeter program help",M_HELP_POLARIMETER_INSTRUCTION);
		HelpMenu -> AddEntry("About",  M_HELP_ABOUT);

    TGLayoutHints * MenuBarItemLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0);
    MenuBar -> AddPopup("&File",FileMenu.get(),MenuBarItemLayout);
    MenuBar -> AddPopup("&Settings",SettingsMenu.get(),MenuBarItemLayout);
    MenuBar -> AddPopup("&View",ViewMenu.get(),MenuBarItemLayout);
    MenuBar -> AddPopup("&Help",HelpMenu.get(),MenuBarItemLayout);

    TGLayoutHints * MenuBarLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX,0, 0, 0, 0);

    AddFrame(MenuBar.get(), MenuBarLayout);
    TGLayoutHints * tmp_layout = new TGLayoutHints(kLHintsLeft | kLHintsExpandX,0, 0, 0, 0);

    TGHorizontalFrame * main_frame = new TGHorizontalFrame(this);
    AddFrame(main_frame,tmp_layout);

    TGVerticalFrame * control_frame = new TGVerticalFrame(main_frame);
    main_frame->AddFrame(control_frame,new TGLayoutHints(kLHintsLeft | kLHintsExpandX,2, 2, 0, 0));
		control_frame->SetWidth(1000);

		TGHorizontalFrame * horizontal_button_frame = new TGHorizontalFrame(control_frame);
		control_frame->AddFrame(horizontal_button_frame, new TGLayoutHints(kLHintsLeft | kLHintsExpandX  ,0, 0, 0, 0));



		TGVerticalFrame * button_frame1 = new TGVerticalFrame(horizontal_button_frame);
		//button_frame->SetLayoutManager(new TGMatrixLayout(button_frame, 0, 2));
		horizontal_button_frame->AddFrame(button_frame1, tmp_layout);

		TGVerticalFrame * button_frame2 = new TGVerticalFrame(horizontal_button_frame);
		//button_frame->SetLayoutManager(new TGMatrixLayout(button_frame, 0, 2));
		horizontal_button_frame->AddFrame(button_frame2, tmp_layout);



    //for(size_t i=0; i<gmanager->fmap.size(); ++i)
    //  b[i] = auto_ptr<TGTextButton>(new TGTextButton(control_frame,gmanager->ModeName[i].c_str(),i));

    b_layout = new TGLayoutHints( kLHintsExpandX,4,4,4,4);

		auto & mlst = gmanager->GetMeasurementConfigList();
		for(unsigned i=0; i<mlst.size();i++)
		{
			auto frame =  i<mlst.size()/2 ? button_frame1 : button_frame2;
			auto but = new TGTextButton(frame, mlst[i].first.c_str(),i);
      frame->AddFrame(but,  b_layout);
      but->Associate(this);
      but->SetState(kButtonUp);
			b[i].reset(but);
		}



    //for (bmap_t::iterator p=b.begin(); p!=b.end(); p++)
    //{
    //  //control_frame->AddFrame(p->second.get(),b_layout);
    //  button_frame->AddFrame(p->second.get(),  b_layout);
    //  p->second->Associate(this);
    //  p->second->SetState(kButtonUp);
    //}

		//int * retval = new int;
    //mview = new MeasurementViewDialog(control_frame, gmanager->ECView, gmanager->fitManager,gmanager->manager,retval);
    mview = new MeasurementViewDialog(control_frame, gmanager->ECView, gmanager->fitManager,gmanager->manager);
    control_frame->AddFrame(mview,new TGLayoutHints(kLHintsLeft | kLHintsExpandX | kLHintsExpandY,0, 1, 1, 1));
		mview->SetWidth(400);

    //depcfg_frame.reset(new ScanConfigFrame(this,manager->current.depolarizer));
    //main_frame->AddFrame(depcfg_frame.get(),b_layout);

    bunch_view_frame.reset(new BunchViewFrame(main_frame,gmanager->manager->GetStatus().BunchInfo, gmanager->ECView));
    main_frame->AddFrame(bunch_view_frame.get(),b_layout);

    //gmanager->manager->EC->StartBackgroundMeasurement();
		gmanager->manager->Start("Count");
    gmanager->manager->Start("CurrentMeasurement");
    gmanager->manager->Start("BeamSizeMeasurement");
    gmanager->manager->Start("FrequencyMeasurement");
    gmanager->manager->Start("StatusMeasurement");
    //gmanager->manager->EC->StartStatusMeasurement();

    Layout();
    SetWindowName("Polarimeter");
    SetWMPosition(0,0);
    Move(0,0);
		//Resize(GetDefaultSize());
    MapSubwindows();
    MapWindow();
		///Update();
		//Resize(1030, 435);
		//SetEditable(kFALSE);
  } 

  void MainWindow::Run(volatile bool & isrun)
  {
    int sleep_time = 10000; //mks
    ibn::timer status_timer;
    while(isrun && fIsRun)
    {
      usleep(sleep_time);
      gSystem->ProcessEvents();
			if(!fIsRun) return;
      if(status_timer.elapsed()>1 && fIsRun)
      {
        Update();
        status_timer.restart();
      }
    }
  }

  void MainWindow::Update(void) 
  { 
		std::clog << LOG(10) << "MainWindow::Update thread=" << hex << std::this_thread::get_id() << dec << std::endl;
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    gmanager->Update();
    bunch_view_frame->Update(); 
		mview->Update();
    //depcfg_frame->Update();
  }

  MainWindow::~MainWindow(void)   
  {
    clog << LOG(DEBUG_LOG) << "~MainWindow: BEGIN" << endl;
    /* All started measurement should be stopped in class which own them (EnergyCalibration)
    manager use EnergyCalibrationView to present result of measurement.
    So we must delete manager before mview */
		//exit(0);
		StopRun();
    delete  mview;
    clog << LOG(DEBUG_LOG) << "~MainWindow: END" << endl;
  }

  const char * filetypes[] = 
  {
    "polarimeter data", "*.pol", 
    "compressed polarimeter data", "*.pol.bz2",
    "polarimeter xml archive", "*.xml", 
    "All files","*", 0
  };

  Bool_t MainWindow::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
  {
		//std::cout << "MainWindow::ProcessMessage " << msg << " " << parm1 << " " << parm2 << std::endl;
    int retval;
    switch( GET_MSG(msg) )
    {
      case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            if(!ismeasure)
            {
              LockButton(parm1);
              clog << LOG(5) << "MainWindow::ProcessMessage: manager->Start"<<endl;
              ismeasure = true;
              string windowname ="Polarimeter " + gmanager->GetMeasurementConfigList()[parm1].first + " measurement";
              SetWindowName(windowname.c_str());
							gmanager->Measure(parm1);
              ismeasure = false;
              UnLockButton();
							return kTRUE;
            }
						else 
						{
							b[parm1]->SetState(kButtonEngaged);
						}
            break;
          case kCM_MENU:
            switch(parm1)
            {
              case M_SET_FIT:
                new FitDialog(this,gmanager->ECView, gmanager->fitManager);
                break;
              case M_SET_DEPOLARISATOR_CFG:
                new DepolarisatorDialog(this, &gmanager->manager->EC->ECStorage->cfg.depolarizer, gmanager->manager->EC->ECCamac);
                break;
              case M_SET_GRAPHICS:
                //InitDataList(manager->EC);
                gmanager->ECView->InitDataList();
                new DataViewDialog(this,gmanager->ECView);
                break;
              case M_SET_COUNTERS:
                //new CounterDialog(this, manager->polarimeter);
                new CounterConfigDialog(this, gmanager->manager->EC->ECCamac);
                break;
              case M_SET_CALCULATOR:
                new CalculatorConfigDialog(this, &gmanager->calculator);
                break;
              case M_SET_DELAYLINE:
                new DelayLineDialog(this, gmanager->manager->EC->ECCamac, gmanager->manager->EC);
                break;
              case M_SET_RARE:
                gmanager->ECView->Rare(2);
                break;
              case M_SET_UNRARE:
                gmanager->ECView->Rare(1);
                break;
              case M_SET_REMEMBER_GRAPH_LIST:
                if(PCFG.gui.remember_graph_list)
                {
                  ViewMenu->UnCheckEntry(M_SET_REMEMBER_GRAPH_LIST);
                  PCFG.gui.remember_graph_list = false;
                  std::cout << "Dont remember graph list " << std::endl;
                }
                else
                {
                  ViewMenu->CheckEntry(M_SET_REMEMBER_GRAPH_LIST);
                  PCFG.gui.remember_graph_list = true;
                  std::cout << "Set remember graph list " << std::endl;
                }
                break;
              case M_VIEW_ORDER_CANVAS:
                gmanager->ECView->OrderWindows();
                break;
              case M_FILE_SAVE:
                new TGMsgBox(gClient->GetRoot(), this, "Polarimeter","Is energy calibration successful?", kMBIconQuestion, kMBYes | kMBNo | kMBCancel, &retval );
                switch(retval)
                {
                  case kMBYes:
                    gmanager->EC->success = true;
                    gmanager->Save2FS();
                    break;
                  case kMBNo:
                    gmanager->EC->success = false;
                    gmanager->SaveBad2FS();
                    break;
                  case kMBCancel:
                  default:
                    break;
                };
								break;
							case M_FILE_SAVE_AS:
								{
                  const char *tmpstr=PCFG.save_dir.string().c_str();
                  TString dir(tmpstr);
                  TGFileInfo fi;
                  fi.fFileTypes = filetypes;
                  fi.fIniDir    = StrDup(dir);
                  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
									dir = fi.fIniDir;
									gmanager->EC->Save2FSSerial(std::string(fi.fFilename));
								}
                break;
              case M_FILE_PRINT:
                gmanager->Print();
                break;
              case M_FILE_DB:
                gmanager->Save2DB();
                break;
              case M_FILE_LOAD:
                {
									std::clog <<  LOG(DEBUG_LOG+1) << "M_FILE_LOAD: BEGIN" << std::endl;
                  const char *tmpstr=PCFG.save_dir.string().c_str();
                  TString dir(tmpstr);
                  TGFileInfo fi;
                  fi.fFileTypes = filetypes;
                  fi.fIniDir    = StrDup(dir);
                  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
                  dir = fi.fIniDir;
                  ifstream ifs(fi.fFilename);
                  if(!ifs)
                  {
                    std::cerr << LOG(NORMAL_LOG) << "ERROR: can't open file " << fi.fFilename << std::endl;
                  } 
                  else 
                  {
                    std::clog << LOG(NORMAL_LOG) << "Reading energy calibration data from  file " << fi.fFilename << endl;
                    gmanager->manager->EC->ECStorage->Clear();
                    gmanager->EC->Load(fi.fFilename);
                    gmanager->ECView->InitDataList();
                    gmanager->ECView->InitGraphList();
                    /* I should init data list again couse some of arrays were not
                     * initialized before load so links for data view may not be valid
                     * */
                    //InitGraphList(manager->EC);
                    std::clog << LOG(1) << "  OK" << std::endl;
                    gmanager->EC->issave = false;
                  }
                  gmanager->ECView->Clear();
                  gmanager->ECView->Add(PCFG.gui.default_graph);
                  gmanager->ECView->Draw();
                  gmanager->ECView->Update();
									std::clog <<  LOG(DEBUG_LOG+1) << "M_FILE_LOAD: END" << std::endl;
                }
                break;
              case M_FILE_LOAD_BAD_POINT_LIST:
                {
                  const char *tmpstr=PCFG.save_dir.string().c_str();
                  TString dir(tmpstr);
                  TGFileInfo fi;
                  const char * filetypes[] = {"All files","*", 0};
                  fi.fFileTypes = filetypes;
                  fi.fIniDir    = StrDup(dir);
                  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
                  dir = fi.fIniDir;
                  ifstream ifs(fi.fFilename);
                  if(!ifs)
                  {
                    std::cerr << LOG(1) << "ERROR: can't open file " << fi.fFilename <<  std::endl;
                  } 
                  else 
                  {
                    LoadBadPointList(ifs);
                  }
                }
                break;
              case M_FILE_SAVE_BAD_POINT_LIST:
                {
                  const char *tmpstr=PCFG.save_dir.string().c_str();
                  TString dir(tmpstr);
                  TGFileInfo fi;
                  const char * filetypes[] = {"All files","*", 0};
                  fi.fFileTypes = filetypes;
                  fi.fIniDir    = StrDup(dir);
                  new TGFileDialog(fClient->GetRoot(), this,kFDSave, &fi);
                  dir = fi.fIniDir;
                  ofstream ofs(fi.fFilename);
                  if(!ofs)
                  {
                    std::cerr << LOG(1) << "ERROR: can't open file " << fi.fFilename << std::endl;
                  } 
                  else 
                  {
                    SaveBadPointList(ofs);
                  }
                }
                break;
              case M_FILE_RESTORE:
                {
                  std::clog << LOG(1) << "Reading energy calibration data from  file " << PCFG.tmp_save_file.string() << " ... ";
                  gmanager->EC->Restore(PCFG.tmp_save_file.string());
                  std::clog << LOG(1) <<  "  OK" << std::endl;
                  gmanager->ECView->Clear();
                  gmanager->ECView->Add(PCFG.gui.default_graph);
                  gmanager->ECView->Draw();
                  gmanager->ECView->Update();
									PCFG.restore = false;
                }
                break;
              case M_FILE_EXIT:
                CloseWindow();
                break;
							case M_HELP_CURRENT_INSTRUCTION:
								//system("xpdf /home/vepp4/dds/doc/CurrentInstruction.pdf &");
								system(("xpdf "+ PCFG.work_dir.string() + "/doc/CurrentInstruction.pdf &").c_str());
								break;
							case M_HELP_NMR_STABILIZATION_INSTRUCTION:
								system(("xpdf "+ PCFG.work_dir.string() + "/doc/NmrStabilizationInstruction.pdf &").c_str());
								break;
							case M_HELP_ABOUT:
								{
									std::ostringstream oss;
									oss <<  "Polarimeter-"+POLARIMETER_VERSION << std::endl;
									//time_t t = time(0);
									oss << __TIMESTAMP__;
									//oss << "Ivan B. Nikolaev" << std::endl;
									//oss << "I.B.Nikolaev@inp.nsk.su" << std::endl;
									new TGMsgBox(gClient->GetRoot(),this, "About polarimeter",oss.str().c_str(), kMBIconExclamation, kMBOk, &retval );
								}
								break;
            }
            break;
        }
        break;
    }
    return true;
  }

  void MainWindow::LockButton(unsigned i )
  {
    std::clog<<LOG(DEBUG_LOG)<<"MainWindow::LockButton BEGIN"<< endl;
		int n=0;
    for (bmap_t::iterator p=b.begin(); p!=b.end(); ++p, ++n)  
		{
			p->second->SetState(kButtonDisabled);
			if(i==n)
			{
				//p->second->SetTextColor(0x00cc00);
				p->second->SetState(kButtonEngaged);
			}
		}
    std::clog<<LOG(DEBUG_LOG)<<"MainWindow::LockButton END"<< endl;
  }

  void MainWindow::UnLockButton(void )
  {
    std::clog<<LOG(DEBUG_LOG)<<"MainWindow::UnLockButton BEGIN" << std::endl;
    for (bmap_t::iterator p=b.begin(); p!=b.end(); ++p)  p->second->SetState(kButtonUp);
    std::clog<<LOG(DEBUG_LOG)<<"MainWindow::UnLockButton FIFNISH"<< std::endl;
  }

  void MainWindow::CloseWindow(void)
  {
    clog << LOG(DEBUG_LOG) << "MainWindow::CloseWindow BEGIN" << endl;
		StopRun();
    clog << LOG(DEBUG_LOG) << "MainWindow::CloseWindow END" << endl;
  }
}
