#include <string>

#include "MeasurementConfigDialog.h"
#include "Log.h"
#include "LabeledNumberEntry.h"
#include "ScanConfigFrame.h"
#include <boost/format.hpp>

#include "Event.h"

namespace gui
{
  MeasurementConfigDialog::MeasurementConfigDialog(const TGWindow * w, measurement::EnergyCalibrationConfig * c, int * r,  bool is_count_only):
    TGTransientFrame(gClient->GetRoot(),w,100,200)
    {
      std::clog<<LOG(3)<<"MeasurementConfigDialog::MeasurementConfigDialog BEGIN" << std::endl;
      SetWindowName((c->name + " Energy calibration configuration").c_str());

      cfg=c;
      initial_config = *cfg;
      retval = r;
			isCountOnly = is_count_only;

      lf.reset( new TGLayoutHints(kLHintsTop | kLHintsExpandX,2,2,2,2));
      ll.reset(new TGLayoutHints(kLHintsLeft,2,2,2,2));
      lr.reset(new TGLayoutHints(kLHintsRight,2,2,2,2));


			fRunNumberEntry.reset( new LabeledNumberEntry(this,"run", TGNumberFormat::kNESInteger, cfg->run,  1,  12));
			AddFrame(fRunNumberEntry.get(), lf.get());

			fCountTimeNumberEntry.reset( new LabeledNumberEntry(this,"Tcount [s]", TGNumberFormat::kNESReal,  cfg->count_time,  0,  12));
			AddFrame(fCountTimeNumberEntry.get(), lf.get());

			if(!isCountOnly)
			{

				//fShowLastPointsNumberEntry.reset( new LabeledNumberEntry(this,"show points", TGNumberFormat::kNESReal,  0,  2,  12));
				//AddFrame(fShowLastPointsNumberEntry.get(), lf);

				auto Efmt = boost::format("%7.3f +- %4.3f MeV") % cfg->energy.value % cfg->energy.error;
				cfg->depolarizer.expected = cfg->energy.value;

				//std::string str="Expected energy: " +  Efmt.str(); //;setString(cfg->energy.value,0) + "+-" + setString(cfg->energy.error,0) + " MeV";
				//een.reset(new TGLabel(this,str.c_str()));
				//AddFrame(een.get(),ll.get());
				fe.reset(new ScanConfigFrame(this, cfg->depolarizer));
				AddFrame(fe.get(),lf.get());
			}

      bf.reset(new TGHorizontalFrame(this, 300,200));
      bl.reset(new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX,2,2,2,2));
      b_name.push_back("<< Back");
      b_name.push_back("Reset");
      b_name.push_back("Start >>");
      b.resize(b_name.size());
      for(size_t i = 0; i< b.size(); i++)
      {
        b[i].reset(new TGTextButton(bf.get(),b_name[i].c_str(),i));
        b[i]->Associate(this);
        bf->AddFrame(b[i].get(),bl.get());
      }
      AddFrame(bf.get(),bl.get());
      Layout();
      MapSubwindows();
      MapWindow();
      Resize(GetDefaultSize());
      fClient->WaitFor(this);
      std::clog<<LOG(3)<<"MeasurementConfigDialog::MeasurementConfigDialog FINISH" << std::endl;
    }

  MeasurementConfigDialog::~MeasurementConfigDialog(void) 
  {
    std::clog<<LOG(3)<<"MeasurementConfigDialog::~MeasurementConfigDialog BEGIN" << std::endl;
    std::clog<<LOG(3)<<"MeasurementConfigDialog::~MeasurementConfigDialog FINISH" << std::endl;
  }

  void MeasurementConfigDialog::CloseWindow(void)
  {
    *retval = BACK;
    delete this;
  }

  Bool_t MeasurementConfigDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 )
  { 
    std::clog<<LOG(3)<<"MeasurementConfigDialog::ProcessMessage BEGIN" << std::endl;
    switch( GET_MSG(msg) )
    {
      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg))
        {
          case kTE_ENTER:
            switch(parm1)
            {
              case 1://update menu
                fe->Update();
                break;
              default:
                break;
            }
            break;
        }
        break;
      case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)
            {
              case 0: //back
                *retval=BACK;
                GetData(cfg);
                delete this;
                break;
              case 1: //reset
                Reset();
                break;
              case 2:
                *retval=NEXT;
                GetData(cfg);
                delete this;
                break;
              default:
                break;
            }
            break;
        }
        break;
    }
    std::clog<<LOG(3)<<"MeasurementConfigDialog::ProcessMessage FINISH" << std::endl;
    return kTRUE;
  }

  void MeasurementConfigDialog::SetData(const measurement::EnergyCalibrationConfig * c)
  {
    std::clog<<LOG(3)<<"MeasurementConfigDialog::SetData BEGIN" << std::endl;
		if(!isCountOnly)
		{
			fe->Update();
		}
		fCountTimeNumberEntry->SetNumber(c->count_time);
		fRunNumberEntry->SetIntNumber(c->run);
    //tc->SetNumber(unsigned(c->count_time));
    //run->SetIntNumber(c->run);
    std::clog<<LOG(3)<<"MeasurementConfigDialog::SetData FINISH" << std::endl;
  }

  void MeasurementConfigDialog::GetData(measurement::EnergyCalibrationConfig *c)  const
  {
    std::clog<<LOG(3)<<"MeasurementConfigDialog::GetData BEGIN" << std::endl;
		if(!isCountOnly)
		{
			fe->Update();
		}
    //c->count_time=tc->GetNumber();
		//c->run = fRunNumberEntry->GetIntNumber();
		c->count_time = fCountTimeNumberEntry->GetData();
		c->run = fRunNumberEntry->GetIntNumber();
    std::clog<<LOG(3)<<"MeasurementConfigDialog::GetData FINISH" << std::endl;
  }

  void MeasurementConfigDialog::Reset(void)
  {
    *cfg=initial_config;
		if(!isCountOnly)
		{
			fe->Reset();
		}
    //tc->SetNumber(unsigned(cfg->count_time));
    //run->SetIntNumber(cfg->run);
		fCountTimeNumberEntry->SetNumber(cfg->count_time);
		fRunNumberEntry->SetIntNumber(cfg->run);
  }
}
