#include "MeasurementViewDialog.h"
#include "DataViewDialog.h"
#include "FitDialog.h"
#include "MainWindow.h"
#include "measurement/StatusMeasurement.h"
#include "measurement/Manager.h"
#include "ScanConfigFrame.h"

#include <TGNumberEntry.h>

#include <TGProgressBar.h>

extern std::recursive_mutex  RootProcessEventsMutex;
namespace gui
{
	enum EVENT { NONE,BACK, NEXT, ATTENUATION_TE, RANGE_FINAL_TE};
	MeasurementViewDialog::MeasurementViewDialog(TGWindow *w, gui::EnergyCalibrationView * ecv,fit::FitManager * f,	measurement::Manager * p):
		TGGroupFrame(w,"Measurement parameters")
	{

		polarimeter=p;
		ECView=ecv;
		fitManager = f;
		m_dialog = static_cast<MainWindow*>(w);
		//retval=r;
		isupdateble=true;


		auto lf=new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX,2,2,2,2);
		auto lc=new TGLayoutHints(kLHintsBottom | kLHintsCenterX | kLHintsExpandX,2,2,2,2);
		//lf=new TGLayoutHints(kLHintsTop | kLHintsLeft ,2,2,2,2);
		//lc=new TGLayoutHints(kLHintsBottom | kLHintsCenterX,2,2,2,2);
		auto ll=new TGLayoutHints(kLHintsTop | kLHintsLeft ,2,2,2,2);
		auto lr=new TGLayoutHints(kLHintsTop | kLHintsRight,2,2,2,2);

    lhints.push_back(unique_ptr<TGLayoutHints>(lf));
    lhints.push_back(unique_ptr<TGLayoutHints>(lc));
    lhints.push_back(unique_ptr<TGLayoutHints>(ll));
    lhints.push_back(unique_ptr<TGLayoutHints>(lr));

		data::EnergyCalibrationStorage* ECStorage = static_cast<data::EnergyCalibrationStorage*>(p->GetStorage("EnergyCalibration"));

		measurement::EnergyCalibrationConfig  cfg=ECStorage->cfg;

    for ( auto name : {"mode", "range","speed", "step"} )
    {
      labels[name].reset(new TGLabel(this,""));
      AddFrame(labels[name].get(),ll);
    }

    auto att_frame = new TGHorizontalFrame(this,100,20);

    labels["dB"].reset(new TGLabel(att_frame,"dB"));
    att_frame->AddFrame(labels["dB"].get(),lr);

    att_ne.reset(new TGNumberEntry(att_frame,cfg.depolarizer.attenuation,2,ATTENUATION_TE,TGNumberFormat::kNESInteger));
    att_ne->SetLimits(TGNumberFormat::kNELLimitMinMax,polarimeter->GetDepolarisator()  ->GetMinimumAttenuation(),60);
    att_frame->AddFrame(att_ne.get(), lr);
    att_ne->Associate(this);

    labels["attenuation"].reset(new TGLabel(att_frame,"Attenuation "));
		att_frame->AddFrame(labels["attenuation"].get(),lr);



    AddFrame(att_frame, lf);

    hframes.push_back(unique_ptr<TGHorizontalFrame>(att_frame));
		
		measured_frequency_l=new TGLabel(this,("Measured frequency: " +  setString(0) + " Hz").c_str());
		AddFrame(measured_frequency_l,ll);

    labels["measured_frequency"]= unique_ptr<TGLabel>(measured_frequency_l);
		//fScanConfigFrame.reset(new ScanConfigFrame(this, ECStorage->cfg.depolarizer, ScanConfigFrame::DIRECTION_SWAP_DISABLE | ScanConfigFrame::SKIP_WIDTH));
		//fScanConfigFrame->Lock();
		//AddFrame(fScanConfigFrame.get(),ll);

		// Buttons =================
		auto bf = new TGHorizontalFrame(this, 300,200);
		auto bl = new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX,2,2,2,2);
    lhints.push_back(unique_ptr<TGLayoutHints>(bl));
		b_name.push_back(" ABORT ");
		b_name.push_back(" GRAPH ");
		b_name.push_back(" FIT ");
		b_name.push_back("FAST FIT");
		b_name.push_back(" STOP ");
		b.resize(b_name.size());
		for(size_t i = 0; i< b.size(); i++)
		{
			b[i].reset(new TGTextButton(bf,b_name[i].c_str(),i));
			b[i]->Associate(this);
			bf->AddFrame(b[i].get(),bl);
		}
		AddFrame(bf,bl);
    hframes.push_back(unique_ptr<TGHorizontalFrame>(bf));

    // Scan progress bar and assotiated informaton: expected, initial, final energy
    // titles
    bf = new TGHorizontalFrame(this);
    auto left_energy_title = new TGLabel(bf,"Initial energy");
    auto right_energy_title = new TGLabel(bf,"Finitial energy");
    auto expected_energy_title = new TGLabel(bf,"Expected energy");
    bf->AddFrame(left_energy_title,ll); 
    bf->AddFrame(expected_energy_title, lc);
    bf->AddFrame(right_energy_title, lr);
    labels["left_energy_title"]=unique_ptr<TGLabel>(left_energy_title);
    labels["right_energy_title"]=unique_ptr<TGLabel>(right_energy_title);
    labels["expected_energy_title"]=unique_ptr<TGLabel>(expected_energy_title);
    AddFrame(bf, lf);

    //min, expected and max energy
    bf = new TGHorizontalFrame(this);
    auto left_energy_label = new TGLabel(bf,"");
    auto right_energy_label = new TGLabel(bf,"");
    auto expected_energy_label = new TGLabel(bf,"");
    bf->AddFrame(left_energy_label,ll); 
    bf->AddFrame(expected_energy_label, lc);
    bf->AddFrame(right_energy_label, lr);
    labels["left_energy_label"]=unique_ptr<TGLabel>(left_energy_label);
    labels["right_energy_label"]=unique_ptr<TGLabel>(right_energy_label);
    labels["expected_energy_label"]=unique_ptr<TGLabel>(expected_energy_label);
    AddFrame(bf, lf);

    //progess bar itself
    scan_progress_bar.reset(new TGHProgressBar(this));
    bar_color = scan_progress_bar->GetBarColor();
    scan_progress_bar->SetPosition(cfg.depolarizer.initial);
    scan_progress_bar->ShowPos(kTRUE);
    scan_progress_bar->Percent(kFALSE);
    AddFrame(scan_progress_bar.get(), lf);


		MapSubwindows();
		Layout();
		MapWindow();
		SetWindowName("Energy calibration");
    auto SM = dynamic_cast<measurement::StatusMeasurement*>(polarimeter->GetMeasurement("StatusMeasurement"));
    sig_con = SM->sig_update_status.connect([this]{Notify2();});
		Resize(300, 200);
	}

	MeasurementViewDialog::~MeasurementViewDialog(void) 
	{
		clog << LOG(DEBUG_LOG) << "~MeasurementViewDialog:" << endl;
		sig_con.disconnect();
		clog << LOG(DEBUG_LOG) << "~MeasurementViewDialog FINISH" << endl;
	}

	void MeasurementViewDialog::SetScanParameters(void)
	{
		data::EnergyCalibrationStorage* ECStorage = static_cast<data::EnergyCalibrationStorage*>(polarimeter->GetStorage("EnergyCalibration"));
		measurement::EnergyCalibrationConfig  cfg=ECStorage->cfg;

    att_ne->SetNumber(cfg.depolarizer.attenuation);
		if(fScanConfigFrame) fScanConfigFrame->Update();
	}

	Bool_t MeasurementViewDialog::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 ) 
	{ 
		//std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
		switch( GET_MSG(msg) )
		{
			case kC_COMMAND:
        switch(GET_SUBMSG(msg))
        {
          case kCM_BUTTON:
            switch(parm1)
            {
              case 0: //ABORT
                retval=BACK;
                polarimeter->GetDepolarisator()->StopScan();
                polarimeter->Stop("EnergyCalibration");
                std::clog << LOG(1) << "Abort measurement" << std::endl;
                return kTRUE;
                break;
              case 1: //GRAPH
                new DataViewDialog(m_dialog,ECView);
                break;
              case 2: //FIT
                new FitDialog(m_dialog, ECView, fitManager);
                //fitManager->Fit(ECView);
                break;
              case 3: //FIT
                //new FitDialog(m_dialog, ECView, fitManager);
                fitManager->Fit(ECView);
                break;
              case 4: //Stop
                retval=NEXT;
                polarimeter->GetDepolarisator()->StopScan();
                polarimeter->Stop("EnergyCalibration");
                std::clog << LOG(1) << "Stop measurement" << std::endl;
                break;
              default:
                break;
            }
            break;
        }
        break;
      case kC_TEXTENTRY:
        switch(GET_SUBMSG(msg))
        {
          case kTE_TEXTCHANGED:
            switch(parm1)
            {
              case  ATTENUATION_TE:
                ECView->GetStorage()->cfg.depolarizer.attenuation = att_ne->GetIntNumber();
                std::cout << LOG(1) << "Set attenuation: " << ECView->GetStorage()->cfg.depolarizer.attenuation << " dB" << std::endl;
                polarimeter->GetDepolarisator()->SetAttenuation(ECView->GetStorage()->cfg.depolarizer.attenuation);
                break;
              case RANGE_FINAL_TE:
                //ECView->GetStorage()->cfg.depolarizer.final = range_ne->GetNumber();
                break;
            }
            break;
        }
        break;
    } //END OF GET MESSAGE
		return kTRUE;
	}

	void MeasurementViewDialog::CloseWindow(void)   {}





	void MeasurementViewDialog::Update(void) 
	{
		std::clog << LOG(5) << "MeasurementViewDialog::Update thread=" << hex<< std::this_thread::get_id()<< dec << std::endl;
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
		if(fScanConfigFrame) fScanConfigFrame->Update();
		data::Status &  status=polarimeter->GetStatus();
		measured_frequency_l->SetText(("Measured frequency: " +  setString(status.measured_frequency) + " Hz").c_str());
		//att_l->SetText(("Attenuation: " +  setString(ECView->storage->cfg.depolarizer.attenuation) + " dB").c_str());
    att_ne->SetNumber(ECView->storage->cfg.depolarizer.attenuation);

		measurement::EnergyCalibrationConfig  cfg=ECView->storage->cfg;

    boost::format info_fmt("%-30s %10.3f %6s");

    labels["mode"]->SetText((boost::format("%-30s %s") % "Scan mode" % cfg.type).str().c_str());

    double range = fabs(cfg.depolarizer.initial-cfg.depolarizer.final);
    if(range > 1) 
    {
		  labels["range"]->SetText((info_fmt % "Scan range" % range % "MeV").str().c_str());
    }
    else
    {
		  labels["range"]->SetText((info_fmt % "Scan range" % (range*1e3) % "keV").str().c_str());
    }

		labels["speed"]->SetText((info_fmt % "Scan speed" % (cfg.depolarizer.speed*1e3) % "keV/s").str().c_str());
		labels["step"]->SetText((info_fmt % "Scan step"  % (cfg.depolarizer.step*1e3)  % "keV").str().c_str());


    double E1 = cfg.depolarizer.initial;
    double E2 = cfg.depolarizer.final;

    boost::format Efmt("%9.3f MeV");
    labels["left_energy_label"]->SetText( (Efmt % std::min(E1,E2)).str().c_str());
    labels["right_energy_label"]->SetText( (Efmt % std::max(E1,E2)).str().c_str());
    labels["expected_energy_label"]->SetText( (Efmt % cfg.energy.value).str().c_str());

    boost::format current_fmt_on("point %3d   %10d/%d   %s MeV  %30.3f Hz");
    boost::format current_fmt_off("point %3d   %10d/%d             %60s        ");

		if(status.frequency >= 0 && status.frequency < 1E-10)
    {
      scan_progress_bar->Format((current_fmt_off % status.N % (int)status.time % (int) status.remain_time % "OFF" ).str().c_str());
    }
    else
    {
      scan_progress_bar->Format((current_fmt_on % status.N % (int)status.time % (int) status.remain_time  % "%25.3f" % status.frequency ).str().c_str());
    }
    
    scan_progress_bar->SetRange(std::min(E1,E2),std::max(E1,E2));
    scan_progress_bar->ShowPos(kTRUE);
    if(E1 <= E2)  
    {
      labels["left_energy_title"]->SetText("Initial energy");
      labels["right_energy_title"]->SetText("Finitial energy");
      scan_progress_bar->SetBarColor(bar_color);
      scan_progress_bar->SetBackgroundColor(0xffffff);
    }
    else 
    {
      labels["right_energy_title"]->SetText("Initial energy");
      labels["left_energy_title"]->SetText("Finitial energy");
      scan_progress_bar->SetBarColor(0xffffff);
      scan_progress_bar->SetBackgroundColor(bar_color);
    }
    scan_progress_bar->SetPosition(status.energy);

		Layout();
		isupdateble=false;
		clog << LOG(5) << "MeasurementViewDialog::Update: FINISH"<< endl;
	}
}
