#include <cstdio>
#include <boost/format.hpp>

#include "Note.h"
#include "Utils.h"

#include <TText.h>
#include "Log.h"

#include <mutex>
#include <thread>
#include "PolarimeterConfig.h"
extern std::recursive_mutex  RootProcessEventsMutex;

namespace gui
{
  Note::Note(void)
  {
    Cons();
  };

  Note::Note(unsigned n)
  {
    Cons();
  }

  Note::~Note(void)
  {	
  }

  void Note::Cons(void)    
  {
    isdraw = false;
    size = 0.025;
    angle = 90;
    rare=1;
  }

  void Note::AddNote(const char *str,	double x, double y)  
  {
  }

  std::string Note::get_label(int n, double data)
  {
    //std::string label = (boost::format("%11.6f") % data).str();
    std::string label = (boost::format(PCFG.gui.note_format) % data).str();
    if(rare==0 || (n % rare) != 0 || data==0) label="-";
    return label;
  }

  void Note::AddNote(double data, double x, double y)	
  {
		std::clog << LOG(DEBUG_LOG) << "Note::AddNote BEGIN" << std::this_thread::get_id() << std::endl;
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    Notes.push_back(Note_t());
    Note_t & note=Notes.back();
    note.data=data;
    note.x=x;
    note.y=y;
    note.n = Notes.size();
    note.label = get_label(note.n,note.data);
    note.text = TText(x,y,note.label.c_str()); 
    note.text.SetTextSize(size);
    note.text.SetTextAngle(angle);
    if(isdraw) note.text.Draw();
		std::clog << LOG(DEBUG_LOG) << "Note::AddNote: END" << std::endl;
  }



  void Note::Draw(void) 
  {
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    if(!isdraw) 
    {
      for(auto & note : Notes) 
      {
        note.text.Draw();
      }
    }
    isdraw = true;
  }

  void Note::SetAngle(double a) 
  {
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    angle = a;
    for(auto & note : Notes) 
    {
      note.text.SetTextAngle(angle);
    }
  }

  void Note::SetSize(double s) 
  {
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    size =s ;
    for(auto & note : Notes) 
    {
      note.text.SetTextSize(size);
    }
  }

  void Note::SetY(double y) 
  {
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    for(auto & note : Notes) 
    {
      note.text.SetY(y);
    }
  }

  void Note::Rare(unsigned r)
  {
		std::lock_guard<std::recursive_mutex> lock(RootProcessEventsMutex);
    rare=r;
    std::clog << LOG(5) << " set rare = " << rare << std::endl;
    for(auto & note : Notes)
    {
      note.label = get_label(note.n, note.data);
      note.text.SetText(note.x, note.y, note.label.c_str());
    }
  }

  void Note::SetNotes(const std::vector<double> & data, const std::vector<double> & X,  const std::vector <double> & Y)
  {
    Notes.clear();
    for(unsigned i=0;i < data.size(); i++)
    {
      AddNote(data[i], X[i], Y[i]);
    }
  }

  void Note::SetNotes(size_t N, double * data, double * X, double *Y)
  {
    Notes.clear();
    for(unsigned i=0;i < N; i++)
    {
      AddNote(data[i], X[i], Y[i]);
    }
  }

  double Note::GetTotalTextSize(void) 
  {
    return size*11;
  }
}//end of namespace gui

