/*
 * =====================================================================================
 *
 *       Filename:  ScanConfigFrame.h
 *
 *    Description:  Basic Scan Config Frame
 *
 *        Version:  1.0
 *        Created:  07/09/2010 04:59:31 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef IBN_SCAN_CONFIG_FRAME_H
#define IBN_SCAN_CONFIG_FRAME_H

#include <memory>
#include <map>

#include "measurement/DepolarizerConfig.h"
class TGLayoutHints;
class TGHorizontalFrame;
class TGCheckButton;
class TGLabel;
class TGNumberEntry;

namespace measurement
{
  class DepolarizerConfig;
}

namespace gui
{
  class ScanConfigFrame : public TGVerticalFrame
  {
		public:
		enum
		{
			NOSKIP = 0, 
			SKIP_WIDTH = 1 , 
			SKIP_CURRENT_FREQUENCY = 2, 
			SKIP_MEAUSURED_FREQUENCY = 4, 
			DIRECTION_SWAP_DISABLE = 8
		};

    public:
      ScanConfigFrame(const TGWindow *w, measurement::DepolarizerConfig &,  int options=(SKIP_WIDTH | SKIP_MEAUSURED_FREQUENCY | SKIP_CURRENT_FREQUENCY));
      ~ScanConfigFrame(void);
      void Update(void);
      Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2 );
      void SetAttCallBack(void(*)(double) , double(*)(void) );
      void Reset();
			void LockFrequency(void);
			void LockEnergy(void);
			void Lock(void);
			void UnlockFrequency(void);
			void UnlockEnergy(void);
			void Unlock(void);
			void Lock(std::string);
			void Unlock(std::string);
    private:
			int fOptions;
      measurement::DepolarizerConfig & cfg;
      measurement::DepolarizerConfig initial_config;
      double Erange;  //energy range to scan, MeV
      std::unique_ptr<TGLayoutHints> ll,lr,lf;
      std::unique_ptr<TGLabel> el; //the units for energy inputs
			std::unique_ptr<TGRadioButton> MeV_rb; //radio button to set MeV energy units
			std::unique_ptr<TGRadioButton> keV_rb; //radio button to set MeV energy units
			std::unique_ptr<TGRadioButton> up_rb; //up scan
			std::unique_ptr<TGRadioButton> down_rb; //up scan
      std::unique_ptr<TGLabel> fl; //units for the frequency inputs
      std::unique_ptr<TGCheckButton> kHz_cb; //lock unlock the frequency field
//      std::unique_ptr<TGCheckButton> scandir_cb; //scan direction check button
      std::unique_ptr<TGHorizontalFrame> tf;  //frame wich has prev. labels and check buttons
      class Item; //items with frequency and energy input TGNubmberEntries
      std::vector< Item > feItems;
      std::map<std::string, Item*> Imap;
      std::unique_ptr<TGHorizontalFrame> att_hf;//attenuator frame 
      std::unique_ptr<TGHorizontalFrame> harm_hf;//harmonic frame
      std::unique_ptr<TGLabel> att_lbl; //attenuator label
      std::unique_ptr<TGLabel> harm_lbl; //harmonic label
      std::unique_ptr<TGNumberEntry> att_ne;//attenuation number entry
      std::unique_ptr<TGNumberEntry> harm_ne;//harmonit number entry
      void UpdateDirectionCB(void);
      void (*HWSetAttenuation)(double att);
      double (*HWGetAttenuation)(void);
		private:
			void SetEnergyScale(double scale);
			double fEnergyUnits;
			const double MeV = 1;
			const double keV = 1e-3;
  };
}//end of namespace gui
#endif //IBN_SCAN_CONFIG_FRAME_H

