/*
 * =====================================================================================
 *
 *       Filename:  BeamSizeMeasurement.cpp
 *
 *    Description:  BeamSizeMeasurement
 *
 *        Version:  1.0
 *        Created:  06/01/2010 04:20:32 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#include <fstream>
#include <iostream>

#include <algorithm>
#include "BeamSizeMeasurement.h"
#include "ibn/timer.h"

#include "PolarimeterConfig.h"

using namespace std;
namespace measurement
{
  BeamSizeMeasurement::BeamSizeMeasurement(void) 
  {
    X.resize(4);
    Y.resize(4);
    Z.resize(4);
    ismeasure=false;
  };
  
  void BeamSizeMeasurement::BeamSize_t::resize(unsigned numbunch)
  {
    averager.resize(numbunch);
    size.resize(numbunch);
    instant_size.resize(numbunch);
  }


  BeamSizeMeasurement::~BeamSizeMeasurement(void)
  {
    Stop();
  }

  void BeamSizeMeasurement::Start(void)
  {
    if(!fThread.joinable())
    {
      fThread = std::thread([this]{measure();});
    }
  }

  void BeamSizeMeasurement::Stop(void)
  {
    ismeasure=false;
    if(fThread.joinable())
    {
      fThread.join();
    }
  }



  void BeamSizeMeasurement::measure(void)
  {
    ismeasure=true;
    while(ismeasure)
    {
      ifstream file_e(PCFG.vepp4.eCCD_file.string().c_str());
      ifstream file_p(PCFG.vepp4.pCCD_file.string().c_str());
      if(!file_e)
      {
        cerr << LOG(WARNING_LOG) << "ERROR: Unable to open file: " << PCFG.vepp4.eCCD_file.string()  << endl;
        ibn::sleep(MEASUREMENT_TIME,&ismeasure,REACTION_TIME);
        continue;
      }
      if(!file_p)
      {
        cerr << LOG(WARNING_LOG) << "ERROR: Unable to open file: " << PCFG.vepp4.pCCD_file.string().c_str() << endl;
        ibn::sleep(MEASUREMENT_TIME,&ismeasure,REACTION_TIME);
        continue;
      }
      for(unsigned i=0;i<7;i++)
      {
        file_e.ignore(1024,'\n');
        file_p.ignore(1024,'\n');
      }
      file_e >> X.instant_size[0].value; file_e.ignore(1024,'\n');
      file_e >> Z.instant_size[0].value; file_e.ignore(1024,'\n');
      file_e >> X.instant_size[0].error; file_e.ignore(1024,'\n');
      file_e >> Z.instant_size[0].error; file_e.ignore(1024,'\n');

      file_p >> X.instant_size[2].value; file_p.ignore(1024,'\n');
      file_p >> Z.instant_size[2].value; file_p.ignore(1024,'\n');
      file_p >> X.instant_size[2].error; file_p.ignore(1024,'\n');
      file_p >> Z.instant_size[2].error; file_p.ignore(1024,'\n');

      X.instant_size[1]=X.instant_size[0];
      Z.instant_size[1]=Z.instant_size[0];

      X.instant_size[3]=X.instant_size[2];
      Z.instant_size[3]=Z.instant_size[2];
      //cout << LOG(5) << X.instant_size[0] <<  "       " << Z.instant_size[0] << endl;
      //cout << LOG(5) << X.instant_size[1] <<  "       " << Z.instant_size[1] << endl;
      //cout << LOG(5) << X.instant_size[2] <<  "       " << Z.instant_size[2] << endl;
      //cout << LOG(5) << X.instant_size[3] <<  "       " << Z.instant_size[3] << endl;
      for(unsigned bunch=0;bunch<4;bunch++)
      {
        Y.instant_size[bunch]=0;
        Y.instant_size[bunch].error=1;
      }
      {
				std::lock_guard<std::mutex> lock(mutex);
        for(unsigned bunch=0;bunch<4;bunch++)
        {
          X.averager[bunch].add(X.instant_size[bunch].value, 1./(X.instant_size[bunch].error*X.instant_size[bunch].error));
          Y.averager[bunch].add(Y.instant_size[bunch].value, 1./(Y.instant_size[bunch].error*Y.instant_size[bunch].error));
          Z.averager[bunch].add(Z.instant_size[bunch].value, 1./(Z.instant_size[bunch].error*Z.instant_size[bunch].error));
        }
      }
      ibn::sleep(MEASUREMENT_TIME,&ismeasure,REACTION_TIME);
    }
  }



  void BeamSizeMeasurement::MakeNewPoint(void)
  {
		std::lock_guard<std::mutex> lock(mutex);
    for(int bunch=0;bunch<4;bunch++) 
    {
      X.size[bunch]=data::PhysValue(X.averager[bunch].average(), X.averager[bunch].sigma_average());
      Y.size[bunch]=data::PhysValue(Y.averager[bunch].average(), Y.averager[bunch].sigma_average());
      Z.size[bunch]=data::PhysValue(Z.averager[bunch].average(), Z.averager[bunch].sigma_average());
      X.averager[bunch].reset();
      Y.averager[bunch].reset();
      Z.averager[bunch].reset();
    }
  }

  data::PhysValue  BeamSizeMeasurement::GetX(unsigned bunch)
  {
    return X.size[bunch];
  }

  data::PhysValue  BeamSizeMeasurement::GetY(unsigned bunch)
  {
    return Y.size[bunch];
  }

  data::PhysValue  BeamSizeMeasurement::GetZ(unsigned bunch)
  {
    return Z.size[bunch];
  }

  data::PhysValue  BeamSizeMeasurement::GetInstantX(unsigned bunch)
  {
    return X.instant_size[bunch];
  }

  data::PhysValue  BeamSizeMeasurement::GetInstantY(unsigned bunch)
  {
    return Y.instant_size[bunch];
  }

  data::PhysValue  BeamSizeMeasurement::GetInstantZ(unsigned bunch)
  {
    return Z.instant_size[bunch];
  }


  
};

