/*
 * =====================================================================================
 *
 *       Filename:  BeamSizeMeasurement.h
 *
 *    Description:  Measure beam size. (read it from file and average)
 *
 *        Version:  1.0
 *        Created:  06/01/2010 03:32:37 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef  IBN_BEAM_SIZE_MEASUREMENT_H
#define  IBN_BEAM_SIZE_MEASUREMENT_H


#include "data/Data.h"
#include "ibn/averager.h"

#include <thread>
#include <mutex>

#include "Measurement.h"
#include "data/Storage.h"

namespace measurement 
{
  class BeamSizeMeasurement : public Measurement, public data::Storage
  {
    public:
    BeamSizeMeasurement(void) ;
    ~BeamSizeMeasurement(void);
    void Start(void);
    void Stop(void);
    void Pause(void) {}
    void UnPause(void) {}
    void Continue(void) {}
    BeamSizeMeasurement * GetStorage(void) { return this; }
    void MakeNewPoint(void);
    /* Storage portion */
    void Load(std::string) {}
    void Save(std::string) {}
    bool IsMeasure(void) const { return ismeasure; }
    data::PhysValue  GetX(unsigned bunch); /* get radial size */
    data::PhysValue  GetY(unsigned bunch); /* get longitudeal size */
    data::PhysValue  GetZ(unsigned bunch); /* get vertical size */
    data::PhysValue  GetInstantX(unsigned bunch); /* get radial size */
    data::PhysValue  GetInstantY(unsigned bunch); /* get longitudeal size */
    data::PhysValue  GetInstantZ(unsigned bunch); /* get vertical size */
    private:
    bool volatile ismeasure;
    std::mutex mutex;
    std::thread  fThread;
    struct BeamSize_t
    {
        std::vector < ibn::averager <double> > averager; /*  averager of data */
        std::vector < data::PhysValue > size;/*  result of averaging  */
        std::vector < data::PhysValue > instant_size ;/*  instant size  */
        void resize(unsigned numbunch );  /* set number of bunches */
    } X,Y,Z;
    void measure(void);
    void fill(const std::vector<double> &);
    static constexpr double REACTION_TIME=10e-3; /* sec */
    static constexpr double MEASUREMENT_TIME=1.0; /*  sec */
  };
};
#endif
