#include "CountMeasurement.h"
#include <chrono>


namespace measurement
{
	CountMeasurement::CountMeasurement(camac::EnergyCalibrationCamac * camac)
	{
		fCamac=camac;
		fIsMeasure=false;
		fCountTime=10;
	}
	void CountMeasurement::Start(void)
	{
		if(!fThread.joinable())
		{
			fIsMeasure=true;
			fIsPause=false;
			Restart(fCountTime);
			fThread = std::thread([this](void){Run();});
		}
	}

	void CountMeasurement::Stop(void)
	{
		fIsMeasure = false;
		fIsTimerWork = false;
		if(fThread.joinable())
		{
			fThread.join();
		}
	}


	void CountMeasurement::Pause(void)
	{
	//	fIsTimerWork=false;
		fIsPause = true;
	}

	void CountMeasurement::UnPause(void)
	{
		fIsPause=false;
	}

	void CountMeasurement::Continue(void)
	{
		fIsPause=false;
	}

	void CountMeasurement::Run(void)
	{
		std::clog<< LOG(DEBUG_LOG) << "CountMeasurement::Run BEGIN" << std::endl;
		if(fCamac==nullptr) 
		{
			fIsMeasure = false;
			return;
		}
		while(fIsMeasure)
		{
			std::clog << LOG(DEBUG_LOG+1) << "CountMeasurement" << std::endl;
			fIsTimerWork=true;
			point_begin_time = clock::now();
			point_end_time=point_begin_time+std::chrono::duration<long, std::ratio<1, 1000>>(long(fCountTime*1e3));
			double begin_freq=fCamac->depolarizer->GetFrequency();
			fCamac->StartTimer(fCountTime,fIsTimerWork);
			double end_freq=fCamac->depolarizer->GetFrequency();
			point_end_time = clock::now();
			std::clog << LOG(DEBUG_LOG+1)<< "After StartTimer fIsMeasure = "<< fIsMeasure << "  fIsTimerWork= " << fIsTimerWork << "  fIsPause = " << fIsPause << std::endl;
      if(fIsTimerWork) 
      {
        fIsTimerWork=false;
        while(fIsPause) usleep(10000);
        fStorage.lock();
				fStorage.measurement_begin_time = measurement_begin_time;
        fStorage.point_begin_time = point_begin_time;
        fStorage.point_end_time = point_end_time;
				fStorage.point_begin_frequency = begin_freq;
				fStorage.point_end_frequency   = end_freq;
        fStorage.Count = fCamac->GetData();
        fStorage.Coinc = fCamac->GetCoinc();
        fStorage.unlock();
        std::clog<< LOG(DEBUG_LOG+1) << "CountMeasurement::Run yeld" << std::endl;
				//std::cout << clock::to_time_t(point_begin_time) << " " << clock::to_time_t(point_end_time) << " " << begin_freq << " " << end_freq << std::endl;
				//auto T = fStorage.to_double(point_end_time-point_begin_time);
				//std::cout << T << "  " << end_freq - begin_freq << " " << (end_freq-begin_freq)/T << std::endl;
				//std::cout << double(point_end_time - point_begin_time) << std::endl;
        //sig_yeld(); //send signal to yeld the measured data
        Notify();
      }
		}
		std::clog<< LOG(DEBUG_LOG) << "CountMeasurement::Run END" << std::endl;
	}

	void CountMeasurement::Restart(double time)
	{
		fCountTime = time;
		fIsTimerWork = false;
		measurement_begin_time = system_clock::now();
    //steady_begin_time = steady_clock::now();
	}


  void CountMeasurement::Notify(void)
  {
    std::unique_lock<std::mutex> new_point(mtx);
    cv.notify_all();
  }

  void CountMeasurement::Wait(void)
  {
    std::unique_lock<std::mutex> new_point(mtx);
    cv.wait(new_point);
  }

  //void CountMeasurement::Wait(volatile bool & b)
  //{
  //  std::unique_lock<std::mutex> new_point(mtx);
  //  auto wait_time = std::chrono::duration<double>(1.5*fCountTime);
  //  cv.wait_for(new_point,wait_time, [&b]()
  //      {
  //        std::cout << "ismeasure in CounteMeasurement::Wait = " << b << std::endl;
  //        return !b;
  //      });
  //}
}
