/*
 * =====================================================================================
 *
 *       Filename:  CountMeasurement.h
 *
 *    Description:  Readout counters using specified types. Save 
 *    read data fill some collateral parameters (begintime,  endtime,  begin frequency
 *    end frequency)
 *
 *        Version:  1
 *        Created:  2016-03-26
 *       Revision:  
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physics, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#pragma once
#include <thread>
#include <mutex>
#include <condition_variable>
#include <memory>
#include <chrono>

//#include <boost/signals2.hpp>

#include "Measurement.h"
#include "EnergyCalibrationCamac.h"
#include "data/Storage.h"
#include "data/EnergyCalibrationStorage.h"

using namespace std::chrono;

namespace data
{
	class CountStorage :public Storage
	{
		typedef std::chrono::system_clock clock;
		public:
		~CountStorage(void){}
		void Save(std::string){}
		void Load(std::string){}
		clock::time_point measurement_begin_time; //time since Reset()
		clock::time_point point_begin_time;
		clock::time_point point_end_time;
		double point_begin_frequency;
		double point_end_frequency;
		std::vector< std::vector<double> > Count;
		std::vector< std::vector<double> > Coinc;
    std::mutex mutex;
    void lock(void)   {mutex.lock();}
    void unlock(void) {mutex.unlock();}
		double to_double(clock::duration dt)
		{
			return (std::chrono::duration_cast< std::chrono::duration<double> > (dt)).count();
		}
	};
};

namespace measurement
{
	class CountMeasurement : public Measurement
	{
		typedef std::chrono::system_clock clock;
		public:
			CountMeasurement(camac::EnergyCalibrationCamac * camac=nullptr);
			void Start(void); //start count in thread
			void Stop(void); //send signal to stop count
			void Pause(void);
			void UnPause(void);
			void Continue(void);
			void SetCamac(camac::EnergyCalibrationCamac *camac) { fCamac = camac;};
			void Restart(double t); //restart points with new count time
			double GetCountTime(void) { return fCountTime;}
			data::CountStorage  * GetStorage(void) { return & fStorage; }
			bool IsMeasure(void) const {return fIsMeasure; }
      void Wait(void);
      void Notify(void);
      clock::time_point GetEndTime(void) const { return point_end_time; }
			double GetCountTime(void) const { return fCountTime; }
		private:
      std::mutex mtx;
      std::condition_variable cv;
			void Run(void); //this function will be run in thread
			camac::EnergyCalibrationCamac * fCamac;
			data::CountStorage fStorage;
			double fCountTime;
			std::thread fThread;
			volatile bool fIsMeasure;
			volatile bool fIsTimerWork;
			volatile bool fIsPause;
			clock::time_point measurement_begin_time;
			clock::time_point point_begin_time;
			clock::time_point point_end_time;
		public:
	};
}
