#include "camac/ModuleInventory.h"
//#include "gui/MeasurementDialog.h"
#include "EnergyCalibrationCamac.h"
#include "CounterCalibrate.h"


#include "PolarimeterConfig.h"

namespace measurement
{

  CounterCalibrate_by_Landao::CounterCalibrate_by_Landao(camac::EnergyCalibrationCamac * ECCam) 
  {
    //is_measure = true;

    //lock = PTHREAD_MUTEX_INITIALIZER;

    pthread_mutex_init(&lock_counter_calibrate,NULL);

    ECCamac = ECCam;
    if(!PCFG.nocamac) //если оператор хочет, то запускаем без камака
    {
      std::list<camac::ModuleInventory* > ModuleList; //Список БЛОКОВ, который передается в Config
      ZCP = new camac::Zcp("Zcp1");
      ModuleList.push_back(ZCP);

      //camac::SetCamac SetBlock; //this class set blocks
      //SetBlock.Bind(&ModuleList,"CounterCalibrate", (CfgPtr->Cfg_Storage.dir_name+"/"+CfgPtr->Cfg_Storage.camac_config_file).c_str());
      //SetBlock.Bind(&ModuleList,"CounterCalibrate", PCFG.camac_config_file.string().c_str());
    }
  }


  CounterCalibrate_by_Landao::~CounterCalibrate_by_Landao(void)
  {
  }


  void CounterCalibrate_by_Landao::SetVviVolt(unsigned num_counter,unsigned volt)
  {
    ECCamac->OnVvi(counter_num, volt); //Switch on vvi. The first parameter is vvi number, the second - volt
  }


  void CounterCalibrate_by_Landao::SetDiskr(unsigned num_counter, unsigned treshold)
  {
    ECCamac->SetDiskr(num_counter,treshold);
  }

  void* GetDataFromZcp(void* CounterCalibrate_RAW)
  {
    std::cout<<LOG(4)<<"GetDataFromZcp: START\n";
    CounterCalibrate_by_Landao *CC = static_cast<CounterCalibrate_by_Landao*>(CounterCalibrate_RAW);
    while(CC->IsMeasure())
    {

      try
      {


        pthread_mutex_lock(&CC->lock_counter_calibrate);

        CC->ZCP->WaitLam();
        CC->current_data = CC->ZCP->GetData();
        std::cout<<LOG(4)<<"GetDataFromZcp: current_data="<<CC->current_data<<"\n";
        //CC->current_data = 1;
        //usleep(1000000);
        pthread_mutex_unlock(&CC->lock_counter_calibrate);
        //	usleep(1000);
      }
      catch(camac::Crate::Error e)    {
        std::cerr << LOG(0) << "polarimeter: camac crate error: " <<e.what() << '\n';
        std::cerr << LOG(0) << "check crate power or check ppi-crate cable" << '\n';
        exit(1);
      }
      catch(camac::Module::Error e)    {
        std::cerr << LOG(0) <<"polarimeter: camac module error: " <<e.what() << '\n';

        exit(1);
      }
      catch(std::runtime_error e)    {
        std::cerr <<LOG(0) << "polarimeter: runtime error: " << e.what() << '\n';

        exit(1);
      }
      catch(std::out_of_range e)	{
        std::cerr <<LOG(0) << "polarimeter: data is out of range: " << e.what() << '\n';

        exit(1);
      }
      catch(std::logic_error e)    {
        std::cerr << LOG(0) <<"polarimeter: logic error:  " <<e.what() << '\n';

        exit(1);
      }
      catch(...)	{
        std::cerr << LOG(0) << "polarimeter: some unrecognize exeption " << '\n';

        exit(1);
      }


    }


    std::cout<<LOG(4)<<"GetDataFromZcp: FINISH\n";
    return NULL;
  }


  void CounterCalibrate_by_Landao::Start(void) //работает в нити
  {
    std::cout<<LOG(4)<<"CounterCalibrate::Start:\n";
    //current_data = Zcp.GetData();
    is_measure = true;

    pthread_create(&GETDATAFROMZCP_THREAD,0,GetDataFromZcp,this); //запустить измерение

    std::cout<<LOG(4)<<"CounterCalibrate::Finish:\n";
  }





  void CounterCalibrate_by_Landao::Stop(void)
  {
    is_measure = false;
  }

  double CounterCalibrate_by_Landao::GetCurrentMeasurement(void)
  {
    std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::GetCurrentMeasurement START\n";
    static double tmp_data;
    pthread_mutex_lock(&lock_counter_calibrate);
    std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::GetCurrentMeasurement 1\n";
    tmp_data = current_data;
    std::cout<<"CounterCalibrate_by_Landao::GetCurrentMeasurement current_data"<<current_data<<"\n";
    pthread_mutex_unlock(&lock_counter_calibrate);
    std::cout<<LOG(4)<<"CounterCalibrate_by_Landao::GetCurrentMeasurement FINISH\n";
    return tmp_data;
  }

  bool CounterCalibrate_by_Landao::IsMeasure(void)
  {
    return is_measure;
  }

  //===================================================================
  //Класс для калибровки сцинтилляционных счетчиков "псевдоклассическим" методом.
  //В классическом методе, описанном в Санта Клаусе Групен Фюрере, используются три счетчика для калибровки одного.
  //Тут же используется два. Несовсем представляю как это будет работать, но, как говорится,
  //с Божьей помощью и нашими молитвами.
  //Да поможет нам Великая Поляризация!
  //Аминь!

  /*
     CounterCalibrate_by_ClassicMethod::CounterCalibrate_by_ClassicMethod(CounterCalibr_Calssic_begin_data* begin_data)
     {
     std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::CounterCalibrate_by_ClassicMethod: Start"<<"\n";
     ECC = begin_data->ECCamac;
     counter_num = begin_data->counter_num;
     volt_begin  = begin_data->volt_begin;
     volt_end    = begin_data->volt_end
     count_time  = begin_data->count_time;

     ECC->OnVvi(counter_num, volt_begin);	//установим напряжение на ВВИ

  //Теперь самым наглым образом используем схему совпадения класса EnergyCalibration.  А в начальное состояние
  //её возращать не будем (как впрочем и ВВИ) :-). Вот такой я коварный и жестокий. Теперь юзеру придется 
  //заново перезапускать всю программу. Увы, таковы суровые реалии нашей жизни.
  //	Concur_L0306* Concurence = static_cast<Concur_L0306*>(ECC->Concur-);

  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::CounterCalibrate_by_ClassicMethod: Finish"<<"\n";
  }

  CounterCalibrate_by_ClassicMethod::~CounterCalibrate_by_ClassicMethod(void)
  {
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::~CounterCalibrate_by_ClassicMethod: Start"<<"\n";
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::~CounterCalibrate_by_ClassicMethod: Finish"<<"\n";
  }

  void CounterCalibrate_by_ClassicMethod::Start(void)
  {
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::Start: Start"<<"\n";
  ismeasure = true;
  pthread_create(&measure_thread,0,Start_Counter_Calibr_Classic_in_tread,this); //запустить измерение
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::Start: Finish"<<"\n";
  }


  void CounterCalibrate_by_ClassicMethod::Stop(void)
  {
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::Stop: Start"<<"\n";
  ismeasure = false;	
  pthread_join(&measure_thread,0);	
  std::cout<<LOG(4)<<"CounterCalibrate_by_ClassicMethod::Stop: Finish"<<"\n";
  }



  void* Start_Counter_Calibr_Classic_in_tread(void* CounterCalibrClassic_RAW)
  {
  std::cout<<LOG(4)<<"Start_Counter_Calibr_Classic_in_tread: Start"<<"\n";
  CounterCalibrate_by_ClassicMethod* CCC = static_cast<CounterCalibrate_by_ClassicMethod*>(CounterCalibrClassic_RAW)	

  while(ismeasure)
  {
  CCC->ECC->StartTimer(CCC->count_time);
  CCC->FillData();
  }
  return NULL;
  std::cout<<LOG(4)<<"Start_Counter_Calibr_Classic_in_tread: Finish"<<"\n";
  }

  //=====================================================================



  //=====================================================================
  //Дамы и господа! Представляю вашему вниманию функцию, которая вычитывает данные из
  //КАМАКа, переваривает их и заполняет вектор data_array
  //Дорогу открытому коду, товарищи!!! Ура!!!
  void CounterCalibrate_by_ClassicMethod::FillData(void)
  {
  static Value_t zaruzka; //Загрузка на счетчике, который мы калибруем
  static Value_t sovpadenie; //Загрузка от совпадений между двумя счетчиками
  static Value_t efficiency; //Эффективность счетчика
  static std::vector<std::vector<Value_t> > current_data; //Здесь будут храниться данные, вычитанные из КАМАКа. Здесь нам нужно не все, а только небольшая часть
  current_data = ECC->GetData(); //Вычитаем данные из КАМАКа

  zaruzka = current_data[1][counter_num]; //Туда куда приходит загрузка от первого пучка в EnergyCalibration в нашем измерении приходит загрузка от совпадений
  sovpadenie = current_data[2][counter_num];//Туда куда приходит загрузка от второго пучка в EnergyCalibration в нашем измерении приходит загрузка от калибруемого счетчика

  efficiency = sovpadenie/zaruzka; //вот она, родимая, эффективность, будь она не ладна... тьфу, наооброт, будь она всегда ладна
  data_array.push_back( data::PhysValue(efficiency,0) ); //закиним измеренную эффектинвность в вектор, а на ошибку измерения пока забьем
  //Итак, мальчики и девочки, все готово, свершилось, данные из КАМАКа перекинуты в вектор.
  //Спи спокойна, дорогой юзер, девелопер позаботился, чтобы все работало нормально.
  //Хочу виллу на канарах!
}
//=====================================================================
*/

}
