/*
 * =====================================================================================
 *
 *       Filename:  CurrentMeasurement.h
 *
 *    Description:  Measure beam current new incarnation
 *
 *        Version:  1.0
 *        Created:  06/01/2010 03:32:37 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#ifndef  IBN_CURRENT_MEASUREMENT_H
#define  IBN_CURRENT_MEASUREMENT_H

#include <thread>
#include <mutex>

#include "data/Data.h"
#include "data/Storage.h"
#include "ibn/averager.h"

#include "Measurement.h"



namespace measurement 
{
  class CurrentMeasurement  : public Measurement, public data::Storage
  {
    public:
    CurrentMeasurement(void) ;
    ~CurrentMeasurement(void);
    void Start(void);
    void Stop(void);
    void Pause(void){};
    void UnPause(void){};
    void Continue(void){};
    bool IsMeasure(void) const { return ismeasure; }
    CurrentMeasurement * GetStorage(void) { return this; }
    void MakeNewPoint(void);
    data::PhysValue  GetCurrent(unsigned bunch);
    double  GetInstantCurrent(unsigned bunch);
    /* Storage part */
    void Load(std::string) {}
    void Save(std::string) {}
    private:
    bool volatile ismeasure;
    std::mutex mutex;
    std::thread  fThread;
    std::vector < ibn::averager <double> > A; /*  averager */
    std::vector < data::PhysValue > Ia; /*  average current  */
    std::vector < double > Ic;         /*  instant current value  */
    bool extract_current(std::string name, const std::map<std::string, std::vector<double> > &dm);
    void measure(void);
    void old_measure(void); /*  look in stap */
    void fill(const std::vector<double> &);
    static constexpr double REACTION_TIME=10e-3; /* sec */
    static constexpr double MEASUREMENT_TIME=1.0; /*  sec */
    void read_old(void);
    void read_vepp3_current(void);
  };
};
#endif
