#include <TRandom.h>

#include <sys/time.h>
#include <algorithm>
#include <cstring>
#include <fstream>


#include "ibn/timer.h"
#include "Log.h" 
#include "PolarimeterConfig.h"

#include "Depolarisator.h"
#include "data/Calculator.h"



#include <boost/format.hpp>

using namespace std;

namespace measurement
{

	void Depolarisator::CreateAttenuators(void)
	{
    attenuator[0].AddAttenuator(new camac::Attenuator_A0608("Attenuator01"));
    attenuator[0].AddAttenuator(new camac::Attenuator_A0608_3("Attenuator03"));

    attenuator[1].AddAttenuator(new camac::Attenuator_A0608("Attenuator11"));
    attenuator[1].AddAttenuator(new camac::Attenuator_A0608_3("Attenuator13"));

    attenuator[2].AddAttenuator(new camac::Attenuator_A0608("Attenuator21"));
    attenuator[2].AddAttenuator(new camac::Attenuator_A0608_3("Attenuator23"));
	}

	void Depolarisator::Init(void)
	{
		//CreateAttenuators();
    CFr=0;
    isscan = false;
    //thread=0;
    IFr = 400000;
    FFr = 500000;
    FrW  = 10;
    DFr  = 5;
    DeT = 10;
    FrS = 5;
    Strategy = MONO;
    frequency =0;
    min_attenuation=10;
	}

  Depolarisator::Depolarisator(void)
  {
		Init();
  }

  //Depolarisator::Depolarisator(
  //    const camac::Address & adr, 
  //    const camac::Address & a1db,
  //    const camac::Address & a3db
  //    ) : Depolarisator()
  //{

  //  attenuator[0].AddAttenuator(new camac::Attenuator_A0608("A1",a1db));
  //  attenuator[0].AddAttenuator(new camac::Attenuator_A0608_3("A3",a3db));
  //  attenuator[1].AddAttenuator(new camac::Attenuator_A0608_3("A3"));
  //  attenuator[2].AddAttenuator(new camac::Attenuator_A0608_3("A4"));
  //  CFr=0;
  //  isscan = false;
  //  thread=0;
  //  IFr = 400000;
  //  FFr = 500000;
  //  FrW  = 10;
  //  DFr  = 5;
  //  DeT = 10;
  //  FrS = 5;
  //  Strategy = MONO;
  //  frequency =0;
  //  min_attenuation = 10;
  //}

  Depolarisator::Depolarisator( const camac::Address & adr)
  {
    CFr=0;
    isscan = false;
    //thread=0;
    IFr = 400000;
    FFr = 500000;
    FrW  = 10;
    DFr  = 5;
    DeT = 10;
    FrS = 5;
    Strategy = MONO;
    frequency =0;
    min_attenuation=10;
  }


  /*
   *--------------------------------------------------------------------------------------
   *       Class:  Depolarisator
   *      Method:  SetConfig
   * Description:  Configure depolarizator 
   *--------------------------------------------------------------------------------------
   */
  void Depolarisator::SetConfig(const DepolarizerConfig & c /* configuration in energy (MeV) units */)
  {
    if(fabs(c.attenuation)<min_attenuation) 
    {
      DepolarizerConfig & dch = const_cast<DepolarizerConfig&>(c);
      cerr << LOG(NORMAL_LOG) << "WARINING: Attenuation " << dch.attenuation << " dB less then minimum allowed attenuation " << min_attenuation << " dB" << endl;
      dch.attenuation=min_attenuation;
      cout << LOG(NORMAL_LOG) << "Set attenuation " << dch.attenuation << " dB" << endl;
    }
    DepolarizerConfig * cfg = (DepolarizerConfig *)(this);
    DepolarizerConfig fcfg;
    data::energy2frequency(&c,&fcfg);
    *cfg=fcfg;
    IFr=initial;
    FFr=final;
    DeT=scan_time;
    FrW=width;
    FrS=step;
		speed = final > initial ? fabs(speed) : -fabs(speed);
  }

  DepolarizerConfig  Depolarisator::GetFrequencyConfig(void)
  {
    DepolarizerConfig * cfg = (DepolarizerConfig *)(this);
    return *cfg;
  }

  void Depolarisator::SetAttenuation(double att)
  {
    SetAttenuation(att,0);
    SetAttenuation(att,1);
    SetAttenuation(att,2);
  }


  void Depolarisator::SetAttenuation(double att, int chan)
  {
    if(fabs(att)<min_attenuation) 
    {
      att=min_attenuation;
      cerr << "WARINING: Attenuation " << att << " dB less then minimum allowed attenuation " << min_attenuation << " dB" << endl;
    }
    attenuation = att;
    attenuator[chan].SetAttenuation(att);
  }

  double Depolarisator::GetAttenuation(int chan)
  {
    //if(IsScan()) return attenuator[chan].GetAttenuation();
    return attenuation;
  }




  static TRandom RND(23);

  void Depolarisator::Start(void)
  {
    clog << LOG(3) << "Depolarisator::Start: " << endl;
    if(!thread.joinable())
    {
      clog << LOG(5) << "Thread is not zero" << endl;
      SetAttenuation(attenuation); //  allways reset the attenuator 
      generator->Init(); /*  allways init generator when start scan */
      generator->On(); /*  allways turn on geneator when start scan */
      ifreq=initial; /*  set initial frequency to  */
      isscan = true;
      thread = std::thread([this]{Scan();});
    }
    else
    {
			clog << LOG(DEBUG_LOG) << "WARING: Depolarizer is already working" << std::endl;
    }
  }

  void Depolarisator::Start(const DepolarizerConfig & c)
  {
    SetConfig(c);
    Start();
  }

  void Depolarisator::Continue(void)
  {
    if(!thread.joinable())
    {
      ifreq=frequency; /*  set initial frequency to  */
      isscan = true;
      thread = std::thread([this]{Scan();});
    }
    else
    {
			clog << LOG(DEBUG_LOG) << "WARING: Depolarizer is already working" << std::endl;
    }
  }

  void Depolarisator::Scan(void) 
  {
    clog << LOG(DEBUG_LOG) << "Depolarisator::Scan" << endl;
    //double direction  = initial < final ?  1 : -1;
    try
    {
      HighResolutionDepFreq.clear();
      frequency=ifreq;
      ibn::timer timer;
      //ibn::timer timer2;
			//int i=0;

      //auto sys_begin = std::chrono::system_clock::now();
      //auto mono_begin = std::chrono::steady_clock::now();
      while( between(frequency,initial,final) && isscan)
      {
        /*  ifreq is equal initial frequency when start and 
         *  last setup frequency when continue*/
        //frequency=ifreq+direction*timer.elapsed()*speed;
				frequency+=timer.elapsed()*speed;
				//double tt =(++i)*timer.elapsed();
				//double tt2 =timer2.elapsed();
				//cout << " depol elapsed: " << tt  <<  " timer2.elapsed: " << tt2 << " dif: " << tt-tt2 << " " << frequency - (ifreq+direction*timer2.elapsed()*speed) << " " << speed << endl;
				timer.restart();
        CFr = frequency;
        if(Strategy==MONO) 
        {
          auto t1 = std::chrono::system_clock::now();
          //auto st1 = std::chrono::steady_clock::now();
          generator->GenerateFrequency(frequency,scan_time);
          //auto t2 = std::chrono::system_clock::now();
          HighResolutionDepFreq[t1]=frequency;
          //auto sys_dt = t1-sys_begin;
          //auto mono_dt = st1-mono_begin;
          //std::cout << boost::format("system_clock = %20.9f s, steady_clock:: = %20.9f s,  dt = %20.9f us, f = %15.3f Hz") % ((t1-sys_begin).count()*1e-9) % ((st1-mono_begin).count()*1e-9) % ((mono_dt - sys_dt).count()*1e-3) % frequency << std::endl;
        }
        else Commit(CFr);
      }
      ofstream ofs(PCFG.high_resolution_depfreq_file.string());

      /*  
      auto p0 = HighResolutionDepFreq.begin();
      auto t0 = p0->first;
      */

      auto p1 = HighResolutionDepFreq.begin();
      auto p2 = ++HighResolutionDepFreq.begin();
      for(; p2!= HighResolutionDepFreq.end(); p1++, p2++)
      {
        auto t1 = p1->first;
        auto f1 = p1->second;

        /* //Commented for future possible use
        auto t2 = p2->first;
        auto f2 = p2->second;
        auto dt = t2-t1;
        auto df = f2 - f1;
        auto fmt = boost::format("%20.9f %20.6f %20.9f %20.6f");
        auto str = fmt %  ((t1-t0).count()*1e-9) % f1 %  (dt.count()*1e-3) % df ; 
        */
        auto fmt = boost::format("%20.9f %20.6f %20.9f");
        auto str = fmt %  (t1.time_since_epoch().count()*1e-9) % f1 %  energy(f1)  ; 
        //std::cout << str << std::endl;
        ofs << str << std::endl;
      }
      ofs.close();

      //auto t_test = t0;
      //for(int test_t = 10; test_t < 100; test_t += 5)
      //{
      //  auto t_find = t0 + std::chrono::seconds(test_t);
      //  if(test_t == 20) t_test = HighResolutionDepFreq.lower_bound(t_find)->first; 
      //  std::cout << boost::format("%5.1f %20.9f Hz") % test_t % fuzzy_find(HighResolutionDepFreq,t_find) << std::endl;
      //}
      //std::cout << boost::format("%5.1f %20.9f Hz") % ((t_test-t0).count()*1e-9) % fuzzy_find(HighResolutionDepFreq,t_test) << std::endl;
    }
    catch(camac::Module::Error e)
    {
      std::cerr << "polarimeter: camac module error in scaning procedure: " << e.what() << '\n' ;
      std::cerr << "Depolarizer: trying to tern off the depolarizer...";
      generator->Off();
    }
    catch(std::runtime_error e)
    {
      std::cerr <<  "polarimeter: Error in scaning procedure:" << '\n';
      std::cerr <<  "polarimeter:" << e.what() << '\n';
    }
    generator->Off();
    CFr=0;
    isscan = false;
  }

  void Depolarisator::Mute(void)
  {
    generator->Off();
  }

  void Depolarisator::UnMute(void)
  {
    generator->On();
  }


  void Depolarisator::Stop(void)
  {
    StopScan();
  }

  void  Depolarisator::StopScan(void)  
  {
    clog << LOG(5) << "Depolarisator::StopScan: " << endl;
    isscan=false;
    if(thread.joinable()) 
    {
      thread.join();
      //delete thread;
      //thread = 0;
      CFr=0;
      WriteCurrentFrequencyToFile("/home/vepp4/kadrs/depfreq.dat");
    }
    clog << LOG(5) << "Depolarisator::StopScan: FINISH" << endl;
  }


  bool operator<(timeval &t1, timeval &t2)
  {
    return (t1.tv_sec < t2.tv_sec) || ( (t1.tv_sec == t2.tv_sec) && (t1.tv_usec < t2.tv_usec) );
  }

  //inline void Depolarisator::LinearScan(double ff)	
  //{
  //  /* соотношение между шагом по частоте и шириной линии
  //   * нормальная связь, когда шаг в два раза меньше, 
  //   * чем ширина линии, в этом случае должно обеспечиваться
  //   * заметание всех частот.
  //   */
  //  const double KOEF = 2.0; 

  //  double df; //маленький шаг по частоте
  //  double dt; //время генерирования частоты
  //  double delta_f; //ширина линии генератора
  //  double Af;

  //  //df = - Fd + sqrt(Fd * (Fd + 2*Df/KOEF) );  
  //  df = sqrt ( 2 * DFr * FrW / KOEF);  
  //  if(df < 1) df = 1;
  //  delta_f =  KOEF * df;
  //  Af = FrW/2.;
  //  if(Af<0) Af=0; 
  //  /* Таким образом, шаг по частоте всегда больше одного герца,
  //   * ширина полосы всегда болше 2 Гц
  //   * время генерирования всегда меньше 0.5 Гц
  //   * амплитуда  --- всегда неотрицательная
  //   * когда она отрицательная, генерируется исключительно одна 
  //   * частота, без всякой девиации. Нулевое значение можно получить
  //   * просто положив равной нулю ширину полосы.
  //   */
  //  dt = 1/delta_f;
  //  generator->SetTime(dt); //time for generation  frequency

  //  timeval e_time,current_time;
  //  gettimeofday(&current_time,0);
  //  e_time.tv_sec =	current_time.tv_sec + time_t(DeT);
  //  e_time.tv_usec = current_time.tv_usec;

  //  CFr = ff;
  //  double f;
  //  double fmin = ff - FrW/2.;
  //  double fmax = ff + FrW/2.;
  //  unsigned count = 0;
  //  while(current_time < e_time && isscan)
  //  {
  //    try
  //    {
  //      for( f = fmin; f <= fmax; f += df) generator->GenerateFrequency(f);
  //      for( f = fmax; f >= fmin; f -= df) generator->GenerateFrequency(f);
  //      gettimeofday(&current_time,0);
  //      count++;
  //    }
  //    catch ( camac::Module::Error e )
  //    {
  //      std::cerr << LOG(1) << "ERROR :  camac module error in scaning procedure : " << e.what () << '\n';
  //    }
  //    catch(std::runtime_error e)
  //    {
  //      std::cerr <<  LOG(1) << "ERROR:  some runtime_error in scaning procedure: " << e.what() << '\n';
  //    }
  //  }//worked out a point
  //  RDFr=double(count)/DeT;
  //}


  inline double Depolarisator::get_dt(void)
  {
    return DeT/std::max(1000., FrW/0.1);
    //double dt = 1./FrW;
    //if(dt < 0.005) dt = 0.005;
  }

  inline void Depolarisator::LinearScan(double ff)	
  {
    double fmin=ff-width/2.;
    double fmax=ff+width/2.;
    double K=10;
    double M=10;
    double fdot= width*width/(K*K); //velocity of modulating
    //double T2 = width/fdot;     //half period of modulation
    double df = sqrt(fdot)/M; //small step
    double dt =  df/fdot;
    generator->SetTime(dt);
    //double e_time = currenttime() + DeT;
    auto e_time = std::chrono::steady_clock::now() + std::chrono::milliseconds(int64_t(DeT*1e3));
    double f=fmin;
    static int flag=0;
    if(flag==0)
    {
      cout << "Linear scan internal parameters: " << endl;
      cout << "K="<< K << ", M=" << M << endl;
      cout << "Width=" << width << " Hz" << endl;
      cout << "fdot=" << fdot << " Hz/sec" << endl;
      cout << "df=" << df << " Hz" << endl;
      cout << "dt=" << dt << " sec" << endl;
      cout << "tscan=" << df << DeT << endl;
      flag=1;
    }
    while(std::chrono::steady_clock::now() < e_time && isscan)
    {
      generator->GenerateFrequency(f,dt);
      f+=df;
      if(f>fmax) f=fmin;
    }
  }

  inline void Depolarisator::GaussRandomScan(double ff)
  {
    //double dt=get_dt();
    double dt=1./width;
    generator->SetTime(dt); //time for generation  frequency
    //double e_time = currenttime() + DeT;
    auto e_time = std::chrono::steady_clock::now() + std::chrono::milliseconds(int64_t(DeT*1e3));
    CFr=ff;
    double f; 
    //while(currenttime() < e_time && isscan)
    while(std::chrono::steady_clock::now() < e_time && isscan)
    {
      f=RND.Gaus( ff, FrW/2.);
      //clog.precision(20);
      //clog << "RRS: mean freq: " << ff << ", current: " << f << " diff=" <<  f-ff << endl;
      generator->GenerateFrequency(f,dt);
    }
  }

  inline void Depolarisator::UniformRandomScan(double ff)
  {
    generator->SetTime(get_dt()); //time for generation  frequency
    //double e_time = currenttime() + DeT;
    auto e_time = std::chrono::steady_clock::now() + std::chrono::milliseconds(int64_t(DeT*1e3));
    CFr=ff;
    while(std::chrono::steady_clock::now() < e_time && isscan) generator->GenerateFrequency( ff + FrW*( -0.5 + RND.Rndm() ) );
  }

  inline void Depolarisator::MonotonicScan(double ff)	
  {
    generator->SetFrequency(ff);
    CFr=ff;
    ibn::timer timer;
    while(timer.elapsed() < DeT && isscan) usleep(1000);//точность выставки 1мс
  }

  void Depolarisator::InitScan(void)
  {
    isscan = true;
    begin_scan_time = time(0);
    dF = IFr < FFr ?  FrS : -FrS; 
    next_fr = IFr;
    CFr = IFr;
  }

  void Depolarisator::NextPoint(void)	
  {
    std::cout << "Next point " << std::endl;
    if(!isscan)
    {	
      CFr=0;
      return;
    }
    CFr = next_fr;

    if( !between(CFr,IFr,FFr)  )
    {
      isscan = false;
      CFr=0;
      generator->Off();
      return;
    }
    Commit(CFr);
		clog << CFr << " " << dF << endl;
    next_fr+=dF;
  }

  void Depolarisator::Commit(double ff)	
  {
    try
    {
      switch(Strategy)	
      {
        case LINEAR:
          LinearScan(ff);
          break;
        case GRND:
          GaussRandomScan(ff);
          break;
        case URND:
          UniformRandomScan(ff);
          break;
        case MONO:
          MonotonicScan(ff);
          break;
        default:
          LinearScan(ff);
          break;
      }
    }
    catch ( camac::Module::Error e )
    {
      std::cerr << LOG(2) << "ERROR :  camac module error in scaning procedure : " << e.what () << '\n';
    }
    catch(std::runtime_error e)
    {
      std::cerr << LOG(2) << "polarimeter: Error in scaning procedure:" << e.what() << '\n';
    }
  }




  void Depolarisator::Bind(const camac::Address & g)
  {
    using namespace camac;
    generator->Bind(g);
    clog << LOG(4) << "Generator: " << g.interface<<":"<<g.crate<<":"<<g.station<<":"<<g.channel << "\n";
    Strategy = MONO;
    isscan=false;
  }


  void Depolarisator::WriteCurrentFrequencyToFile(std::string file)
  {
    /*
     * Write current depolarizer energy
     */
    std::ofstream depfreq_file("/home/vepp4/kadrs/depfreq.dat");
    if(depfreq_file)
    {
      time_t current_time =time(0);
      depfreq_file << boost::format("%-20d # %s") % current_time  % ctime(&current_time);
      depfreq_file << boost::format("%-20.3f # %s") % CFr % "end point frequency, Hz" << std::endl;
      depfreq_file << boost::format("%-20.6f # %s") % (CFr!=0 ? energy(CFr) : 0) %  "corresponding energy, MeV" << std::endl;
    }
  }

} //namespace measurement


