#ifndef IBN_DEPOLARISATOR_H
#define IBN_DEPOLARISATOR_H

#include "camac/Generator.h"
#include "camac/Attenuator.h"

#include "Measurement.h"
#include "DepolarizerConfig.h"

#include <thread>
#include <unistd.h>


namespace measurement 	
{
  class EnergyCalibration;

  class Depolarisator  : private DepolarizerConfig
  {
    public:
      camac::Generator * generator;

      Depolarisator(void);
      Depolarisator(const camac::Address &);
      //Depolarisator(const camac::Address &, const camac::Address &, const camac::Address &);
      ~Depolarisator(void){}

      enum  STRATEGY { LINEAR, GRND, URND,MONO};
      enum  MODE { SYNC };

			camac::AttenuatorArray attenuator[3];

      void SetConfig(const measurement::DepolarizerConfig &); //Configurate depolarizer using energy inits (MeV)
      void SetGenerator(camac::Generator *g) {generator = g; } /* Set generator */
      void SetAttenuation(double att);
      void SetAttenuation(double att, int chan);
      void SetStrategy(STRATEGY str) { Strategy = str; }


      DepolarizerConfig GetFrequencyConfig(void);
      double GetAttenuation(int chan=0);
      STRATEGY GetStrategy(void) const {return Strategy; }
      //double GetScanFrequency(void) {return CFr; }
      double GetFrequency(void) { return CFr;}
      double GetInstantFrequency(void) { return CFr; }


      void   SetInitialFrequency(double f)  { initial=f; IFr = f;}
      void   SetMinimumAttenuation(double matt) { min_attenuation = fabs(matt);};
      double GetMinimumAttenuation(void) const {return min_attenuation; };
			void   SetFrequencyStep(double s) { FrS = s; dF = IFr < FFr ?  FrS : -FrS; speed=s/scan_time; }
			double GetFrequencyStep(void) const  {return FrS;}
			double GetScanSpeed(void) const {return speed; } //in Hz
			void   SetScanSpeed(double s) 
			{ 
				speed= final > initial ? fabs(s) : -fabs(s); 
				FrS = s*scan_time; 
				dF = IFr < FFr ?  FrS : -FrS;
			}
      //void SetScanParameters(EnergyCalibration *) ;
      //void SetFinalFrequency(double f)  	{ FFr = f;}
      //void SetFrequencyStep(double step)  { FrS = step;}
      //void SetDeviationFrequency(double fdev) { DFr = fdev;	}
      //void SetFrequencyWidth(double f)  { FrW = f; }
      //void SetDeviationTime(double t)  { DeT = t;  }


      //void GetScanParameters(EnergyCalibration *);
      //double GetCurrentFrequency(void) const { return CFr; }
      //double GetFrequencyStep(void) const { return FrS;   }
      //double GetFrequencyWidth(void) const { return FrW;   }
      //double GetInitialFrequency(void) const { return IFr;   }
      //double GetFinalFrequency(void) const { return FFr;   }
      //double GetDeviationFrequency(void) const {	return DFr;  }
      //double GetRealDeviationFrequency(void) const {  return RDFr; }
      //double GetDeviationTime(void) const  { return DeT;  }


      time_t GetBeginTime(void)   const { return begin_scan_time; }
      time_t GetScanTime(void)    const { return time(0) - GetBeginTime();    }

      bool IsScan()  const   { return isscan;   }

      void Start(const DepolarizerConfig & c);
      void StopScan(void);
      void PauseScan(void);
      void UnPauseScan(void);

      /* Control the depolarizer state */
      void Start(void); /* Turn ON generator and  Change to state SCAN */
      void Stop(void); /*  turn of the depolarizer and change the state to STOP*/
      void Continue(void); /* Continue scanning from last frequency */
      void Mute(void); /*  Turn off generator does not metter the state of depolarizer (STOP or SCAN) */
      void UnMute(void); /*  Turn on generator  */
      bool IsMute(void) { return !generator->IsOn(); }

      static bool between(double x, double x1, double x2) { return  (x1<=x && x<=x2) || (x2<=x && x<=x1); }

      void InitScan(void);	
      void NextPoint(void);


      void Bind(const camac::Address &);

    private:
      double delay; 
      STRATEGY Strategy; //Scaning Algortthm 
      //MODE 		 Mode;
      double IFr;  //Initial Frequency,Hz
      double FFr;  //Final Frequency, Hz
      double FrW;  //width of frequency deviation, Hz
      double DFr;  //deviadion Frequency, Hz
      double DeT;  //time scan per point, sec
      double FrS;  //frequency step,  Hz
      double dF; //frequency step which sing depend on up or down scan
      double CFr;  // Current Frequency, Hz
      double next_fr; // next frequency.
      double RDFr; // Real deviation frequency
      time_t begin_scan_time;
      void Commit(double );
      double min_attenuation;
    public:

      void WriteCurrentFrequencyToFile(std::string filename);
      std::map<std::chrono::system_clock::time_point, double>  HighResolutionDepFreq;

    private:
			void CreateAttenuators(void);
			void Init(void);

      inline void LinearScan(double F);
      inline void GaussRandomScan(double F);
      inline void UniformRandomScan(double F);
      inline void MonotonicScan(double ff);
      inline double get_dt(void);

      double frequency; /*  current frequency  */
      double ifreq; /* initial frequency  */ 
      volatile bool isscan; /*  flag for background measurement */
      /* The states of the depolarizer  */
      std::thread  thread; /* thread for background measurement */
      void Scan(void);

  };
} //namespace measurement
#endif // IBN_DEPOLARISATOR_H
