#include "PhysConst.h"
#include "DepolarizerConfig.h"
namespace measurement
{
	/* *****************************************************************************************
	 * calculate frequency (Hz) from energy (MeV) using revolution frequency and harmonic number
	 ******************************************************************************************/
	double DepolarizerConfig::frequency(double E)  const
	{
		using namespace phys;
		return revolution_frequency.value*( muea*E/me - harmonic_number);
	}

	/* *****************************************************************************************
	 * calculate energy (MeV) from  frequency (Hz) using revolution frequency and harmonic number
	 ******************************************************************************************/
	double DepolarizerConfig::energy(double f) const
	{
		using namespace phys;
		double x = f/revolution_frequency.value;
		return me/muea*(harmonic_number + x);
	}

	/* *****************************************************************************************
	 * calculate step frequency (Hz) from step energy (MeV) using revolution frequency and harmonic number
	 ******************************************************************************************/
	double DepolarizerConfig::delta_frequency(double dE)  const
	{
		using namespace phys;
		return revolution_frequency.value*muea*dE/me;
	}

	/* *****************************************************************************************
	 * calculate energy step (MeV) from  frequency step(Hz) using revolution frequency and harmonic number
	 ******************************************************************************************/
	double DepolarizerConfig::delta_energy(double df) const
	{
		using namespace phys;
		return me/muea*df/revolution_frequency.value;;
	}
}



measurement::DepolarizerConfig to_frequency(const measurement::DepolarizerConfig &c)
{
	measurement::DepolarizerConfig f;
	f.scan_time = c.scan_time;
	f.attenuation = c.attenuation;
	f.direction = c.direction;
	f.harmonic_number = c.harmonic_number;
	f.RF = c.RF;
	f.revolution_frequency = c.revolution_frequency;
	f.algorithm = c.algorithm;
	f.voltage = c.voltage;
	f.direction = c.direction;
	f.isenergy = 0;

	f.speed = c.delta_frequency(c.speed);
	f.step  = c.delta_frequency(c.step);
	f.initial = c.frequency(c.initial);
	f.final = c.frequency(c.final);
	f.width = c.delta_frequency(c.width);
	f.expected = c.frequency(c.expected);
	f.range = c.delta_frequency(c.range);
	f.current = c.frequency(c.current);
	f.measured =c.frequency(c.measured);
	return f;
}

measurement::DepolarizerConfig to_energy(const measurement::DepolarizerConfig &c)
{
	measurement::DepolarizerConfig E;
	E.scan_time = c.scan_time;
	E.attenuation = c.attenuation;
	E.direction = c.direction;
	E.harmonic_number = c.harmonic_number;
	E.RF = c.RF;
	E.revolution_frequency = c.revolution_frequency;
	E.algorithm = c.algorithm;
	E.voltage = c.voltage;
	E.direction = c.direction;
	E.isenergy = 1;

	E.speed = c.delta_energy(c.speed);
	E.step  = c.delta_energy(c.step);
	E.initial = c.energy(c.initial);
	E.final = c.energy(c.final);
	E.width = c.delta_energy(c.width);
	E.expected = c.energy(c.expected);
	E.range = c.delta_energy(c.range);
	E.current = c.energy(c.current);
	E.measured =c.energy(c.measured);
	return E;
}
