#ifndef SMV_MEASUREMENT_MEASUREENERGECALIBRATION_H
#define SMV_MEASUREMENT_MEASUREENERGECALIBRATION_H

#include <iostream>
#include <string>

#include <boost/signals2.hpp>
#include <thread>
#include <atomic>

#include "Log.h"
#include "ibn/timer.h"
#include "data/Data.h"
#include "data/EnergyCalibrationStorage.h"

#include "Time.h"

#include "Measurement.h"

class PolarimeterConfig;

namespace data
{
  class CountStorage;
}

namespace camac
{
  class EnergyCalibrationCamac;
}

namespace measurement
{

  typedef double Value_t;

  class Depolarisator;
  class CurrentMeasurement;
  class BeamSizeMeasurement;
  class FrequencyMeasurement;
  class CountMeasurement;

  class EnergyCalibration : public Measurement
  {
    public:
      EnergyCalibration(unsigned number_of_bunches, unsigned number_of_counters);
      virtual ~EnergyCalibration(void);
      void Start(void);	
      void Stop(void);
      void Pause(void);
      void UnPause(void);
      void Continue(void);
			bool IsMeasure(void) const {return ismeasure;};
      data::EnergyCalibrationStorage* GetStorage(void);	

      //void Start(std::string);	
      bool isMeasure(void);	
      bool isPause(void);		
      bool isNewPoint(void);
      void SetNewPoint(void);

      data::EnergyCalibrationStorage* ECStorage; /* contaner of measurement result */
      camac::EnergyCalibrationCamac* ECCamac;    /* camac hardware */
      void DeleteOldData(void); //удаляет данные из data::EnergyCalibrationStorage от страрого измерения
      //void SendCount2Odrenok(const vector <double> & N, const vector<double> & delta);
      //bool IsUDAR();

      void ToggleDelayLineMeasurement(void)
      {
        is_delay_line_measure = is_delay_line_measure ? false : true;
      }
      bool IsDelayLineMeasurement(void) const { return is_delay_line_measure; }

    private:
      bool freqmeasure;
      bool ispause;	 //если оператор нажал кнопку pause, эта переменная = true. Функции 
      ibn::timer global_timer; //считает время с начала измерения
      Timer<std::chrono::system_clock>  Global_timer; 
      ibn::timer point_timer;  //отсчитывает время с начала измерения текущей точки
      std::chrono::system_clock::time_point last_time_sent_to_lsrp=std::chrono::system_clock::now(); 
      bool is_delay_line_measure; //change delay line for every new points

      void Prepare(void);

      void MakeNewPoint(void);

      void FillNMR(void);
      CurrentMeasurement   * fCurrentMeasurement;  //store result of the current measurement
      BeamSizeMeasurement  * fBeamSizeMeasurement;  //store result of beam size measurement
			FrequencyMeasurement * fFrequencyMeasurement;
      CountMeasurement     * fCountMeasurement;
      data::CountStorage   * fCountStorage; 

      static const unsigned long MAX_MEASUREMENT_TIME = 10000;//максималоное время измерения


      const unsigned numbunch;//число пучков
      const unsigned numcounter;//число счетчиков
      Value_t GetRadioFrequency(void);
    private:
			volatile bool ismeasure; //flag for count measurement
			std::thread fThread;
      void Measure(void);

      volatile bool isstatusmeasure; /*  flag for status measurement */
      std::unique_ptr<std::thread> status_thread; /*  status measurement thread */

      volatile bool is_new_point; //is new ponint available
      boost::signals2::connection point_yeld_connection;
    public:
      //some signals
      boost::signals2::signal <void(void)> sig_new_point; // Информирует о появлении новой точки.

      void SetCamac(camac::EnergyCalibrationCamac * c) { ECCamac = c; }
      void SetCurrentMeasurement(CurrentMeasurement * c)    { fCurrentMeasurement   = c;}
      void SetBeamSizeMeasurement(BeamSizeMeasurement *b)   { fBeamSizeMeasurement  = b;}
      void SetFrequencyMeasurement(FrequencyMeasurement *f) { fFrequencyMeasurement = f;}
      void SetCountStorage( data::CountStorage * cs) { fCountStorage = cs; }
      void SetCountMeasurement( CountMeasurement * c) { fCountMeasurement = c; }
  };
}//end of namespace measurement

#endif //SMV_MEASUREMENT_MEASUREENERGECALIBRATION_H
