//Модуль Camac отвечает за железо. Здесь объявлены все используемые блоки.

#ifndef SMV_MEASUREMENT_ENERGYCALIBRATIONCAMAC_H
#define SMV_MEASUREMENT_ENERGYCALIBRATIONCAMAC_H

#include <iostream>
#include <string>
#include <vector>
#include <stdio.h>
#include <list>
#include <memory>

#include "camac/Vvi.h"
#include "camac/Concur.h"
#include "camac/Counter.h"
#include "camac/Delay.h"
#include "camac/Diskr.h"
#include "camac/Form.h"
#include "camac/Delay.h"
#include "camac/ModuleInventory.h"
#include "camac/Timer.h"
#include "camac/Freq.h"

#include "MeasurementCamac.h"
#include "Depolarisator.h"

#include <mutex>


namespace camac
{
  class EnergyCalibrationCamac : public MeasurementCamac
  {
    typedef double Value_t;
    public:
    EnergyCalibrationCamac(void);
    EnergyCalibrationCamac(unsigned , unsigned );
    ~EnergyCalibrationCamac(void);
    std::vector<std::vector<Value_t> > GetData(void);/* read data from camac for counters*/
    std::vector< std::vector<Value_t> > GetCoinc(void); /*  read data from camac for pair of counter (coincidece) */

    std::vector<std::vector<Value_t> > GetDataWithout_reset(void);
    std::vector< std::vector<Value_t> > GetCoincWithout_reset(void);

    void ResetCounters(void);

    void StartTimer(double time,  volatile bool & iswait);
    void StartTimer(void);
		void StopTimer(void);
    void WaitTimer(const volatile bool & iswait);
    bool GetStatusTimer(void);
    void OnVvi(unsigned, unsigned); //Switch on vvi. The first parameter is vvi number, the second - volt

    std::vector<int> GetVviStatus(void);

		camac::Address GetVviAddress(unsigned counter) const;
		bool IsOn(unsigned counter) const;
		bool IsOverload(unsigned counter) const;
		double GetVoltage(unsigned counter) const ;
		void SetVoltage(unsigned counter,  double voltage);
		void Off(unsigned counter);
		void On(unsigned counter);
		void Reset(unsigned counter);

    void SetDiskr(unsigned num_counter, unsigned treshold);

    void SetBunchDelay(unsigned counter_pair, unsigned bunch, unsigned delay);
    unsigned GetBunchDelay(unsigned counter_pair, unsigned bunch);
    void SetDelay(unsigned delay);


		std::map<std::string,  std::shared_ptr<camac::ModuleInventory> > fModules;

    //std::list<ModuleInventory* > ModuleList; //Список БЛОКОВ, который передается в Config

    unsigned GetBunchNumber(void) const { return numbunch;}
    unsigned GetCounterNumber(void) const { return numcounter;}

    measurement::Depolarisator * depolarizer; /*  depolarizer */
    camac::Generator * generatorDDS300; 
    camac::Generator * generatorDDS500;
    camac::Generator * generatorDDS500_2;
    camac::GeneratorArray * double_generator;
    DelayLine* DepolarizerDelayLine;
    DelayLine* DepolarizerDelayLine2;


    camac::Freq * frequencymeter; //частотомер


    private:
    // Scheme of counter location
    //        
    //                                   EXTERNAL
    //
    //                  2             4     6                 8
    // e- bunch -->	 ----depolarizer-...---...--KEDR--...------->
    //                  1             3     5                 7
    //
    //	                                 INTERNAL              
    //	


    const unsigned numbunch;  /* number of bunches */
    const unsigned numcounter;/* number of counters */
    std::vector < std::vector <ModuleInventory*> > Concur; /* Coincidence scheme     Concur[bunch][Concur] */
    std::vector < std::vector <ModuleInventory*> > ConcurCoinc; /* Coincidence scheme ConcurCoinc[bunch][pair] */
    std::vector < std::vector <CounterC0303*> > Count;/* counters Counter[bunch][Count] */
    std::vector < std::vector <CounterC0303*> > CounterCoinc; /* пересчётки для совпадений Counter[bunch][pair] */

    std::vector < ModuleInventory* > Vvi;
    std::vector < ModuleInventory* > Diskr;
    std::vector < ModuleInventory* > DelayLineCounter; /* delay line for counters */
    std::vector < std::vector <DelayLine*> > DelayLineBunch;

    std::vector < Timer* > Timer_;


    std::mutex   counter_mutex;  /* mutex for counters */
    std::mutex   coincidence_counter_mutex;  /* mutex for coincidence counters */
    std::mutex   timer_mutex; /*  mutex for timer control */

		volatile bool timer_is_started=false;

		public:
		void ShowElectronDepolarizerAmplitude(void);
		void ShowPositronDepolarizerAmplitude(void);
		void ShowDepolarizerAmplitude(int channel);
		int WhichAmplitudeToShow(void);

		template<class CamacModule> CamacModule * getModule(std::string name);
		//shared_ptr<CamacModule>  getModule(std::string name);
  };	



}//end of namespase camac


#endif //SMV_MEASUREMENT_ENERGYCALIBRATIONCAMAC_H
