#include <boost/property_tree/json_parser.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <boost/property_tree/info_parser.hpp>

#include "EnergyCalibrationConfig.h"
#include "data/ArraySerializaton.h"
#include "PolarimeterConfig.h"

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>
namespace measurement
{
	EnergyCalibrationConfig::EnergyCalibrationConfig(void)
	{
		run=0;
		name="";
		type="";
		experiment_name="";
		date=0;
		count_time=1;
		delta=0;
		nmr=0;
		temperature=0;
		N=0;
		energy=0;
		alpha_t = 0;
		fit_method = "LINEAR_JUMP";
		fit_graph  = "delta-1";
		pass=0;
		shiftman="";
    reason = 0;
		std::vector<int> v{2, 1, 4, 3};
		bunch_norm.resize(PCFG.bunch_number);
		for(int i=0;i< std::min(v.size(), bunch_norm.size()); i++)
		{
			bunch_norm[i] = v[i];
		}
		depolarizer.speed=0;
		depolarizer.step=0;
		depolarizer.width=0;
		depolarizer.scan_time = 1;
		depolarizer.attenuation=60;
		depolarizer.harmonic_number=5;
		depolarizer.initial=0;
		depolarizer.final=0;
		depolarizer.algorithm=0;
		depolarizer.voltage=0;
    depolarizer.range = 0;
	}

	EnergyCalibrationConfig::EnergyCalibrationConfig(const boost::property_tree::ptree & pt)
	{
		pt >> *this;
	}


	template<typename Container>
		std::string to_string( const Container & list)
		{
			std::ostringstream oss;
			for(auto  & l : list)
			{
				oss << l << " ";
			}
			return oss.str();
		}


	template <typename T>
		std::vector<T> as_vector(boost::property_tree::ptree const& pt, boost::property_tree::ptree::key_type const& key)
		{
			std::vector<T> r;
			std::string s = pt.get<std::string>(key);
			std::stringstream ss(s);
			T data;
			while(ss>>data) r.push_back(data);
			return r;
		}


	const boost::property_tree::ptree & operator>>(const boost::property_tree::ptree & pt,  measurement::EnergyCalibrationConfig & cfg)
	{
    cfg.name            = pt.get<std::string>("name",cfg.name);
    cfg.type            = pt.get<std::string>("type",cfg.type);
		cfg.experiment_name = pt.get<std::string>("experiment_name", cfg.experiment_name);
		cfg.run             = pt.get<unsigned>("run", cfg.run);
		cfg.date            = pt.get<time_t>("date", cfg.date);
		cfg.energy          = pt.get<ibn::valer<double>>("energy", cfg.energy);
		cfg.reason          = pt.get<int>("reason", cfg.reason);

		cfg.depolarizer.generator_name = pt.get<std::string>("generator_name", cfg.depolarizer.generator_name);
		cfg.depolarizer.generator_type = pt.get<double>("generator_type", cfg.depolarizer.generator_type);
		cfg.depolarizer.speed          = pt.get<double>("speed", cfg.depolarizer.speed);
		cfg.depolarizer.step           = pt.get<double>("step", cfg.depolarizer.step);
		cfg.depolarizer.attenuation    = pt.get<double>("attenuation", cfg.depolarizer.attenuation);
		cfg.depolarizer.scan_time      = pt.get<double>("scan_time", cfg.depolarizer.scan_time);
		if(cfg.depolarizer.speed == 0) 
		{
			cfg.depolarizer.step = 0;
			cfg.depolarizer.scan_time = 100;
		}
		else
		{
			cfg.depolarizer.scan_time   = pt.get<double>("scan_time",  cfg.depolarizer.step/cfg.depolarizer.speed);
		}
		cfg.depolarizer.initial         = pt.get<double>("initial", cfg.depolarizer.initial);
		cfg.depolarizer.final           = pt.get<double>("final", cfg.depolarizer.final);
		cfg.depolarizer.harmonic_number = pt.get<int>("harmonic_number", cfg.depolarizer.harmonic_number);
		cfg.depolarizer.width           = pt.get<double>("width", cfg.depolarizer.width);
		cfg.depolarizer.voltage         = pt.get<double>("voltage", cfg.depolarizer.voltage);
		cfg.depolarizer.expected        = pt.get<double>("expected", cfg.depolarizer.expected);
		cfg.depolarizer.measured        = pt.get<double>("expected", cfg.depolarizer.measured);
		cfg.depolarizer.direction       = pt.get<int>("direction", cfg.depolarizer.direction);
    cfg.depolarizer.range           = pt.get<double>("range", cfg.depolarizer.range);

		cfg.count_time  = pt.get<double>("count_time", cfg.count_time);
		cfg.delta       = pt.get<ibn::valer<double>>("delta", cfg.delta);
		cfg.nmr         = pt.get<double>("nmr", cfg.nmr);
		cfg.temperature = pt.get<double>("temperature", cfg.temperature);
		cfg.alpha_t     = pt.get<double>("alpha_t", cfg.alpha_t);
		cfg.fit_method  = pt.get<std::string>("fit_method", cfg.fit_method);
		cfg.fit_graph   = pt.get<std::string>("fit_graph", cfg.fit_graph);
		cfg.shiftman    = pt.get<std::string>("shiftman", cfg.fit_method);

		if(pt.get_child_optional("norm_bunch"))
		{
			auto nb =   as_vector<unsigned>(pt, "norm_bunch");
			for(int i=0;i<std::min(cfg.bunch_norm.size(), nb.size());i++)
			{
				cfg.bunch_norm[i] = nb[i];
			}
		}
		return pt;
	}

	boost::property_tree::ptree & operator<<(boost::property_tree::ptree & pt,  const measurement::EnergyCalibrationConfig & cfg)
	{
		//boost::property_tree::xml_parser::read_xml(PCFG.measurement_config_file.string(), tree, boost::property_tree::xml_parser::trim_whitespace);
		//auto & pt = tree.get_child("measurement_cfg.EnergyCalibration.last");
		pt.put("name", cfg.name);
		pt.put("type", cfg.type);
		pt.put("experiment_name", cfg.experiment_name);
		pt.put("run", cfg.run);
		pt.put("reason", cfg.reason);
		pt.put("pass", cfg.pass);
		pt.put("date", cfg.date);
		pt.put("energy", cfg.energy);
		pt.put("generator_name", cfg.depolarizer.generator_name);
		pt.put("generator_type", cfg.depolarizer.generator_type);
		pt.put("attenuation", cfg.depolarizer.attenuation);
		pt.put("speed", cfg.depolarizer.speed);
		pt.put("step", cfg.depolarizer.step);
		pt.put("scan_time", cfg.depolarizer.scan_time);
		pt.put("initial", cfg.depolarizer.initial);
		pt.put("final", cfg.depolarizer.final);
		pt.put("direction", cfg.depolarizer.direction);
		pt.put("width", cfg.depolarizer.width);
		pt.put("harmonic_number", cfg.depolarizer.harmonic_number);
		pt.put("count_time", cfg.count_time);
		pt.put("norm_bunch",  to_string(cfg.bunch_norm));
		pt.put("delta", cfg.delta);
		pt.put("nmr", cfg.nmr);
		pt.put("temperature", cfg.temperature);
		pt.put("alpha_t", cfg.alpha_t);
		pt.put("fit_method", cfg.fit_method);
		pt.put("fit_method", cfg.fit_graph);
		pt.put("shiftman", cfg.shiftman);
		time_t t = time(0);
		pt.put("config_save_time", ctime(&t));
		return pt;
	}

} //namespace measurement

