/*
 * =====================================================================================
 *
 *       Filename:  EnergyCalibrationConfig.h
 *
 *    Description:  Конфигурация для калибровки энергии
 *
 *        Version:  1.0
 *        Created:  02/21/2008 06:35:57 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physics
 *
 * =====================================================================================
 */

#pragma once


#include <string>
#include <iostream>

#include "Config.h"
#include "DepolarizerConfig.h"

#include <boost/property_tree/ptree.hpp>

#include <boost/serialization/nvp.hpp>
#include <boost/serialization/access.hpp>

#include "Log.h"
#include "data/Storage.h"

using namespace boost::serialization;

namespace measurement 
{
	class EnergyCalibrationConfig : public Config
	{
		public:
			EnergyCalibrationConfig(void);
			EnergyCalibrationConfig(const boost::property_tree::ptree & ptree);
			~EnergyCalibrationConfig(void){}
			unsigned run;
			std::string name;
			std::string type; //measurement type (Double, Single,  etc...)
			std::string experiment_name; //global experiment name
			time_t date;
			double count_time;
			data::PhysValue energy;
			data::PhysValue delta;
			DepolarizerConfig depolarizer;
			double nmr;
			double temperature;
			double alpha_t;
			int N;
			int pass; //the scan pass number
			std::vector<unsigned> bunch_norm; //vector elemet storages bunch number which normalization on. Element number is bunch number 
			std::string fit_method;
			std::string fit_graph;
			std::string shiftman;
      int reason; //the reseason of the calibration
		private:
			friend class boost::serialization::access;
			template<class Archive> 
				void serialize(Archive & ar,  const unsigned int version);
	};
	inline std::ostream & operator<<(std::ostream & os, const measurement::EnergyCalibrationConfig & cfg)
	{
		using namespace std;
		os <<  cfg.run <<  " # run " << endl;
		os <<  cfg.name << " # name " << endl;
		os <<  cfg.date << " # date " << endl;
		os <<  cfg.depolarizer.direction << " # direction " << endl;
		os <<  cfg.count_time << " # count time ,sec " << endl;
		os <<  cfg.depolarizer << endl;
		//os <<  cfg.bunch_norm << " # norm bunch " << endl;
		os <<  cfg.nmr <<  " # nmr " << endl;
		os <<  cfg.temperature << " # temp " <<  endl;
		os <<  cfg.alpha_t << " # alpha_t " << endl;
		os <<  cfg.N << " # N " << endl;
		os <<  cfg.delta << " #delta" << endl;
		os <<  cfg.energy << " # energy " << endl;
		return os;
	}
	boost::property_tree::ptree & operator<<(boost::property_tree::ptree & pt,  const measurement::EnergyCalibrationConfig & cfg);
	const boost::property_tree::ptree & operator>>(const boost::property_tree::ptree & pt,  measurement::EnergyCalibrationConfig & cfg);
}; //end of namespace measurement



BOOST_CLASS_VERSION(measurement::EnergyCalibrationConfig,data::Storage::VERSION);


