#include "FrequencyMeasurement.h"
#include <Log.h>
#include "PolarimeterConfig.h"

namespace measurement
{
	FrequencyMeasurement::FrequencyMeasurement(void)
	{
		fIsMeasure = false;
		fFrequencymeter = 0;
	}

	FrequencyMeasurement::~FrequencyMeasurement(void)
	{
		Stop();
	}

	void FrequencyMeasurement::Start(void)
	{
		if(!fThread.joinable())
		{
			fThread = std::thread([this] {Measure();});
		}
		else
		{
			clog << LOG(DEBUG_LOG) << "WARING: Frequency already measured" << std::endl;
		}
	}

	void FrequencyMeasurement::Stop(void)
	{
		clog << LOG(DEBUG_LOG) << "FrequencyMeasurement::Stop BEGIN" << std::endl;
		if(fThread.joinable())
		{
			fIsMeasure=false;
			fThread.join();
		}
		clog << LOG(DEBUG_LOG) << "FrequencyMeasurement::Stop END" << std::endl;
	}

	void FrequencyMeasurement::Measure(void)
	{
    std::clog << LOG(DEBUG_LOG) << "FrequencyMeasurement::Measure BEGIN" << std::endl;
		fIsMeasure = true;
		double freq;
		while(fIsMeasure)
		{
			if(fFrequencymeter != nullptr)
			{
				try
				{
					if(!PCFG.nocamac)
					{
						freq = fFrequencymeter->GetFrequency();
						std::lock_guard<std::mutex> lock(mutex);
						fFa.add(freq);
					}
					else
					{
						freq =  0;
					}
					fCurrent_freq = ibn::valer<double>(freq, 1);
					if(fFa.size()!=0)
					{
						fFreq = ibn::valer<double>( fFa.average(),  fFa.rms());
						//std::cout << "current freq = " << freq << "   average freq = " << fFreq.value << " +- " << fFreq.error << "  size=" << fFa.size() << std::endl;
					}
					else
					{
						fFreq = fCurrent_freq;
					}
				}
				catch(std::runtime_error e)
				{
					clog << LOG(DEBUG_LOG) << "ERROR in Frequency measurement:  " << e.what()<< std::endl;
				}
			}
			else 
			{
				clog << LOG(1) << "WARING: Please set up frequencymeneter" << std::endl;
			}
			sleep(1);
		}
    std::clog << LOG(DEBUG_LOG) << "FrequencyMeasurement::Measure STOP" << std::endl;
	}

	void FrequencyMeasurement::MakeNewPoint(void)
	{
		std::lock_guard<std::mutex> lock(mutex);
		fFa.reset();
	}
}
