
#include <fstream>

#include "Log.h"

#include "Frequencymeter.h"

namespace   measurement
{
Frequencymeter * FREQUENCYMETER;

Frequencymeter::Frequencymeter(const camac::Address &low, const camac::Address &high)  : camac::Freq(low,high) {}

double Frequencymeter::GetMeasuredFrequency(void)
{
	if(ismeasure) return MFr;
	else return Freq::GetFrequency();
}

void Frequencymeter::StartMeasurement(void)
{
	FREQUENCYMETER = this;
    ismeasure = true;
	pthread_create(&thread, 0,Measure,0);
}

void Frequencymeter::StopMeasurement(void)
{
	ismeasure = false;
	cerr << "before pthread stop for frequencymeter" << endl;
	pthread_join(thread,0);
	cerr << "after pthread in frequencymeter" << endl;
}

void Frequencymeter::WriteDataFile(void)
{
	const char * depfreq_file_name = "/home/vepp4/kadrs/depfreq.dat";
	ofstream file (depfreq_file_name);
	if(!file)
  {
		cerr << "polarimeter: error opening file " << depfreq_file_name << endl;
	}
	file.precision(10);
	file << MFr << endl << flush;
	file.close();	
}

void * Frequencymeter::Measure(void * )
{
	Frequencymeter * freq = FREQUENCYMETER;
	while(freq->ismeasure)
  {
			while(freq->GetStatus() && freq->ismeasure) sleep(1);
			if(freq->ismeasure)
      {
				try
        {
					freq->MFr = freq->GetFrequency();
					freq->WriteDataFile();
				}	
				catch ( std::range_error e)
        {
					cerr << LOG (1) << "WARNIGN: frequency measurement: " << e.what() << endl;
				} 
				catch ( std::invalid_argument e)
        {
					cerr << LOG (1) << "WARNIGN: frequency measurement: invalid_argument:  " << e.what()  << endl;
				}
				catch (camac::Module::Error e)
        {	
					cerr << LOG (1) << "Frequency measurement error: camac module error: " << e.what() << endl;	
				}
				catch (camac::Crate::Error e)
        {  
					cerr << LOG (1) << "Frequency measurement error: crate error: "  << e.what() << endl;	
				}
			}
			else freq->MFr=0;
	}
	freq->MFr = 0;
	freq->WriteDataFile();
	return 0;
};

} //end of namespase measurement

