#ifndef SMV_MEASUREMENT_MANAGER_H
#define SMV_MEASUREMENT_MANAGER_H

#include <iostream>
#include <string>
#include <map>
#include <memory>

#include "data/Storage.h"
#include "data/PolarimeterStatus.h"
#include "Measurement.h"
#include "EnergyCalibration.h"
#include "CounterCalibrate.h"
#include "Depolarisator.h"



namespace measurement
{

  extern const double PRECISION;//точность измерения.


  class Manager
  {
    public:
      static bool isnocamac; //true if works in nocamac mode

      Manager(unsigned nbunchs, unsigned ncounters);
      ~Manager(void);
			void Stop(void); //stop all measurements

      //GUI сам кидает в Storage параметры измерения
      void  Start(std::string); 
      void  Stop(std::string);
      void  Pause(std::string);
      inline void  UnPause(std::string);
      inline void  Continue(std::string);
      bool IsMeasure(std::string/*назавания измерения*/);
      data::Storage* GetStorage(std::string);

      unique_ptr<camac::EnergyCalibrationCamac>  fCamac;

      measurement::EnergyCalibration* EC;
      //measurement::Depolarisator*     DepolarisatoR; //generator + attunuator
      measurement::CounterCalibrate_by_Landao* CounterCalibrate_Landao; //for measuremetn counter efficiency 
      measurement::Depolarisator* GetDepolarisator(void);
      data::Status & GetStatus(void); //measurement status

      const unsigned numbunch;//число пучков
      const unsigned numcounter;//число счетчиков
      Measurement * GetMeasurement(std::string name) { return fMeasurementList[name].get(); }
    private:
      std::map<std::string, std::shared_ptr<Measurement> > fMeasurementList;//Списоск доступных измерений		
  };

}//end of namespace measurement

#endif //SMV_MESUREMENT_MANAGER_H
