#include "StatusMeasurement.h"
#include "EnergyCalibrationCamac.h"
#include "CurrentMeasurement.h"
#include "CountMeasurement.h"

#include <boost/asio.hpp>

namespace measurement
{
  StatusMeasurement::StatusMeasurement(void)
  {
    fStatus.BunchInfo.resize(PCFG.bunch_number, data::BunchInfo_t(PCFG.counter_number));
  }

  StatusMeasurement::~StatusMeasurement(void)
  {
    Stop();
  }

  void StatusMeasurement::Start(void)
  {
    if(!fThread.joinable() && !ismeasure)
    {
      ismeasure = true;
      fThread =std::thread([this]{Measure();});
    }
  }

  void StatusMeasurement::Stop(void)
  {
    std::clog << LOG(DEBUG_LOG) << "StatusMeasurement::Stop BEGIN" << std::endl;
    ismeasure=false;
    if(fThread.joinable()) 
    {
      fThread.join();
    }
    std::clog << LOG(DEBUG_LOG) << "StatusMeasurement::Stop END" << std::endl;
  }


  void StatusMeasurement::Measure(void)
  {
    std::clog << LOG(DEBUG_LOG) << "EnergyCalibration::StatusMeasurement: BEGIN" << std::endl;
    std::vector<  std::vector<double> > Data_from_camac;
    std::vector<  std::vector<double> > Coinc_from_camac;

    auto numbunch = fCamac->GetBunchNumber();
    auto numcounter = fCamac->GetCounterNumber();

    std::vector< data::BunchInfo_t > & bi = fStatus.BunchInfo;
    data::Status * status = &fStatus;
    bi.resize(numbunch, data::BunchInfo_t(numcounter));	

    //status->energy    = 0;
    //status->frequency = 0;
    unsigned updating_time = 1; //sec

    using namespace std::chrono;
    auto vvi_status_update_time = minutes(2);
    auto vvi_status_time = steady_clock::now()-vvi_status_update_time*2;
    std::vector<int> vvi_status;
    //fStatus.VviCounterStatus.resize(numcounter);
    //auto vvi_status = fCamac->GetVviStatus();
    //for(int i=0;i<v.size();i++) fStatus.VviCounterStatus[i] = v[i];
    while( ismeasure )  
    {
      std::clog << LOG(DEBUG_LOG+1) << "EnergyCalibration::StatusMeasurement heartbit" << std::endl;

      // get status of the vvi every 1 minute
      if( steady_clock::now()- vvi_status_time > vvi_status_update_time)
      {
        vvi_status = fCamac->GetVviStatus();
        //for(int i=0;i<v.size();i++) fStatus.VviCounterStatus[i] = v[i];
        vvi_status_time = steady_clock::now();
      } 
      

      std::vector <double> sum(fCamac->GetBunchNumber());
      std::vector <double> sum_cncr(fCamac->GetBunchNumber());
      usleep(useconds_t(updating_time*1e6));
      status->time=fCountMeasurement->GetCountTime();
      auto remain_time = std::chrono::duration_cast< std::chrono::duration<long> > (fCountMeasurement->GetEndTime()-std::chrono::system_clock::now());
      //auto remain_time = std::chrono::duration_cast< std::chrono::duration<int, std::ratio<1, 10>>> (fCountMeasurement->GetEndTime()-std::chrono::system_clock::now());
      status->remain_time = remain_time.count();
      status->N=fEnergyCalibrationStorage->N;

      double fset = fCamac->depolarizer->GetFrequency();
      double fmeas = fCamac->frequencymeter->GetLastMeasuredFrequency();
      auto & dep_cfg = fEnergyCalibrationStorage->cfg.depolarizer;
      dep_cfg.current = dep_cfg.energy(fset);
      dep_cfg.measured = dep_cfg.energy(fmeas);
      status->frequency = fset;
      status->measured_frequency = fmeas;
      status->energy = dep_cfg.current;
      Data_from_camac = fCamac->GetDataWithout_reset();
      Coinc_from_camac = fCamac->GetCoincWithout_reset();
      for ( unsigned bn=0; bn < numbunch; bn++)  
      {
        bi[bn].I = fCurrentMeasurement->GetInstantCurrent(bn);
        for(unsigned ncounter=0 ; ncounter<numcounter ; ncounter++)
        {
          bi[bn].counter[ncounter] = Data_from_camac[bn][ncounter];
          sum[bn]+=bi[bn].counter[ncounter];
          bi[bn].VviCounterStatus[ncounter] = vvi_status[ncounter];
        }
        for(unsigned p=0;p<numcounter/2;p++)
        {
          bi[bn].cncr[p] = Coinc_from_camac[bn][p];
          sum_cncr[bn]+=bi[bn].cncr[p];
        }
        bi[bn].counter_sum=sum[bn];
        bi[bn].cncr_sum=sum_cncr[bn];
      }
      for( unsigned b=0; b< numbunch; b++)
      {
        bi[b].delta=data::GetDelta(sum[b], sum[fEnergyCalibrationStorage->cfg.bunch_norm[b]-1]).value;
        bi[b].delta_cncr=data::GetDelta(sum_cncr[b], sum_cncr[fEnergyCalibrationStorage->cfg.bunch_norm[b]-1]).value;
      }
      if(ismeasure) sig_update_status();
      SendDepolarizerStatusToLsrp();
    }
  }

  void StatusMeasurement::SendDepolarizerStatusToLsrp(int sig)
  {
    std::lock_guard<std::mutex> lock(depol_status_mutex);
    auto & dfreq = fCamac->depolarizer->HighResolutionDepFreq;
    struct item_t
    {
      long int t;
      double E;
      double f;
    };
    std::vector<item_t> v;
    v.reserve(1000); 
    auto last_prepared_time=last_time_sent_to_lsrp;
    switch (sig)
    {
      case 0:
        if(!dfreq.empty())
        {
          auto low = dfreq.lower_bound(last_time_sent_to_lsrp);
          auto up =  (dfreq.end()--);
          if(low!=up && low != (--dfreq.end()))
          {
            for(auto p = ++low; p!=up; p++)
            {
              auto t   = p->first.time_since_epoch().count(); //this is the time since epoch (1970) in nanoseconds
              double f = p->second; //this is the depolarizer frequency in Hz
              double E = fEnergyCalibrationStorage->cfg.depolarizer.energy(f); //the corresponding energy
              v.push_back({t,E,f});
              last_prepared_time= p->first;
            }
          }
          else
          {
            if(fCamac->depolarizer->GetFrequency() == 0)
            {
              v.push_back({std::chrono::system_clock::now().time_since_epoch().count(),0,0});
            }
          }
        }
        else
        {
          v.push_back({std::chrono::system_clock::now().time_since_epoch().count(),0,0});
        }
        break;
      case 1:
        v.push_back({std::chrono::system_clock::now().time_since_epoch().count(),0,0});
        break;
    }
    try 
    {
      boost::asio::io_service service;
      boost::asio::ip::tcp::endpoint ep(boost::asio::ip::address::from_string(PCFG.lsrp.ip_address), PCFG.lsrp.port);
      boost::asio::ip::tcp::socket sock(service);
      sock.connect(ep);
      boost::asio::write(sock,boost::asio::buffer(v));
      last_time_sent_to_lsrp = last_prepared_time;
    }
    catch(boost::system::system_error &e)
    {
      std::cout << LOG(2) <<  "Sending high resolution frequency map to rokk1m-lsrp: " << e.what() << std::endl;
    }
  }
}
