#pragma once

#include <thread>
#include <mutex>
#include <boost/signals2.hpp>

#include "Measurement.h"
#include "data/EnergyCalibrationStorage.h"
#include "data/PolarimeterStatus.h"


namespace camac
{
  class EnergyCalibrationCamac;
}

namespace measurement
{
  class CurrentMeasurement;
	class CountMeasurement;
  class StatusMeasurement : public Measurement
  {
    public:
      StatusMeasurement(void);
      ~StatusMeasurement(void);
      void Start(void);
      void Stop(void);
      void Pause(void) {};
      void UnPause(void) {};
      void Continue(void) {};
      bool IsMeasure(void) const {return ismeasure; }
      data::Status * GetStorage(void) { return &fStatus; }

      void SetCurrentMeasurement(CurrentMeasurement *c) { fCurrentMeasurement = c; }
      void SetCamac(camac::EnergyCalibrationCamac * camac) { fCamac = camac; }
      void SetEnergyCalibrationStorage(data::EnergyCalibrationStorage * s) { fEnergyCalibrationStorage = s; }
			void SetCountMeasurement(CountMeasurement * c) {fCountMeasurement =c ;}

      boost::signals2::signal <void(void)> sig_update_status;
      std::chrono::system_clock::time_point last_time_sent_to_lsrp=std::chrono::system_clock::now(); 
      void SendDepolarizerStatusToLsrp(int sig=0);
    private:
      volatile bool ismeasure;
      void Measure(void);
      data::Status fStatus;
      boost::signals2::connection fConnection;
      std::thread fThread;
      CurrentMeasurement * fCurrentMeasurement=nullptr;
      camac::EnergyCalibrationCamac * fCamac=nullptr;
      data::EnergyCalibrationStorage * fEnergyCalibrationStorage=nullptr;
			CountMeasurement * fCountMeasurement;
      std::mutex   depol_status_mutex;
  };
}
