/*
 * =====================================================================================
 *
 *       Filename:  test_stat.cpp
 *
 *    Description:  Test statistics and corrrelated events extraction
 *
 *        Version:  1.0
 *        Created:  07/24/2010 09:56:48 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */

#include <iostream>
#include <TRandom.h>
#include <TH1.h>
#include <TTree.h>
#include <TNtupleD.h>
#include <TFile.h>

#include <TROOT.h>
#include <TApplication.h>
#include <TSystem.h>
extern void InitGui();
VoidFuncPtr_t initfuncs[] = { InitGui, 0 };
TROOT root("polarimeter","polarimeter", initfuncs);


using namespace std;

int main(int argc , char ** argv)
{
  double N0=1e6; //number of tryes
  //double N=1e7; //number of particle in the beem
  unsigned ntryes = 5000;
  TRandom R;
  double p=0.3;
  double q=0.9;
  TNtupleD * t = new TNtupleD("d","d","p12:p1:p2");
  TFile f("tmp.root","RECREATE");
  for(unsigned n = 0; n<ntryes; ++n)
  {
    double N1=0; //counter 1;
    double N2=0; //counter 2
    double N12=0; //coincidence
    for(unsigned i=0;i<N0;++i)
    {
      bool istoushek= R.Rndm() < p; 
      bool isgas1 = R.Rndm() < q;
      bool isgas2 = R.Rndm() < q;
      bool ishit1 = istoushek || isgas1;
      bool ishit2 = istoushek || isgas2;
      if(ishit1) N1++;
      if(ishit2) N2++;
      if(ishit1 && ishit2) N12++;
    }

    //cout << "Hit 1: " << N1 << " prob: " << N1/N0 << endl;
    //cout << "Hit 2: " << N2 << " prob: " << N2/N0 << endl;
    //cout << "Coinc: " << N12 << " prob: " << N12/N0 << endl;
    double p1 = N1/N0;
    double p2 = N2/N0;
    double p12 = N12/N0;
    double cor = (p12 - p1*p2)/(1.+p12-p1-p2);
    double r1 = (p1 - p12)/(1.-p2);
    double r2 = (p2 - p12)/(1.-p1);
    cout << n <<  " " << cor << " random1: " << r1 <<  " random2: " << r2 << endl;
    t->Fill(cor,r1,r2);
  }
  t->Write();
  TApplication theApp("polarimeter", &argc, argv);
  t->Draw("p12");
  theApp.Run(kTRUE);
  return 0;
}


