/*
 * =====================================================================================
 *
 *       Filename:  valer.cpp
 *
 *    Description:  Test for valer class
 *
 *        Version:  1.0
 *        Created:  07/13/2010 12:31:34 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (), I.B.Nikolaev@inp.nsk.su
 *        Company:  Budker Institute of Nuclear Physycs, Novosibirsk, Russia
 *
 * =====================================================================================
 */
#include <iostream>
#include <cstdlib>

#include <assert.h>
#include "../ibn/valer.h"
#include "../data/Data.h"
#include "../Formula.h"

bool check(const data::PhysValue & pv, const ibn::valer<double> & v)
{
  double pres=1e-13;
  return fabs(pv.value-v.value)<pres && fabs(pv.error-v.error)<pres;
}

using namespace std;
 int main(int argc, char ** argv)
 {
   double M=1;
   double ME=0.7;
   math::Random R(200);
   unsigned i=0;
   unsigned ntryes=100000;
   cout << ntryes << " tryes: " <<endl;
   unsigned wrong=0;
   for(i=0;i<ntryes;++i)
   {
     ibn::valer<double> x(M+ME*R(0,1), ME*R(0,1));
     ibn::valer<double> y(M+ME*R(0,1), ME*R(0,1));
     double D = R(0,1000);
     data::PhysValue X(x.value,x.error);
     data::PhysValue Y(y.value,y.error);
     ibn::valer<double> zp=x+y;
     ibn::valer<double> zm=x-y;
     ibn::valer<double> zd=x/y;
     ibn::valer<double> zD=x/D;
     ibn::valer<double> zP=pow(x,y);
     data::PhysValue Zp=X+Y;
     data::PhysValue Zm=X-Y;
     data::PhysValue Zd=X/Y;
     data::PhysValue ZD=X/D;
     data::PhysValue ZP=pow(X,Y);
     if(!(check(Zp,zp) && check(Zm,zm) && check(Zd,zd) && check(ZD,zD) && check(ZP,zP))) 
     {
       cout << "count="<< i << ": +" <<  check(Zp,zp) << " -" << check(Zm,zm) << " /"<<check(Zd,zd)<< endl;
       cout << "x: " << x.value << " " << x.error << endl;
       cout << "y: " << y.value << " " << y.error << endl;
       cout << "X-x: " << X.value - x.value << " " << X.error - x.error << endl;
       cout << "Y-y: " << Y.value - y.value << " " << Y.error - y.error << endl;
       cout << "D=" << D << endl;
       cout << "zd: " << zd.value << " " << zd.error << endl;
       cout << "Zd-zd: " << Zd.value -zd.value << " " << Zd.error - zd.error << endl;
       cout << "zD: " << ZD.value  << " " << ZD.error  << endl;
       cout << "ZD-zD: " << ZD.value -zD.value << " " << ZD.error - zD.error << endl;
       cout << "zP: " << zP.value  << " " << zP.error  << endl;
       cout << "ZP-zP: " << ZP.value -zP.value << " " << ZP.error - zP.error << endl;
       wrong++;
     }
   }
   cout << "Wrong was " << wrong << " times"<<endl;
 }

