#include<iostream>
#include<camac/Timer.h>
#include<camac/Counter.h>
#include<measurement/Concurrence.h>
#include<measurement/Counter.h>
#include<valarray>
#include<vector>
camac::Timer * Timer;

const int MASK_ALL = camac::Concur::CH1 | camac::Concur::CH3 | camac::Concur::CH4;


void  Measure 
	(  
		vector<measurement::Counter *> &C, 
		measurement::CounterConcur_L0306 * Base,
		measurement::CounterConcur_L0306 * Count,
		int cnum
	)
{	
	unsigned Vmin = 1400;
	unsigned Vmax = 2400;
	if(Vmax < Vmin) {
		cerr <<" Wrong range of volts \n";
        exit(1);
	}
	unsigned dV = 50;
    unsigned N = (Vmax-Vmin)/dV;
	unsigned Nmax = 1; //the max number of catched particles
	valarray<double> NBase(N+1);
	valarray<double> NCount(N+1);
	valarray<double> Volts(N+1);
	valarray<double> eff(N+1);
	valarray<double> err(N+1);

	Base->SetMask(MASK_ALL & ~(0x1 << cnum));

	cout << "# volts,V\tN base\tn\teffect\terror" <<endl;
	for(double v = Vmin; v <= Vmax; v+=dV)  {
		unsigned i = (int)((v-Vmin)/dV);
		Base->ResetCount();
		Count->ResetCount();
		C[cnum]->SetVolts(unsigned(v));
		sleep(5);
		while(Base->GetCount() <= Nmax) {
			Timer->Start(10);
			Timer->Wait();
			NBase[i] = Base->GetCount();
			NCount[i] = Count->GetCount();
			/*
            	cout << NBase[i] << "  " << NCount[i] <<  " " << C[cnum]->GetCount() <<endl;
			for( int i=0; i<4;i++) {
				cout << C[i]->GetCount() << "   ";
				
			}
			cout <<endl;
			*/
		}
		NBase[i] = Base->GetCount();
		NCount[i] = Count->GetCount();
		eff[i] = NCount[i]/NBase[i];
       	err[i] = sqrt(eff[i]*(1-eff[i])/NBase[i]);
		cout << v << "  " << NBase[i] << "  " << NCount[i] << "  " << eff[i] << "  " << err[i] << endl;
	}

}

const unsigned nif = 0 ; //The number of camac interface in cumputer
const unsigned ncr = 0;  //The crete number in inteface
const unsigned ncnt = 6; // The counters number in crate
const unsigned ncnt_main = 10; // The counters number in crate
const unsigned ndskr = 5; //Diskriminator position
const unsigned nvvi = 2; // vvi position
const unsigned ncc = 9; // coinsidence block position
   	
measurement::Counter * InitCounter(unsigned ch)	{
	const unsigned Threshold = 50;
	const unsigned Voltage=2000;
	const unsigned MaxVoltage=2300;
	camac::Address counter_adr("counter",nif,ncr,ncnt,ch);
	camac::Address vvi_adr("vvi",nif,ncr,nvvi + 2*unsigned(ch/2),ch%2);
	camac::Address diskr_adr("diskr",nif,ncr,ndskr,ch);
	using measurement::Counter;
	return new Counter(counter_adr,vvi_adr,diskr_adr,Voltage,MaxVoltage,Threshold,0);
}

int 
main(int argc, char ** argv)    {
	try {
		cout<< "#    " << endl;
		//timer initializing
		Timer= new camac::Timer(camac::Address("Timer",nif,ncr,21), 10);
		//counte initializing
		measurement::Counter * C1 = InitCounter(0);
		measurement::Counter * C2 = InitCounter(1);
		measurement::Counter * C3 = InitCounter(2);
		measurement::Counter * C4 = InitCounter(3);
		using camac::Address;
		using camac::Concur;
		using measurement::CounterConcur_L0306;
		CounterConcur_L0306 * Base = new CounterConcur_L0306	
			(
				Address("Base",nif,ncr,ncnt_main,0),
				Address("Base",nif,ncr,ncc,0),
				0
			);
		//All counter are coinsidened
		CounterConcur_L0306 * Count = new measurement::CounterConcur_L0306	
			(
				Address("Count",nif,ncr,ncnt_main,1),
				Address("Count",nif,ncr,ncc,1),
				MASK_ALL 
			);
		
		//turn on the valtage
		C1->On();
		C2->On();
		C3->On();
		C4->On();
		sleep(10);
		vector<measurement::Counter *> CounterList(4);
		CounterList[0] = C1;
		CounterList[1] = C2;
		CounterList[2] = C3;
		CounterList[3] = C4;
		
		Measure(CounterList,Base,Count,0);
	}
	catch(camac::system_error e)	{ cerr << "***** " << e.what() << endl;}
	catch(camac::station_error e)	{ cerr << "***** " << e.what() << endl;} 
	catch(std::logic_error e)	{ cerr << "***** " << e.what() << endl;}
	catch(std::runtime_error e)	{ cerr << "***** " << e.what() << endl;} 
}
