#include <iostream>
#include <fstream>
#include <iomanip>
//#include "../measurement/Frequencymeter.h"
#include "../camac/Freq.h"
#include "../Opt.h"
#include "../Log.h"

#include<unistd.h>
#include<getopt.h>

unsigned Log::debug_level = 0; 
std::string Log::program_name = "rfmeasure";
bool Log::is_time = false;
bool isplan=false;

using namespace std;
void usage(char ** argv, ostream & ofstr)	
{
  ofstr << "Program read data from frequencymeter for VEPP4 RF" << endl;
  ofstr << "Writen by Ivan B. Nikolaev, 39-41-78 21 JAN 2002" << endl;
  ofstr << "Usege: " << argv[0] << " --address low_addr high_addr [option] [file1 ...]" << endl;
  ofstr << " camac address is in format: module_name/interface/crate/station\n";
  ofstr << "Option is:\n";
  ofstr <<	"  --start time : start frequency measurement with time" << endl;
  ofstr << "  --measure n :  measure n times" << endl;
  ofstr << "  --help : print this help\n";
  exit(0);
}

const double UN_DISPL_FREQ = 1e8; //     

//measure frequency of VEPP4 RF system.
double getfrequency(camac::Freq & freq)	
{
  double  rf = freq.GetFrequency();
  double lost=0;
  if(!isplan) {
    if( rf < 1e8) lost = 100e6;
    if( rf < 1e7) lost = 180e6;
    if( rf < 1e6) lost = 181e6;
    if( rf < 1e5) lost = 181.8e6;
  }
  rf += lost;
  return rf;
}

//Write measured frequency to specified file
void write2file(ostream & file,double rf)	
{
}

int main(int argc, char ** argv)    
{
  ibn::Opt opt;
  opt.add("--plan");
  opt.add("--address",2);
  opt.add("--start",1);
  opt.add("--stop", 0);
  opt.add("--measure",1);
  opt.add("--help",0);
  opt.add("--file",1);
  opt.fill(argc,argv);
  unsigned time;
  string fname1,fname2;
  double rf;
  vector<string> vaddr(2);
  if(argc < 3) 
  {

  }
  if(opt.isopt("--plan")) isplan=true;
  try	
  {
    if(!opt.getopt("--address",&vaddr))	
    {
      cerr << "rfmeasure: You must specify address of camac bloch for frequencymeter" << endl;
      usage(argv,cerr);
      exit(1);
    }	
    if( opt.isopt("--help") )	
    {
      usage(argv,cout);
      exit(0);
    }

    camac::Address low(vaddr[0].c_str(),'/');
    camac::Address high(vaddr[1].c_str(),'/');
    //measurement::Frequencymeter *freq = new measurement::Frequencymeter(low,high);
    camac::Freq freq(low,high);

    if(opt.getopt("--start",&time) )	
    {
      cout << "# start frequency measurement " << flush;
      cout << " [OK]" << endl;
      while(true)    
      {
        sleep(time);
        cout << setprecision(10) << getfrequency(freq) << endl;
      }
    }
    bool isfile=false;
    ostream & s = cout;
    string filename;
    if(opt.isopt("--file")) 
    {
      opt.getopt("--file",&filename);
      isfile=true;
    }
    if(opt.getopt("--measure",&time))
    {
      for(unsigned i =0 ; i< time; i++)
      {
        sleep(1);
        rf = getfrequency(freq);
        if(isfile)
        {
          ofstream file(filename.c_str());
          file << setprecision(10) << rf << endl;
          file.close();
        }
        cout << setprecision(10) << rf << endl;
      }
    }
  }
  catch(camac::Crate::Error e)
  {
    cerr << "rfmeasure: camac crate error: " <<e.what() << endl << flush;
    cerr << "check crate power or check ppi-crate cable" << endl;
    return 1;
  }
  catch(camac::Module::Error e)
  {
    cerr << "rfmeasure: camac module error: " <<e.what() << endl;
    return 1;
  }
  catch(std::runtime_error e)
  {
    cerr << "rfmeasure: runtime error: " << e.what() << endl;
    return 1;
  }
  catch(std::out_of_range e)
  {
    cerr << "rfmeasure: data is out of range: " << e.what() << endl;
    return 1;
  }
  catch(std::logic_error e)
  {
    cerr << "rfmeasure: logic error:  " <<e.what() << endl;
    return 1;
  }
  catch(...)
  {
    cerr << "rfmeasure: some unrecognize exeption " << endl;
    return 1;
  }
  return 0;
}

