#include <iostream>
#include <fstream>

#include "Log.h"
#include "Status.h"
#include "Stap.h"

namespace vepp4
{
	Status::Status(void)
	{
		Update();
	}

	void Status::Read(const std::string & fname)
	{
		using namespace std;
		vepp4::Stap stap;
		std::ifstream status_file(fname.c_str());
		if(status_file)
		{
			read(status_file);
		}
		else {
			std::cerr<<LOG(WARNING_LOG) << "WARING: unable to open file for reading\n";
		}
		status_file.close();
		time_t t = time(0);
		switch(stap.status)  {
			case 1: injection_time=t;     break;
			case 2: acceleration_time=t;  break;
			case 3: experiment_time=t;    break;
			case 5: cycle_time=t;         break;
			case 6: measurement_time=t;   break;
			default:  break;
		}
		status=stap.status; status_time=t;
		if(stap.Eset!=set_energy)
		{
			set_energy=stap.Eset;
			energy_time=t;
		}
	}

	void Status::Write(const std::string &fname)
	{
		using namespace std;
		ofstream status_file(fname.c_str());
		write(status_file);
		status_file.close();
	}
	void Status::Update(void)
	{
		Read();
		Write();
	}

	void Status::read(std::istream & is)
	{
		using namespace std;
		int BUFSIZE=2024;
		is >> status_time;        is.ignore(BUFSIZE,'\n');
		is >> status;            is.ignore(BUFSIZE,'\n');
		is >> set_energy;            is.ignore(BUFSIZE,'\n');
		is >> energy_time;            is.ignore(BUFSIZE,'\n');
		is >> injection_time;     is.ignore(BUFSIZE,'\n');
		is >> acceleration_time;  is.ignore(BUFSIZE,'\n');
		is >> experiment_time;    is.ignore(BUFSIZE,'\n');
		is >> cycle_time;         is.ignore(BUFSIZE,'\n');
		is >> measurement_time;   is.ignore(BUFSIZE,'\n');
	}

	void Status::write(std::ostream &os) const
	{
		os << status_time <<"\t#\t"<<ctime(&status_time);
		os << status<< "\t# status\n";
		os << set_energy<<"\t#\tsetup_energy, MeV\n";
		os << energy_time <<"\t#\tset_energy_time :"<<ctime(&energy_time);
		os << injection_time << "\t#\tinjection_time :" << ctime(&injection_time);
		os << acceleration_time << "\t#\tacceleration_time :" << ctime(&acceleration_time);
		os << experiment_time << "\t#\texperiment_time :" << ctime(&experiment_time);
		os << cycle_time << "\t#\tcycle_time :" << ctime(&cycle_time);
		os << measurement_time << "\t#\tmeasurement_time :" << ctime(&measurement_time);
	}
}
